@@ -22,6 +22,8 @@
  *  <philippe@algernon.localdomain>
  ****************************************************************************/
 
+#include <string.h>
+
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
@@ -46,47 +48,50 @@ brasero_burn_quark (void)
 	return quark;
 }
  
-const char *
+const gchar *
 brasero_burn_action_to_string (BraseroBurnAction action)
 {
-	char *strings [BRASERO_BURN_ACTION_LAST] = { 	"",
+	gchar *strings [BRASERO_BURN_ACTION_LAST] = { 	"",
 							N_("Getting size"),
+							N_("Creating checksum"),
 							N_("Creating image"),
 							N_("Copying disc"),
 							N_("Copying file"),
 							N_("Analysing audio information"),
 							N_("Transcoding song"),
 							N_("Preparing to write"),
+							N_("Writing leadin"),
 							N_("Writing"),
 							N_("Writing CD-TEXT information"),
 							N_("Fixating"),
+							N_("Writing leadout"),
 							N_("Blanking"),
 							N_("Success") };
 	return _(strings [action]);
 }
 
 gchar *
-brasero_track_source_get_iso_localpath (BraseroTrackSource *track)
+brasero_track_source_get_image_localpath (BraseroTrackSource *track)
 {
 	gchar *localpath;
-	gchar *escaped_uri;
+	gchar *uri;
 
-	if (track->type != BRASERO_TRACK_SOURCE_ISO
-	&&  track->type != BRASERO_TRACK_SOURCE_ISO_JOLIET)
+	if (track->type != BRASERO_TRACK_SOURCE_IMAGE)
 		return NULL;
 
-	if (!track->contents.iso.image)
+	if (!(track->format & BRASERO_IMAGE_FORMAT_ISO)
+	&&    track->format != BRASERO_IMAGE_FORMAT_NONE)
 		return NULL;
 
-	if (track->contents.iso.image [0] == '/')
-		return g_strdup (track->contents.iso.image);
-
-	if (!g_str_has_prefix (track->contents.iso.image, "file://"))
+	if (!track->contents.image.image)
 		return NULL;
 
-	escaped_uri = gnome_vfs_escape_host_and_path_string (track->contents.iso.image);
-	localpath = gnome_vfs_get_local_path_from_uri (escaped_uri);
-	g_free (escaped_uri);
+	if (track->contents.image.image [0] == '/')
+		return g_strdup (track->contents.image.image);
+
+	uri = gnome_vfs_make_uri_from_input (track->contents.image.image);
+	localpath = gnome_vfs_get_local_path_from_uri (uri);
+	g_free (uri);
 
 	return localpath;
 }
@@ -95,24 +100,22 @@ gchar *
 brasero_track_source_get_raw_localpath (BraseroTrackSource *track)
 {
 	gchar *localpath;
-	gchar *escaped_uri;
+	gchar *uri;
 
 	/* NOTE: here cdrecord doesn't need *.toc image but the raw part */
-	if (track->type != BRASERO_TRACK_SOURCE_RAW)
+	if (track->type != BRASERO_TRACK_SOURCE_IMAGE
+	|| !(track->format & BRASERO_IMAGE_FORMAT_CLONE))
 		return NULL;
 
-	if (!track->contents.raw.image)
+	if (!track->contents.image.image)
 		return NULL;
 
-	if (track->contents.raw.image [0] == '/')
-		return g_strdup (track->contents.raw.image);
+	if (track->contents.image.image [0] == '/')
+		return g_strdup (track->contents.image.image);
 
-	if (!g_str_has_prefix (track->contents.raw.image, "file://"))
-		return NULL;
-
-	escaped_uri = gnome_vfs_escape_host_and_path_string (track->contents.raw.image);
-	localpath = gnome_vfs_get_local_path_from_uri (escaped_uri);
-	g_free (escaped_uri);
+	uri = gnome_vfs_make_uri_from_input (track->contents.image.image);
+	localpath = gnome_vfs_get_local_path_from_uri (uri);
+	g_free (uri);
 
 	return localpath;
 }
@@ -121,23 +124,44 @@ gchar *
 brasero_track_source_get_cue_localpath (BraseroTrackSource *track)
 {
 	gchar *localpath;
-	gchar *escaped_uri;
+	gchar *uri;
 
-	if (track->type != BRASERO_TRACK_SOURCE_CUE)
+	if (track->type != BRASERO_TRACK_SOURCE_IMAGE
+	|| !(track->format & BRASERO_IMAGE_FORMAT_CUE))
 		return NULL;
 
-	if (!track->contents.cue.toc)
+	if (!track->contents.image.toc)
 		return NULL;
 
-	if (track->contents.cue.toc [0] == '/')
-		return g_strdup (track->contents.cue.toc);
+	if (track->contents.image.toc [0] == '/')
+		return g_strdup (track->contents.image.toc);
 
-	if (!g_str_has_prefix (track->contents.cue.toc, "file://"))
+    	uri = gnome_vfs_make_uri_from_input (track->contents.image.toc);
+	localpath = gnome_vfs_get_local_path_from_uri (uri);
+	g_free (uri);
+
+	return localpath;
+}
+
+gchar *
+brasero_track_source_get_cdrdao_localpath (BraseroTrackSource *track)
+{
+	gchar *localpath;
+	gchar *uri;
+
+	if (track->type != BRASERO_TRACK_SOURCE_IMAGE
+	|| !(track->format & BRASERO_IMAGE_FORMAT_CDRDAO))
 		return NULL;
 
-	escaped_uri = gnome_vfs_escape_host_and_path_string (track->contents.cue.toc);
-	localpath = gnome_vfs_get_local_path_from_uri (escaped_uri);
-	g_free (escaped_uri);
+	if (!track->contents.image.toc)
+		return NULL;
+
+	if (track->contents.image.toc [0] == '/')
+		return g_strdup (track->contents.image.toc);
+
+    	uri = gnome_vfs_make_uri_from_input (track->contents.image.toc);
+	localpath = gnome_vfs_get_local_path_from_uri (uri);
+	g_free (uri);
 
 	return localpath;
 }
@@ -210,33 +234,20 @@ brasero_track_source_free (BraseroTrackSource *track)
 	}
 	else if (track->type == BRASERO_TRACK_SOURCE_DISC)
 		nautilus_burn_drive_unref (track->contents.drive.disc);
-	else if (track->type == BRASERO_TRACK_SOURCE_INF) {
-		GSList *iter;
+	else if (track->type == BRASERO_TRACK_SOURCE_INF
+	     ||  track->type == BRASERO_TRACK_SOURCE_AUDIO) {
+		if (track->contents.audio.album)
+			g_free (track->contents.audio.album);
 
-		for (iter = track->contents.inf.files; iter; iter = iter->next)
-			g_free (iter->data);
+		g_slist_foreach (track->contents.audio.infos,
+				 (GFunc) brasero_song_info_free,
+				 NULL);
 
-		g_slist_free (track->contents.inf.files);
+		g_slist_free (track->contents.audio.infos);
 	}
-	else if (track->type == BRASERO_TRACK_SOURCE_AUDIO) {
-		GSList *iter;
-
-		for (iter = track->contents.audio.files; iter; iter = iter->next)
-			g_free (iter->data);
-
-		g_slist_free (track->contents.audio.files);
-	}
-	else if (track->type == BRASERO_TRACK_SOURCE_CUE) {
-		g_free (track->contents.cue.image);
-		g_free (track->contents.cue.toc);
-	}
-	else if (track->type == BRASERO_TRACK_SOURCE_RAW) {
-		g_free (track->contents.raw.image);
-		g_free (track->contents.raw.toc);
-	}
-	else if (track->type == BRASERO_TRACK_SOURCE_ISO
-	      ||  track->type == BRASERO_TRACK_SOURCE_ISO_JOLIET) {
-		g_free (track->contents.iso.image);
+	else if (track->type == BRASERO_TRACK_SOURCE_IMAGE) {
+		g_free (track->contents.image.image);
+		g_free (track->contents.image.toc);
 	}
 	else if (track->type == BRASERO_TRACK_SOURCE_IMAGER)
 		g_object_unref (track->contents.imager.obj);
@@ -253,6 +264,7 @@ brasero_track_source_copy (const BraseroTrackSource *track)
 
 	copy = g_new0 (BraseroTrackSource, 1);
 	copy->type = track->type;
+	copy->format = track->format;
 
 	if (track->type == BRASERO_TRACK_SOURCE_SONG) {
 		GSList *iter;
@@ -306,13 +318,11 @@ brasero_track_source_copy (const BraseroTrackSource *track)
 
 		/* copy the excluded files */
 		for (iter = track->contents.data.excluded; iter; iter = iter->next) {
-			char *uri;
+			gchar *uri;
 
 			uri = iter->data;
 			copy->contents.data.excluded = g_slist_append (copy->contents.data.excluded, g_strdup (uri));
 		}
-
-		copy->contents.data.use_joliet = track->contents.data.use_joliet;
 	}
 	else if (track->type == BRASERO_TRACK_SOURCE_GRAFTS) {
 		if (track->contents.grafts.grafts_path)
@@ -323,48 +333,36 @@ brasero_track_source_copy (const BraseroTrackSource *track)
 
 		if (track->contents.grafts.label)
 			copy->contents.grafts.label = g_strdup (track->contents.grafts.label);
-
-		copy->contents.grafts.use_joliet = track->contents.grafts.use_joliet;
 	}
 	else if (track->type == BRASERO_TRACK_SOURCE_DISC) {
 		copy->contents.drive.disc = track->contents.drive.disc;
 		nautilus_burn_drive_ref (track->contents.drive.disc);
 	}
-	else if (track->type == BRASERO_TRACK_SOURCE_INF) {
+	else if (track->type == BRASERO_TRACK_SOURCE_INF
+	     ||  track->type == BRASERO_TRACK_SOURCE_AUDIO) {
 		GSList *iter;
 
-		for (iter = track->contents.inf.files; iter; iter = iter->next) {
-			char *file;
+		if (track->contents.audio.album)
+			copy->contents.audio.album = g_strdup (track->contents.audio.album);
 
-			file = g_strdup (iter->data);
-			copy->contents.inf.files = g_slist_append (copy->contents.inf.files, file);
+		for (iter = track->contents.audio.infos; iter; iter = iter->next) {
+			BraseroSongInfo *info;
+
+			info = iter->data;
+			copy->contents.audio.infos = g_slist_append (copy->contents.audio.infos,
+								     brasero_song_info_copy (info));
 		}
 	}
-	else if (track->type == BRASERO_TRACK_SOURCE_AUDIO) {
-		GSList *iter;
-
-		for (iter = track->contents.audio.files; iter; iter = iter->next) {
-			char *file;
-
-			file = g_strdup (iter->data);
-			copy->contents.audio.files = g_slist_append (copy->contents.audio.files, file);
-		}
+	else if (track->type == BRASERO_TRACK_SOURCE_IMAGE) {
+		copy->contents.image.image = g_strdup (track->contents.image.image);
+		copy->contents.image.toc = g_strdup (track->contents.image.toc);
 	}
-	else if (track->type == BRASERO_TRACK_SOURCE_CUE) {
-		copy->contents.cue.image = g_strdup (track->contents.cue.image);
-		copy->contents.cue.toc = g_strdup (track->contents.cue.toc);
-	}
-	else if (track->type == BRASERO_TRACK_SOURCE_RAW) {
-		copy->contents.raw.image = g_strdup (track->contents.raw.image);
-		copy->contents.raw.toc = g_strdup (track->contents.raw.toc);
-	}
-	else if (track->type == BRASERO_TRACK_SOURCE_ISO
-	      ||  track->type == BRASERO_TRACK_SOURCE_ISO_JOLIET)
-		copy->contents.iso.image = g_strdup (track->contents.iso.image);
 	else if (track->type == BRASERO_TRACK_SOURCE_IMAGER) {
 		copy->contents.imager.obj = track->contents.imager.obj;
 		g_object_ref (copy->contents.imager.obj);
 	}
+	else if (track->type == BRASERO_TRACK_SOURCE_SUM)
+		memcpy (copy, track, sizeof (BraseroTrackSource));
 	else {
 		g_free (copy);
 		copy = NULL;
@@ -409,3 +407,35 @@ brasero_graft_point_copy (BraseroGraftPt *graft) {
 	newgraft->excluded = g_slist_reverse (newgraft->excluded);
 	return newgraft;
 }
+
+void
+brasero_song_info_free (BraseroSongInfo *info)
+{
+	if (info->path)
+		g_free (info->path);
+
+	g_free (info->title);
+	g_free (info->artist);
+	g_free (info->composer);
+	g_free (info);
+}
+
+BraseroSongInfo *
+brasero_song_info_copy (BraseroSongInfo *info)
+{
+	BraseroSongInfo *copy;
+
+	copy = g_new0 (BraseroSongInfo, 1);
+
+	if (info->path)
+		copy->path = g_strdup (info->path);
+
+	copy->title = g_strdup (info->title);
+	copy->artist = g_strdup (info->artist);
+	copy->composer = g_strdup (info->composer);
+	copy->isrc = info->isrc;
+	copy->duration = info->duration;
+	copy->sectors = info->sectors;
+
+	return copy;
+}
