@@ -33,12 +33,41 @@ extern "C"
 
 #include <nautilus-burn-drive.h>
 
+#include <libgnomevfs/gnome-vfs.h>
+
 #define CDR_SPEED 153600
 #define DVD_SPEED 1385000
 
+#define BRASERO_GET_BASENAME_FOR_DISPLAY(uri, name)				\
+{										\
+    	gchar *escaped_basename;						\
+	escaped_basename = g_path_get_basename (uri);				\
+    	name = gnome_vfs_unescape_string_for_display (escaped_basename);	\
+	g_free (escaped_basename);						\
+}
+
 GQuark brasero_burn_quark (void);
 #define BRASERO_BURN_ERROR brasero_burn_quark()
 
+#define BRASERO_BURN_LOG_DOMAIN				"BraseroBurn"
+#define BRASERO_BURN_LOG(format, ...)				\
+		g_log (BRASERO_BURN_LOG_DOMAIN,			\
+		       G_LOG_LEVEL_DEBUG,			\
+		       format,					\
+		       ##__VA_ARGS__);
+#define BRASERO_BURN_LOGV(format)				\
+	{							\
+		va_list arg_list;				\
+		va_start (arg_list, format);			\
+		g_logv (BRASERO_BURN_LOG_DOMAIN,		\
+			G_LOG_LEVEL_DEBUG,			\
+			format,					\
+			arg_list);				\
+		va_end (arg_list);				\
+	}
+	
+#define BRASERO_BURN_TMP_FILE_NAME		"brasero_tmp_XXXXXX"
+
 typedef enum {
 	BRASERO_BURN_ERROR_NONE,
 	BRASERO_BURN_ERROR_GENERAL,
@@ -83,16 +112,19 @@ typedef enum {
 typedef enum {
 	BRASERO_BURN_ACTION_NONE		= 0,
 	BRASERO_BURN_ACTION_GETTING_SIZE,
+	BRASERO_BURN_ACTION_CHECKSUM,
 	BRASERO_BURN_ACTION_CREATING_IMAGE,
 	BRASERO_BURN_ACTION_DRIVE_COPY,
 	BRASERO_BURN_ACTION_FILE_COPY,
 	BRASERO_BURN_ACTION_ANALYSING,
 	BRASERO_BURN_ACTION_TRANSCODING,
 	BRASERO_BURN_ACTION_PREPARING,
+	BRASERO_BURN_ACTION_LEADIN,
 	BRASERO_BURN_ACTION_WRITING,
 	BRASERO_BURN_ACTION_WRITING_CD_TEXT,
 	BRASERO_BURN_ACTION_FIXATING,
-	BRASERO_BURN_ACTION_BLANKING,
+	BRASERO_BURN_ACTION_LEADOUT,
+	BRASERO_BURN_ACTION_ERASING,
 	BRASERO_BURN_ACTION_FINISHED,
 	BRASERO_BURN_ACTION_LAST
 } BraseroBurnAction;
@@ -113,87 +145,111 @@ brasero_burn_action_to_string (BraseroBurnAction action);
 typedef enum {
 	BRASERO_TRACK_SOURCE_DEFAULT,
 	BRASERO_TRACK_SOURCE_UNKNOWN,
+	BRASERO_TRACK_SOURCE_SUM,
 	BRASERO_TRACK_SOURCE_SONG,		/* imager */
 	BRASERO_TRACK_SOURCE_INF,		/* recorder */
 	BRASERO_TRACK_SOURCE_AUDIO,		/* recorder */
 	BRASERO_TRACK_SOURCE_DATA,		/* imager */
+	BRASERO_TRACK_SOURCE_GRAFTS,		/* used internally mostly (must be local files) */
+	BRASERO_TRACK_SOURCE_IMAGE,
 	BRASERO_TRACK_SOURCE_DISC,		/* imager */
-	BRASERO_TRACK_SOURCE_CUE,		/* recorder */
-	BRASERO_TRACK_SOURCE_ISO,		/* recorder */
-	BRASERO_TRACK_SOURCE_RAW,		/* recorder */
 	BRASERO_TRACK_SOURCE_IMAGER,		/* used internally for on the fly burning */
-	BRASERO_TRACK_SOURCE_GRAFTS,		/* used internally mostly + must be local files */
-	BRASERO_TRACK_SOURCE_ISO_JOLIET,	/* used internally mostly */
 } BraseroTrackSourceType;
 
+typedef enum {
+	BRASERO_IMAGE_FORMAT_NONE		= 0,
+	BRASERO_IMAGE_FORMAT_JOLIET 		= 1,
+	BRASERO_IMAGE_FORMAT_ISO		= 1 << 1,
+	BRASERO_IMAGE_FORMAT_CUE		= 1 << 2,
+	BRASERO_IMAGE_FORMAT_CLONE		= 1 << 3,
+	BRASERO_IMAGE_FORMAT_CDRDAO		= 1 << 4,
+	BRASERO_IMAGE_FORMAT_VIDEO		= 1 << 5,
+	BRASERO_IMAGE_FORMAT_ANY		= BRASERO_IMAGE_FORMAT_JOLIET|
+						  BRASERO_IMAGE_FORMAT_ISO|
+						  BRASERO_IMAGE_FORMAT_CUE|
+						  BRASERO_IMAGE_FORMAT_CDRDAO|
+						  BRASERO_IMAGE_FORMAT_CLONE|
+						  BRASERO_IMAGE_FORMAT_VIDEO,
+} BraseroImageFormat;
+
 typedef struct _BraseroImager BraseroImager;
 
+struct _BraseroMD5 {
+	guint32 A;
+	guint32 B;
+	guint32 C;
+	guint32 D;
+};
+typedef struct _BraseroMD5 BraseroMD5;
+
 struct _BraseroTrackSource {
 	BraseroTrackSourceType type;
+	BraseroImageFormat format;
 
 	union {
 		struct {
 			gchar *label;
-			GSList *grafts;		/* BraseroGraftPt *graft */
+			GSList *grafts;			/* BraseroGraftPt *graft */
 			GSList *excluded;		/* list of uris (char*) that are to be always excluded */
-
-			gint use_joliet:1;
 		} data;
 		struct {
 			gchar *label;
 			gchar *grafts_path;
 			gchar *excluded_path;
-
-			gint use_joliet:1;
 		} grafts;
 		struct {
 			gchar *album;
-			GSList *files;			/* BraseroSongFile *file */
+			GSList *files;			/* BraseroSongFile * */
 		} songs;
+		/* NOTE: _INF and _AUDIO share the same structure but the 
+		 * difference is that with _INF the song are not transcoded.
+		 * Files must be local. */
 		struct {
-			GSList *files;
+			gchar *album;
+			GSList *infos;			/* BraseroSongInfo * */
 		} audio;
-		struct {
-			GSList *files;			/* char *path_to_inf */
-		} inf;
 		struct {
 			NautilusBurnDrive *disc;
-			BraseroTrackSourceType target;
 		} drive;
 		struct {
 			gchar *image;
 			gchar *toc;
-		} cue;
-		struct {
-			gchar *image;
-		} iso;
-		struct {
-			gchar *image;
-			gchar *toc;
-		} raw;
+		} image;
 		struct {
 			BraseroImager *obj;
 		} imager;
+		struct {
+			BraseroMD5 md5;
+		} sum;
 			
 	} contents;
 };
 typedef struct _BraseroTrackSource BraseroTrackSource;
 
 gchar *
-brasero_track_source_get_iso_localpath (BraseroTrackSource *track);
+brasero_track_source_get_image_localpath (BraseroTrackSource *track);
 gchar *
 brasero_track_source_get_raw_localpath (BraseroTrackSource *track);
 gchar *
 brasero_track_source_get_cue_localpath (BraseroTrackSource *track);
+gchar *
+brasero_track_source_get_cdrdao_localpath (BraseroTrackSource *track);
 
 void
 brasero_track_source_free (BraseroTrackSource *source);
 BraseroTrackSource *
 brasero_track_source_copy (const BraseroTrackSource *source);
 
+#define BRASERO_TRACK_SOURCE_ALLOW_DVD(source)	\
+	(source->type != BRASERO_TRACK_SOURCE_AUDIO && \
+	 source->type != BRASERO_TRACK_SOURCE_SONG && \
+	 source->type != BRASERO_TRACK_SOURCE_INF && \
+	(source->format == BRASERO_IMAGE_FORMAT_NONE ||	\
+	(source->format & BRASERO_IMAGE_FORMAT_ISO)))
+
 struct _BraseroGraftPt {
-	char *uri;
-	char *path;
+	gchar *uri;
+	gchar *path;
 	GSList *excluded; /* list of uris (char *) that are to be excluded only for this path */
 };
 typedef struct _BraseroGraftPt BraseroGraftPt;
@@ -204,16 +260,35 @@ BraseroGraftPt *
 brasero_graft_point_copy (BraseroGraftPt *graft);
 
 struct _BraseroSongFile {
-	char *title;
-	char *artist;
-	char *composer;
-	int isrc;
-	char *uri;
+	gchar *title;
+	gchar *artist;
+	gchar *composer;
+	gint isrc;
+	gchar *uri;
 
-	gboolean gap;
+	gint64 gap;
 };
 typedef struct _BraseroSongFile BraseroSongFile;
 
+struct _BraseroSongInfo {
+	gchar *path;
+	gchar *title;
+	gchar *artist;
+	gchar *composer;
+	gint isrc;
+
+	gint64 duration;
+	gint sectors;
+};
+
+typedef struct _BraseroSongInfo BraseroSongInfo;
+
+void
+brasero_song_info_free (BraseroSongInfo *info);
+
+BraseroSongInfo *
+brasero_song_info_copy (BraseroSongInfo *info);
+
 #ifdef __cplusplus
 }
 #endif
