@@ -37,6 +37,8 @@
 #include <glib/gi18n-lib.h>
 #include <glib/gstdio.h>
 
+#include <libgnomevfs/gnome-vfs-utils.h>
+
 #include <nautilus-burn-drive.h>
 
 #include "burn-cdrdao.h"
@@ -45,6 +47,8 @@
 #include "burn-process.h"
 #include "burn-recorder.h"
 #include "burn-imager.h"
+#include "burn-toc2cue.h"
+#include "burn-caps.h"
 #include "brasero-ncb.h"
 
 static void brasero_cdrdao_class_init (BraseroCdrdaoClass *klass);
@@ -64,16 +68,6 @@ brasero_cdrdao_set_argv (BraseroProcess *process,
 static BraseroBurnResult
 brasero_cdrdao_post (BraseroProcess *process,
 		     BraseroBurnResult retval);
-
-static BraseroBurnResult
-brasero_cdrdao_get_rate (BraseroJob *job,
-			 gint64 *speed);
-static BraseroBurnResult
-brasero_cdrdao_get_written (BraseroJob *job,
-			    gint64 *written);
-static BraseroBurnResult
-brasero_cdrdao_get_fifo (BraseroRecorder *recorder,
-			 gint *fifo);
 				  
 static BraseroBurnResult
 brasero_cdrdao_set_drive (BraseroRecorder *recorder,
@@ -86,7 +80,7 @@ brasero_cdrdao_set_flags (BraseroRecorder *recorder,
 			  GError **error);
 static BraseroBurnResult
 brasero_cdrdao_set_rate (BraseroJob *job,
-			  gint64 speed);
+			  gint64 rate);
 
 static BraseroBurnResult
 brasero_cdrdao_record (BraseroRecorder *recorder,
@@ -97,7 +91,8 @@ brasero_cdrdao_blank (BraseroRecorder *recorder,
 
 static BraseroBurnResult
 brasero_cdrdao_get_track_type (BraseroImager *imager,
-			       BraseroTrackSourceType *type);
+			       BraseroTrackSourceType *type,
+			       BraseroImageFormat *format);
 
 static BraseroBurnResult
 brasero_cdrdao_get_size_image (BraseroImager *imager,
@@ -122,13 +117,9 @@ brasero_cdrdao_set_output (BraseroImager *imager,
 static BraseroBurnResult
 brasero_cdrdao_set_output_type (BraseroImager *imager,
 				BraseroTrackSourceType type,
+				BraseroImageFormat format,
 				GError **error);
 
-static BraseroBurnResult
-brasero_cdrdao_get_action_string (BraseroJob *job, 
-				  BraseroBurnAction action,
-				  char **string);
-
 typedef enum {
 	BRASERO_CDRDAO_ACTION_NONE,
 	BRASERO_CDRDAO_ACTION_IMAGE,
@@ -139,27 +130,20 @@ typedef enum {
 
 struct BraseroCdrdaoPrivate {
 	BraseroCdrdaoAction action;
-
-	GTimer *timer;
-	GSList *rates;
-	guint64 start;
-
 	BraseroTrackSource *track;
 	NautilusBurnDrive *drive;
 
-	char *src_output;
-	char *toc;
+	BraseroImageFormat format;
 
-	int fifo;
-	int speed;               /* speed at which we should write */
-	gint64 current_rate;           /* speed at which we're actually writing/imaging */
+	gchar *src_output;
 
-	int track_num;
-	int track_start;
+	gchar *output;
+	gchar *datafile;
+
+	gint rate;               /* speed at which we should write */
 
-	int sectors_num;
 	gint64 isosize;
-	gint64 b_written;
+	gint sectors_num;
 
 	int dao:1;
 	int dummy:1;
@@ -229,7 +213,6 @@ brasero_cdrdao_iface_init_record (BraseroRecorderIFace *iface)
 {
 	iface->set_drive = brasero_cdrdao_set_drive;
 	iface->set_flags = brasero_cdrdao_set_flags;
-	iface->get_fifo = brasero_cdrdao_get_fifo;
 	iface->record = brasero_cdrdao_record;
 	iface->blank = brasero_cdrdao_blank;
 }
@@ -256,10 +239,7 @@ brasero_cdrdao_class_init (BraseroCdrdaoClass *klass)
 	object_class->finalize = brasero_cdrdao_finalize;
 
 	job_class->set_rate = brasero_cdrdao_set_rate;
-	job_class->get_rate = brasero_cdrdao_get_rate;
 	job_class->set_source = brasero_cdrdao_set_source;
-	job_class->get_written = brasero_cdrdao_get_written;
-	job_class->get_action_string = brasero_cdrdao_get_action_string;
 
 	process_class->stderr_func = brasero_cdrdao_read_stderr;
 	process_class->set_argv = brasero_cdrdao_set_argv;
@@ -272,51 +252,67 @@ brasero_cdrdao_init (BraseroCdrdao *obj)
 	obj->priv = g_new0(BraseroCdrdaoPrivate, 1);
 	obj->priv->isosize = -1;
 	obj->priv->clean = TRUE;
-	obj->priv->current_rate = -1;
+}
+
+static void
+brasero_cdrdao_clean_output (BraseroCdrdao *cdrdao)
+{
+	if (cdrdao->priv->output) {
+		BraseroImageFormat format = BRASERO_IMAGE_FORMAT_NONE;
+
+		if (cdrdao->priv->format == BRASERO_IMAGE_FORMAT_ANY) {
+			BraseroBurnCaps *caps;
+
+			caps = brasero_burn_caps_get_default ();
+			format = brasero_burn_caps_get_imager_default_format (caps,
+									      cdrdao->priv->track);
+			g_object_unref (caps);
+		}
+		else
+			format = cdrdao->priv->format;
+
+		if (cdrdao->priv->clean
+		|| !(format & BRASERO_IMAGE_FORMAT_CDRDAO))
+			g_remove (cdrdao->priv->output);
+
+		g_free (cdrdao->priv->output);
+		cdrdao->priv->output = NULL;
+	}
+
+	if (cdrdao->priv->datafile) {
+		if (cdrdao->priv->clean && cdrdao->priv->track_ready)
+			g_remove (cdrdao->priv->datafile);
+
+		g_free (cdrdao->priv->datafile);
+		cdrdao->priv->datafile = NULL;
+	}
+
+	cdrdao->priv->track_ready = 0;
 }
 
 static void
 brasero_cdrdao_finalize (GObject *object)
 {
 	BraseroCdrdao *cobj;
-	cobj = BRASERO_CDRDAO(object);
+	cobj = BRASERO_CDRDAO (object);
 
-	if (cobj->priv->timer) {
-		g_timer_destroy (cobj->priv->timer);
-		cobj->priv->timer = NULL;
+	if (cobj->priv->src_output) {
+		g_free (cobj->priv->src_output);
+		cobj->priv->src_output = NULL;
 	}
-	
+
+	brasero_cdrdao_clean_output (cobj);
+
 	if (cobj->priv->drive) {
 		nautilus_burn_drive_unref (cobj->priv->drive);
 		cobj->priv->drive = NULL;
 	}
 	
-	if (cobj->priv->rates) {
-		g_slist_free (cobj->priv->rates);
-		cobj->priv->rates = NULL;
-	}
-
 	if (cobj->priv->track) {
 		brasero_track_source_free (cobj->priv->track);
 		cobj->priv->track = NULL;
 	}
 
-	if (cobj->priv->src_output) {
-		if (cobj->priv->clean && cobj->priv->track_ready)
-			g_remove (cobj->priv->src_output);
-
-		g_free (cobj->priv->src_output);
-		cobj->priv->src_output = NULL;
-	}
-
-	if (cobj->priv->toc) {
-		if (cobj->priv->clean  && cobj->priv->track_ready)
-			g_remove (cobj->priv->toc);
-
-		g_free (cobj->priv->toc);
-		cobj->priv->toc = NULL;
-	}
-
 	g_free(cobj->priv);
 	G_OBJECT_CLASS(parent_class)->finalize(object);
 }
@@ -337,76 +333,36 @@ brasero_cdrdao_read_stderr_image (BraseroCdrdao *cdrdao, const char *line)
 	int min, sec, sub, s1;
 
 	if (sscanf (line, "%d:%d:%d", &min, &sec, &sub) == 3) {
-		if (cdrdao->priv->isosize > 0) {
-			gdouble fraction;
-			long remaining = -1;
-			guint64 secs = min * 60 + sec;
+		gdouble fraction;
+		guint64 secs = min * 60 + sec;
 
-			cdrdao->priv->current_rate = -1;
-			fraction = (gdouble) secs / cdrdao->priv->isosize;
-			cdrdao->priv->b_written = cdrdao->priv->sectors_num * 2352 * fraction;
+		fraction = (gdouble) secs / cdrdao->priv->isosize;
+		BRASERO_JOB_TASK_SET_WRITTEN (cdrdao, secs * 75 * 2352);
+		BRASERO_JOB_TASK_SET_PROGRESS (cdrdao, fraction);
 
-			if (!cdrdao->priv->timer && secs > 2) {
-				/* let the speed regulate before keeping track */
-				cdrdao->priv->timer = g_timer_new ();
-				cdrdao->priv->start = secs;
-			}
-
-			if (cdrdao->priv->timer) {
-				gdouble elapsed;
-				gdouble rate;
-
-				elapsed = g_timer_elapsed (cdrdao->priv->timer, NULL);
-				rate = (gdouble) (secs - cdrdao->priv->start) / elapsed;
-
-				if (rate > 0.0) {
-					gdouble ave_rate;
-
-					ave_rate = brasero_burn_common_get_average_rate (&cdrdao->priv->rates, rate);
-					remaining = (cdrdao->priv->isosize - secs) / ave_rate;
-
-					/* NOTE: for me (Philippe)
-					 * ave_rate is sec/sec
-					 * 1 sec = 75 sectors,
-					 * 1 sector = 2352 bytes,
-					 * speed (CD) = 153600 bytes (2048*75) / sec for data CD
-					 * speed (CD) = 176400 bytes (2352*75) / sec for audio CD */
-					cdrdao->priv->current_rate = ave_rate * CDR_SPEED;
-				}
-			}
-
-			if (remaining >= 0)
-				brasero_job_progress_changed (BRASERO_JOB (cdrdao),
-							      fraction,
-							      remaining);
-			else
-				brasero_job_progress_changed (BRASERO_JOB (cdrdao),
-							      fraction,
-							      -1);
-		}
+		if (secs > 2)
+			BRASERO_JOB_TASK_START_PROGRESS (cdrdao, FALSE);
 	}
 	else if (sscanf (line, "Leadout %*s %*d %d:%d:%*d(%i)", &min, &sec, &s1) == 3) {
 		/* we get the number of sectors. As we added -raw each sector = 2352 bytes */
 		cdrdao->priv->sectors_num = s1;
-		cdrdao->priv->isosize = min * 60 + sec;
+		BRASERO_JOB_TASK_SET_TOTAL (cdrdao, s1 * 2352);
 
+		cdrdao->priv->isosize = min * 60 + sec;
 		if (cdrdao->priv->action == BRASERO_CDRDAO_ACTION_GET_SIZE)
 			brasero_job_finished (BRASERO_JOB (cdrdao));
 	}
 	else if (strstr (line, "Copying audio tracks")) {
-		brasero_job_action_changed (BRASERO_JOB (cdrdao),
-					    BRASERO_BURN_ACTION_DRIVE_COPY,
-					    FALSE);
+		BRASERO_JOB_TASK_SET_ACTION (cdrdao,
+					     BRASERO_BURN_ACTION_DRIVE_COPY,
+					     _("Copying audio track"),
+					     FALSE);
 	}
 	else if (strstr (line, "Copying data track")) {
-		brasero_job_action_changed (BRASERO_JOB (cdrdao),
-					    BRASERO_BURN_ACTION_DRIVE_COPY,
-					    FALSE);
-	}
-	else if (strstr (line, "toc and track data finished successfully")) {
-		brasero_job_progress_changed (BRASERO_JOB (cdrdao),
-					      1.0,
-					      0);
+		BRASERO_JOB_TASK_SET_ACTION (cdrdao,
+					     BRASERO_BURN_ACTION_DRIVE_COPY,
+					     _("Copying data track"),
+					     FALSE);
 	}
 	else
 		return FALSE;
@@ -421,105 +377,80 @@ brasero_cdrdao_read_stderr_record (BraseroCdrdao *cdrdao, const char *line)
 	guint written, total;
 
 	if (sscanf (line, "Wrote %u of %u (Buffers %d%%  %*s", &written, &total, &fifo) >= 2) {
-		long secs = -1;
-		gdouble fraction;
-
 		brasero_job_set_dangerous (BRASERO_JOB (cdrdao), TRUE);
-		cdrdao->priv->fifo = fifo;
-		cdrdao->priv->current_rate = -1;
-		cdrdao->priv->b_written = written * 1048576;
 
-		fraction = (gdouble) written / total;
+		BRASERO_JOB_TASK_SET_WRITTEN (cdrdao, written * 1048576);
+		BRASERO_JOB_TASK_SET_TOTAL (cdrdao, total * 1048576);
+		BRASERO_JOB_TASK_SET_ACTION (cdrdao,
+					     BRASERO_BURN_ACTION_WRITING,
+					     NULL,
+					     FALSE);
 
-		if (!cdrdao->priv->timer) {
-			/* let the speed regulate before keeping track */
-			cdrdao->priv->timer = g_timer_new ();
-			cdrdao->priv->start = (gint64) written;
-		}
-		else if (cdrdao->priv->timer) {
-			gdouble elapsed;
-			gdouble rate;
-
-			elapsed = g_timer_elapsed (cdrdao->priv->timer, NULL);
-			rate = (gdouble) (written - cdrdao->priv->start);
-
-			if (rate > 0 && elapsed > 1) {
-				gdouble ave_rate;
-
-				rate /= (gdouble) elapsed;
-				ave_rate = brasero_burn_common_get_average_rate (&cdrdao->priv->rates, rate);
-
-				secs = (total - written) / ave_rate;
-				cdrdao->priv->current_rate = rate * 1048576;
-			}
-		}
-
-		brasero_job_action_changed (BRASERO_JOB (cdrdao),
-					    BRASERO_BURN_ACTION_WRITING,
-					    FALSE);
-
-		brasero_job_progress_changed (BRASERO_JOB (cdrdao), 
-					      fraction * 0.98,
-					      secs);
+		BRASERO_JOB_TASK_START_PROGRESS (cdrdao, FALSE);
 	}
 	else if (sscanf (line, "Wrote %*s blocks. Buffer fill min") == 1) {
 		/* this is for fixating phase */
-		brasero_job_progress_changed (BRASERO_JOB (cdrdao), 
-					      0.98,
-					      -1);
-		brasero_job_action_changed (BRASERO_JOB (cdrdao),
-					    BRASERO_BURN_ACTION_FIXATING,
-					    FALSE);
+		BRASERO_JOB_TASK_SET_ACTION (cdrdao,
+					     BRASERO_BURN_ACTION_FIXATING,
+					     NULL,
+					     FALSE);
 	}
 	else if (sscanf (line, "Analyzing track %d %*s start %d:%d:%*d, length %*d:%*d:%*d", &track, &min, &sec) == 3) {
-		cdrdao->priv->track_num = track;
-		cdrdao->priv->track_start = min * 60 + sec;
+		gchar *string;
 
-		brasero_job_action_changed (BRASERO_JOB (cdrdao),
-					    BRASERO_BURN_ACTION_ANALYSING,
-					    TRUE);
+		string = g_strdup_printf (_("Analysing track %02i"), track);
+		BRASERO_JOB_TASK_SET_ACTION (cdrdao,
+					     BRASERO_BURN_ACTION_ANALYSING,
+					     string,
+					     TRUE);
+		g_free (string);
 	}
 	else if (sscanf (line, "%d:%d:%*d", &min, &sec) == 2) {
-		gdouble fraction;
-		guint64 secs = min * 60 + sec + cdrdao->priv->track_start;
+		gint64 total;
+		gint64 written;
+		guint64 secs = min * 60 + sec;
 
-		if (!cdrdao->priv->track_start && !cdrdao->priv->isosize)
-			return TRUE;
+		if (secs > 2)
+			BRASERO_JOB_TASK_START_PROGRESS (cdrdao, FALSE);
 
-		cdrdao->priv->b_written = sec * 75 * 2352;
+		written = secs * 75 * 2352;
+		BRASERO_JOB_TASK_SET_WRITTEN (cdrdao, written);
 
-		fraction = (gdouble) secs / (gdouble) cdrdao->priv->isosize;
-		brasero_job_progress_changed (BRASERO_JOB (cdrdao),
-					      fraction,
-					      -1);
-	}
-	else if (strstr (line, "Q sub-channels with CRC errors")) {
-		cdrdao->priv->track_num = 0;
-	}
-	else if (strstr (line, "Starting on-the-fly CD copy")
-	      ||  strstr (line, "Starting write at speed")) {
-		brasero_job_action_changed (BRASERO_JOB (cdrdao),
-					    BRASERO_BURN_ACTION_PREPARING,
-					    FALSE);
-	}
-	else if (strstr (line, "Executing power calibration")) {
-		brasero_job_action_changed (BRASERO_JOB (cdrdao),
-					    BRASERO_BURN_ACTION_PREPARING,
-					    FALSE);
+		total = cdrdao->priv->isosize * 75 * 2352;
+		BRASERO_JOB_TASK_SET_TOTAL (cdrdao, total);
 	}
 	else if (strstr (line, "Writing track")) {
 		brasero_job_set_dangerous (BRASERO_JOB (cdrdao), TRUE);
-		brasero_job_action_changed (BRASERO_JOB (cdrdao),
-					    BRASERO_BURN_ACTION_WRITING,
-					    FALSE);
 	}
 	else if (strstr (line, "Writing finished successfully")
 	     ||  strstr (line, "On-the-fly CD copying finished successfully")) {
 		brasero_job_set_dangerous (BRASERO_JOB (cdrdao), FALSE);
-		brasero_job_progress_changed (BRASERO_JOB (cdrdao), 1.0, -1);
 	}
-	else /* this is to report progress when a cd is being analysed before copy */
-		return FALSE;
+	else {
+		gchar *cuepath, *name;
+
+		if (!cdrdao->priv->track)
+			return FALSE;
+
+		cuepath = brasero_track_source_get_cue_localpath (cdrdao->priv->track);
+		if (!cuepath)
+			return FALSE;
+
+		if (!strstr (line, cuepath)) {
+			g_free (cuepath);
+			return FALSE;
+		}
+
+		name = g_path_get_basename (cuepath);
+		g_free (cuepath);
+
+		brasero_job_error (BRASERO_JOB (cdrdao),
+				   g_error_new (BRASERO_BURN_ERROR,
+						BRASERO_BURN_ERROR_GENERAL,
+						_("the cue file (%s) seems to be invalid"),
+						name));
+		g_free (name);
+	}
 
 	return TRUE;
 }
@@ -566,19 +497,33 @@ brasero_cdrdao_read_stderr (BraseroProcess *process, const char *line)
 
 static BraseroBurnResult
 brasero_cdrdao_get_track_type (BraseroImager *imager,
-			       BraseroTrackSourceType *type)
+			       BraseroTrackSourceType *type,
+			       BraseroImageFormat *format)
 {
 	BraseroCdrdao *cdrdao;
 
 	cdrdao = BRASERO_CDRDAO (imager);
 
 	if (!cdrdao->priv->track)
-		return BRASERO_BURN_NOT_READY;
+		BRASERO_JOB_NOT_READY (cdrdao);
 
-	if (cdrdao->priv->track->type != BRASERO_TRACK_SOURCE_DISC)
-		return BRASERO_BURN_NOT_SUPPORTED;
+	if (type)
+		*type = BRASERO_TRACK_SOURCE_IMAGE;
 
-	*type = BRASERO_TRACK_SOURCE_CUE;
+	if (format) {
+		BraseroImageFormat obj_format;
+		BraseroBurnCaps *caps;
+
+		caps = brasero_burn_caps_get_default ();
+		if (cdrdao->priv->format == BRASERO_IMAGE_FORMAT_ANY)
+			obj_format = brasero_burn_caps_get_imager_default_format (caps,
+										  cdrdao->priv->track);
+		else
+			obj_format = cdrdao->priv->format;
+		g_object_unref (caps);
+
+		*format = obj_format;
+	}
 
 	return BRASERO_BURN_OK;
 }
@@ -593,12 +538,8 @@ brasero_cdrdao_set_source (BraseroJob *job,
 	cdrdao = BRASERO_CDRDAO (job);
 
 	/* Remove any current output */
-	if (cdrdao->priv->src_output) {
-		if (cdrdao->priv->clean && cdrdao->priv->track_ready)
-			g_remove (cdrdao->priv->src_output);
-		if (cdrdao->priv->clean && cdrdao->priv->track_ready)
-			g_remove (cdrdao->priv->toc);
-	}
+	brasero_cdrdao_clean_output (cdrdao);
+
 	cdrdao->priv->track_ready = 0;
 
 	/* NOTE: we can accept ourselves as our source (cdrdao is both imager
@@ -615,8 +556,8 @@ brasero_cdrdao_set_source (BraseroJob *job,
 	/* NOTE: we don't check the type of source precisely since this check partly
 	 * depends on what we do with cdrdao afterward (it's both imager/recorder) */
         if (source->type != BRASERO_TRACK_SOURCE_DISC
-	&&  source->type != BRASERO_TRACK_SOURCE_CUE)
-		return BRASERO_BURN_NOT_SUPPORTED;
+	&& !(source->format & (BRASERO_IMAGE_FORMAT_CUE|BRASERO_IMAGE_FORMAT_CDRDAO)))
+		BRASERO_JOB_NOT_SUPPORTED (cdrdao);
 
 	cdrdao->priv->track = brasero_track_source_copy (source);
 	cdrdao->priv->isosize = -1;
@@ -636,24 +577,13 @@ brasero_cdrdao_set_output (BraseroImager *imager,
 
 	cdrdao = BRASERO_CDRDAO (imager);
 
-	if (cdrdao->priv->src_output) {
-		if (cdrdao->priv->clean && cdrdao->priv->track_ready)
-			g_remove (cdrdao->priv->src_output);
+	brasero_cdrdao_clean_output (cdrdao);
 
+	if (cdrdao->priv->src_output) {
 		g_free (cdrdao->priv->src_output);
 		cdrdao->priv->src_output = NULL;
 	}
 
-	if (cdrdao->priv->toc) {
-		if (cdrdao->priv->clean && cdrdao->priv->track_ready)
-			g_remove (cdrdao->priv->toc);
-
-		g_free (cdrdao->priv->toc);
-		cdrdao->priv->toc = NULL;
-	}
-
-	cdrdao->priv->track_ready = 0;
-
 	if (output)
 		cdrdao->priv->src_output = g_strdup (output);
 
@@ -666,6 +596,7 @@ brasero_cdrdao_set_output (BraseroImager *imager,
 static BraseroBurnResult
 brasero_cdrdao_set_output_type (BraseroImager *imager,
 				BraseroTrackSourceType type,
+				BraseroImageFormat format,
 				GError **error)
 {
 	BraseroCdrdao *cdrdao;
@@ -673,28 +604,73 @@ brasero_cdrdao_set_output_type (BraseroImager *imager,
 	cdrdao = BRASERO_CDRDAO (imager);
 
 	if (type != BRASERO_TRACK_SOURCE_DEFAULT
-	&&  type != BRASERO_TRACK_SOURCE_CUE)
-		return BRASERO_BURN_NOT_SUPPORTED;
+	&&  type != BRASERO_TRACK_SOURCE_IMAGE)
+		BRASERO_JOB_NOT_SUPPORTED (cdrdao);
 
-	/* NOTE: no need to keep this value since cdrdao can only output cue */
+	if (!(format & (BRASERO_IMAGE_FORMAT_CDRDAO|BRASERO_IMAGE_FORMAT_CUE)))
+		BRASERO_JOB_NOT_SUPPORTED (cdrdao);
+
+	/* no need to keep the type since it can only be an image */
+	cdrdao->priv->format = format;
 	return BRASERO_BURN_OK;
 }
 
+static BraseroBurnResult
+brasero_cdrdao_toc2cue (BraseroCdrdao *cdrdao,
+			const BraseroTrackSource *toc_track,
+			BraseroTrackSource **cue_track,
+			GError **error)
+{
+	BraseroImager *imager;
+	BraseroBurnResult result;
+
+	imager = BRASERO_IMAGER (brasero_toc2cue_new ());
+	brasero_job_set_slave (BRASERO_JOB (cdrdao), BRASERO_JOB (imager));
+	g_object_unref (imager);
+
+	result = brasero_job_set_source (BRASERO_JOB (imager),
+					 toc_track,
+					 error);
+	if (result != BRASERO_BURN_OK)
+		return result;
+
+	result = brasero_imager_set_output (imager,
+					    cdrdao->priv->src_output,
+					    cdrdao->priv->overwrite,
+					    cdrdao->priv->clean,
+					    error);
+	if (result != BRASERO_BURN_OK)
+		return result;
+
+	brasero_job_set_relay_slave_signals (BRASERO_JOB (cdrdao), TRUE);
+	result = brasero_imager_get_track (imager,
+					   cue_track,
+					   error);
+	brasero_job_set_relay_slave_signals (BRASERO_JOB (cdrdao), FALSE);
+
+	if (result != BRASERO_BURN_OK)
+		return result;
+
+	brasero_job_set_slave (BRASERO_JOB (cdrdao), NULL);
+	return result;
+}
+
 static BraseroBurnResult
 brasero_cdrdao_get_track (BraseroImager *imager,
 			  BraseroTrackSource **track,
 			  GError **error)
 {
 	BraseroCdrdao *cdrdao;
+	BraseroImageFormat format;
 	BraseroTrackSource *retval;
 
 	cdrdao = BRASERO_CDRDAO (imager);
 
 	if (!cdrdao->priv->track)
-		return BRASERO_BURN_NOT_READY;
+		BRASERO_JOB_NOT_READY (cdrdao);
 
 	if (cdrdao->priv->track->type != BRASERO_TRACK_SOURCE_DISC)
-		return BRASERO_BURN_NOT_SUPPORTED;
+		BRASERO_JOB_NOT_SUPPORTED (cdrdao);
 
 	if (!cdrdao->priv->track_ready) {
 		BraseroBurnResult result;
@@ -707,17 +683,39 @@ brasero_cdrdao_get_track (BraseroImager *imager,
 			return result;
 
 		cdrdao->priv->track_ready = 1;
-		cdrdao->priv->b_written = cdrdao->priv->sectors_num * 2352;
 	}
 
 	retval = g_new0 (BraseroTrackSource, 1);
 
-	retval->type = BRASERO_TRACK_SOURCE_CUE;
-	retval->contents.cue.toc = g_strdup_printf ("file://%s.toc", cdrdao->priv->src_output);
-	retval->contents.cue.image = g_strdup_printf ("file://%s", cdrdao->priv->src_output);
+	retval->type = BRASERO_TRACK_SOURCE_IMAGE;
+	retval->format = BRASERO_IMAGE_FORMAT_CDRDAO;
+
+	/* the output given is the .cue file */
+	retval->contents.image.toc = gnome_vfs_get_uri_from_local_path (cdrdao->priv->output);
+	retval->contents.image.image = gnome_vfs_get_uri_from_local_path (cdrdao->priv->datafile);
+
+	if (cdrdao->priv->format == BRASERO_IMAGE_FORMAT_ANY) {
+		BraseroBurnCaps *caps;
+
+		caps = brasero_burn_caps_get_default ();
+		format = brasero_burn_caps_get_imager_default_format (caps,
+								      cdrdao->priv->track);
+		g_object_unref (caps);
+	}
+	else
+		format = cdrdao->priv->format;
+
+	if (format == BRASERO_IMAGE_FORMAT_CUE) {
+		BraseroBurnResult result;
+
+		result = brasero_cdrdao_toc2cue (cdrdao, retval, track, error);
+
+		g_remove (retval->contents.image.toc);
+		brasero_track_source_free (retval);
+		return result;
+	}
 
 	*track = retval;
-
 	return BRASERO_BURN_OK;
 }
 
@@ -748,8 +746,7 @@ brasero_cdrdao_get_size_image (BraseroImager *imager,
 			return result;
 
 		/* now we must remove the .toc and .bin in case they were created */
-		g_remove (cdrdao->priv->src_output);
-		g_remove (cdrdao->priv->toc);
+		brasero_cdrdao_clean_output (cdrdao);
 	}
 
 	if (sectors) {
@@ -766,7 +763,6 @@ static void
 brasero_cdrdao_set_argv_device (BraseroCdrdao *cdrdao,
 				GPtrArray *argv)
 {
-	/* it seems that cdrdao (1.2.0) doesn't like the cdrecord_id on my computer */
 	g_ptr_array_add (argv, g_strdup ("--device"));
 	if (NCB_DRIVE_GET_DEVICE (cdrdao->priv->drive))
 		g_ptr_array_add (argv, g_strdup (NCB_DRIVE_GET_DEVICE (cdrdao->priv->drive)));
@@ -776,10 +772,10 @@ static void
 brasero_cdrdao_set_argv_common_rec (BraseroCdrdao *cdrdao,
 				    GPtrArray *argv)
 {
-	char *speed_str;
+	gchar *speed_str;
 
 	g_ptr_array_add (argv, g_strdup ("--speed"));
-	speed_str = g_strdup_printf ("%d", cdrdao->priv->speed);
+	speed_str = g_strdup_printf ("%d", cdrdao->priv->rate);
 	g_ptr_array_add (argv, speed_str);
 
 	if (cdrdao->priv->overburn)
@@ -790,7 +786,7 @@ brasero_cdrdao_set_argv_common_rec (BraseroCdrdao *cdrdao,
 
 static void
 brasero_cdrdao_set_argv_common (BraseroCdrdao *cdrdao,
-				    GPtrArray *argv)
+				GPtrArray *argv)
 {
 	if (cdrdao->priv->dummy)
 		g_ptr_array_add (argv, g_strdup ("--simulate"));
@@ -810,11 +806,11 @@ brasero_cdrdao_set_argv_record (BraseroCdrdao *cdrdao,
 	BraseroTrackSource *track;
 
 	if (!cdrdao->priv->drive)
-		return BRASERO_BURN_NOT_READY;
+		BRASERO_JOB_NOT_READY (cdrdao);
 
 	track = cdrdao->priv->track;
 	if (!track)
-		return BRASERO_BURN_NOT_READY;
+		BRASERO_JOB_NOT_READY (cdrdao);
 
         if (track->type == BRASERO_TRACK_SOURCE_DISC) {
 		NautilusBurnDrive *source;
@@ -831,24 +827,35 @@ brasero_cdrdao_set_argv_record (BraseroCdrdao *cdrdao,
 		g_ptr_array_add (argv, g_strdup ("--source-device"));
 		g_ptr_array_add (argv, g_strdup (NCB_DRIVE_GET_DEVICE (source)));
 	}
-	else if (track->type == BRASERO_TRACK_SOURCE_CUE) {
+	else if (track->type == BRASERO_TRACK_SOURCE_IMAGE) {
 		gchar *cuepath;
 
+		if (track->format & BRASERO_IMAGE_FORMAT_CUE)
+			cuepath = brasero_track_source_get_cue_localpath (track);
+		else if (track->format & BRASERO_IMAGE_FORMAT_CDRDAO)
+			cuepath = brasero_track_source_get_cdrdao_localpath (track);
+		else
+			BRASERO_JOB_NOT_SUPPORTED (cdrdao);
+
+		if (!cuepath)
+			return BRASERO_BURN_ERR;
+
 		g_ptr_array_add (argv, g_strdup ("write"));
 
 		brasero_cdrdao_set_argv_device (cdrdao, argv);
 		brasero_cdrdao_set_argv_common (cdrdao, argv);
 		brasero_cdrdao_set_argv_common_rec (cdrdao, argv);
 
-		cuepath = brasero_track_source_get_cue_localpath (track);
-		if (!cuepath)
-			return BRASERO_BURN_ERR;
-
 		g_ptr_array_add (argv, cuepath);
 	}
 	else
-		return BRASERO_BURN_NOT_SUPPORTED;
+		BRASERO_JOB_NOT_SUPPORTED (cdrdao);
 
+	BRASERO_JOB_TASK_SET_USE_AVERAGE_RATE (cdrdao, TRUE);
+	BRASERO_JOB_TASK_SET_ACTION (cdrdao,
+				     BRASERO_BURN_ACTION_PREPARING,
+				     NULL,
+				     FALSE);
 	return BRASERO_BURN_OK;
 }
 
@@ -857,7 +864,7 @@ brasero_cdrdao_set_argv_blank (BraseroCdrdao *cdrdao,
 			       GPtrArray *argv)
 {
 	if (!cdrdao->priv->drive)
-		return BRASERO_BURN_NOT_READY;
+		BRASERO_JOB_NOT_READY (cdrdao);
 
 	g_ptr_array_add (argv, g_strdup ("blank"));
 
@@ -869,6 +876,11 @@ brasero_cdrdao_set_argv_blank (BraseroCdrdao *cdrdao,
 		g_ptr_array_add (argv, g_strdup ("full"));
 	}
 
+	BRASERO_JOB_TASK_SET_ACTION (cdrdao,
+				     BRASERO_BURN_ACTION_ERASING,
+				     NULL,
+				     FALSE);
+
 	return BRASERO_BURN_OK;
 }
 
@@ -877,11 +889,13 @@ brasero_cdrdao_set_argv_image (BraseroCdrdao *cdrdao,
 			       GPtrArray *argv,
 			       GError **error)
 {
+	gchar *output, *datafile;
 	BraseroBurnResult result;
 	NautilusBurnDrive *source;
+	BraseroImageFormat format;
 
 	if (!cdrdao->priv->track)
-		return BRASERO_BURN_NOT_READY;
+		BRASERO_JOB_NOT_READY (cdrdao);
 
 	g_ptr_array_add (argv, g_strdup ("read-cd"));
 	g_ptr_array_add (argv, g_strdup ("--device"));
@@ -891,25 +905,63 @@ brasero_cdrdao_set_argv_image (BraseroCdrdao *cdrdao,
 
 	g_ptr_array_add (argv, g_strdup ("--read-raw"));
 
-	result = brasero_burn_common_check_output (&cdrdao->priv->src_output,
+	/* This is done so that if a cue file is required we first generate
+	 * a temporary toc file that will be later converted to a cue file.
+	 * The datafile is written where it should be from the start. */
+	if (cdrdao->priv->format == BRASERO_IMAGE_FORMAT_ANY) {
+		BraseroBurnCaps *caps;
+
+		caps = brasero_burn_caps_get_default ();
+		format = brasero_burn_caps_get_imager_default_format (caps,
+								      cdrdao->priv->track);
+		g_object_unref (caps);
+	}
+	else
+		format = cdrdao->priv->format;
+
+	if (cdrdao->priv->src_output
+	&& (format & BRASERO_IMAGE_FORMAT_CDRDAO))
+		output = g_strdup (cdrdao->priv->src_output);
+	else
+		output = NULL;
+
+	if (cdrdao->priv->src_output)
+		datafile = brasero_get_file_complement (format,
+							FALSE,
+							cdrdao->priv->src_output);
+	else
+		datafile = NULL;
+
+	result = brasero_burn_common_check_output (&output,
+						   BRASERO_IMAGE_FORMAT_CDRDAO,
+						   FALSE,
 						   cdrdao->priv->overwrite,
-						   &cdrdao->priv->toc,
+						   &datafile,
 						   error);
 	if (result != BRASERO_BURN_OK)
 		return result;
 
+	brasero_cdrdao_clean_output (cdrdao);
+	cdrdao->priv->datafile = g_strdup (datafile);
+	cdrdao->priv->output = g_strdup (output);
+
+	if (cdrdao->priv->action == BRASERO_CDRDAO_ACTION_GET_SIZE) {
+		BRASERO_JOB_TASK_SET_ACTION (cdrdao,
+					     BRASERO_BURN_ACTION_GETTING_SIZE,
+					     NULL,
+					     FALSE);
+		BRASERO_JOB_TASK_START_PROGRESS (cdrdao, FALSE);
+	}
+
 	g_ptr_array_add (argv, g_strdup ("--datafile"));
-	g_ptr_array_add (argv, g_strdup (cdrdao->priv->src_output));
+	g_ptr_array_add (argv, datafile);
 
 	g_ptr_array_add (argv, g_strdup ("-v"));
 	g_ptr_array_add (argv, g_strdup ("2"));
 
-	g_ptr_array_add (argv, g_strdup (cdrdao->priv->toc));
+	g_ptr_array_add (argv, output);
 
-	if (cdrdao->priv->action == BRASERO_CDRDAO_ACTION_GET_SIZE)
-		brasero_job_action_changed (BRASERO_JOB (cdrdao),
-					    BRASERO_BURN_ACTION_GETTING_SIZE,
-					    FALSE);
+	BRASERO_JOB_TASK_SET_USE_AVERAGE_RATE (cdrdao, TRUE);
 
 	return BRASERO_BURN_OK;
 }
@@ -925,9 +977,8 @@ brasero_cdrdao_set_argv (BraseroProcess *process,
 	cdrdao = BRASERO_CDRDAO (process);
 
 	if (has_master)
-		return BRASERO_BURN_NOT_SUPPORTED;
+		BRASERO_JOB_NOT_SUPPORTED (cdrdao);
 
-	cdrdao->priv->start = 0;
 	brasero_job_set_run_slave (BRASERO_JOB (cdrdao), FALSE);
 
 	/* sets the first argv */
@@ -943,7 +994,7 @@ brasero_cdrdao_set_argv (BraseroProcess *process,
 		return brasero_cdrdao_set_argv_image (cdrdao, argv, error);
 
 
-	return BRASERO_BURN_NOT_READY;
+	BRASERO_JOB_NOT_READY (cdrdao);
 }
 
 static BraseroBurnResult
@@ -954,24 +1005,8 @@ brasero_cdrdao_post (BraseroProcess *process,
 
 	cdrdao = BRASERO_CDRDAO (process);
 
-	if (cdrdao->priv->timer) {
-		g_timer_destroy (cdrdao->priv->timer);
-		cdrdao->priv->timer = NULL;
-	}
-
-	if (cdrdao->priv->rates) {
-		g_slist_free (cdrdao->priv->rates);
-		cdrdao->priv->rates = NULL;
-	}
-
-	if (retval == BRASERO_BURN_CANCEL) {
-		if (cdrdao->priv->src_output)
-			g_remove (cdrdao->priv->src_output);
-		if (cdrdao->priv->toc)
-			g_remove (cdrdao->priv->toc);
-	}
-
-	cdrdao->priv->current_rate = -1;
+	if (retval == BRASERO_BURN_CANCEL)
+		brasero_cdrdao_clean_output (cdrdao);
 
 	return BRASERO_BURN_OK;
 }
@@ -988,7 +1023,7 @@ brasero_cdrdao_set_drive (BraseroRecorder *recorder,
 
 	media = nautilus_burn_drive_get_media_type (drive);
 	if (media > NAUTILUS_BURN_MEDIA_TYPE_CDRW)
-		return BRASERO_BURN_NOT_SUPPORTED;
+		BRASERO_JOB_NOT_SUPPORTED (cdrdao);
 
 	if (cdrdao->priv->drive) {
 		nautilus_burn_drive_unref (cdrdao->priv->drive);
@@ -1031,7 +1066,7 @@ brasero_cdrdao_set_rate (BraseroJob *job,
 		return BRASERO_BURN_RUNNING;
 	
 	cdrdao = BRASERO_CDRDAO (job);
-	cdrdao->priv->speed = speed / CDR_SPEED;
+	cdrdao->priv->rate = speed / CDR_SPEED;
 
 	return BRASERO_BURN_OK;
 
@@ -1077,65 +1112,3 @@ brasero_cdrdao_blank (BraseroRecorder *recorder,
 	return result;
 }
 
-static BraseroBurnResult
-brasero_cdrdao_get_rate (BraseroJob *job, gint64 *rate)
-{
-	BraseroCdrdao *cdrdao;
-
-	cdrdao = BRASERO_CDRDAO (job);
-
-	if (rate)
-		*rate = cdrdao->priv->current_rate;
-
-	return BRASERO_BURN_OK;
-}
-
-static BraseroBurnResult
-brasero_cdrdao_get_written (BraseroJob *job,
-			    gint64 *written)
-{
-	BraseroCdrdao *cdrdao;
-
-	cdrdao = BRASERO_CDRDAO (job);
-
-	if (written)
-		*written = cdrdao->priv->b_written;
-
-	return BRASERO_BURN_OK;
-}
-
-static BraseroBurnResult
-brasero_cdrdao_get_fifo (BraseroRecorder *recorder,
-			 gint *fifo)
-{
-	BraseroCdrdao *cdrdao;
-
-	cdrdao = BRASERO_CDRDAO (recorder);
-
-	if (fifo)
-		*fifo = cdrdao->priv->fifo;
-
-	return BRASERO_BURN_OK;
-}
-
-static BraseroBurnResult
-brasero_cdrdao_get_action_string (BraseroJob *job, 
-				  BraseroBurnAction action,
-				  char **string)
-{
-	BraseroCdrdao *cdrdao;
-
-	cdrdao = BRASERO_CDRDAO (job);
-
-	if (action != BRASERO_BURN_ACTION_ANALYSING) {
-		const char *tmp;
-
-		tmp = brasero_burn_action_to_string (action);
-		*string = g_strdup (tmp);
-	}
-	else
-		*string = g_strdup_printf (_("Analysing track %02i"),
-					   cdrdao->priv->track_num);
-
-	return BRASERO_BURN_OK;
-}
