@@ -29,6 +29,12 @@
 
 #include <string.h>
 #include <math.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <unistd.h>
+#include <errno.h>
+#include <stdlib.h>
 
 #include <glib.h>
 #include <glib-object.h>
@@ -61,16 +67,9 @@ brasero_cdrecord_set_argv (BraseroProcess *process,
 			   GPtrArray *argv,
 			   gboolean has_master, 
 			   GError **error);
-
 static BraseroBurnResult
-brasero_cdrecord_get_rate (BraseroJob *job,
-			   gint64 *rate);
-static BraseroBurnResult
-brasero_cdrecord_get_written (BraseroJob *job,
-			      gint64 *written);
-static BraseroBurnResult
-brasero_cdrecord_get_fifo (BraseroRecorder *recorder,
-			   gint *fifo);
+brasero_cdrecord_post (BraseroProcess *process,
+		       BraseroBurnResult retval);
 
 static BraseroBurnResult
 brasero_cdrecord_set_drive (BraseroRecorder *recorder,
@@ -86,7 +85,7 @@ brasero_cdrecord_set_flags (BraseroRecorder *recorder,
 			    GError **error);
 static BraseroBurnResult
 brasero_cdrecord_set_rate (BraseroJob *job,
-			    gint64 speed);
+			   gint64 rate);
 
 static BraseroBurnResult
 brasero_cdrecord_record (BraseroRecorder *recorder,
@@ -95,11 +94,6 @@ static BraseroBurnResult
 brasero_cdrecord_blank (BraseroRecorder *recorder,
 			GError **error);
 
-static BraseroBurnResult
-brasero_cdrecord_get_action_string (BraseroJob *job,
-				    BraseroBurnAction action,
-				    char **string);
-
 typedef enum {
 	BRASERO_CD_RECORD_ACTION_NONE,
 	BRASERO_CD_RECORD_ACTION_BLANK,
@@ -111,31 +105,29 @@ struct BraseroCDRecordPrivate {
 
 	NautilusBurnDrive *drive;
 	BraseroTrackSource *track;
-	BraseroTrackSource *inf;
-	int speed;
+	GSList *infs;
+	gint rate;
+
+	gchar *tmpdir;
 
 	gint64 current_track_end_pos;
 	gint64 current_track_written;
 	gint64 tracks_total_bytes;
-	gint64 b_written;
 
-	int current_track_num;
-	int track_count;
+	gint current_track_num;
+	gint track_count;
 
-	int fifo;
-	gint64 cur_speed;
+	gint minbuf;
 
-	int minbuf;
+	gint dao:1;
+	gint dummy:1;
+	gint multi:1;
+	gint nograce:1;
+	gint overburn:1;
+	gint immediate:1;
+	gint burnproof:1;
 
-	int dao:1;
-	int dummy:1;
-	int multi:1;
-	int nograce:1;
-	int overburn:1;
-	int immediate:1;
-	int burnproof:1;
-
-	int blank_fast:1;
+	gint blank_fast:1;
 };
 
 static GObjectClass *parent_class = NULL;
@@ -187,15 +179,13 @@ brasero_cdrecord_class_init (BraseroCDRecordClass *klass)
 	parent_class = g_type_class_peek_parent(klass);
 	object_class->finalize = brasero_cdrecord_finalize;
 
-	job_class->get_action_string = brasero_cdrecord_get_action_string;
-	job_class->get_written = brasero_cdrecord_get_written;
 	job_class->set_source = brasero_cdrecord_set_source;
 	job_class->set_rate = brasero_cdrecord_set_rate;
-	job_class->get_rate = brasero_cdrecord_get_rate;
 
 	process_class->stderr_func = brasero_cdrecord_stderr_read;
 	process_class->stdout_func = brasero_cdrecord_stdout_read;
 	process_class->set_argv = brasero_cdrecord_set_argv;
+	process_class->post = brasero_cdrecord_post;
 }
 
 static void
@@ -205,7 +195,6 @@ brasero_cdrecord_iface_init_record (BraseroRecorderIFace *iface)
 	iface->record = brasero_cdrecord_record;
 	iface->set_drive = brasero_cdrecord_set_drive;
 	iface->set_flags = brasero_cdrecord_set_flags;
-	iface->get_fifo = brasero_cdrecord_get_fifo;
 }
 
 static void
@@ -214,11 +203,37 @@ brasero_cdrecord_init (BraseroCDRecord *obj)
 	obj->priv = g_new0 (BraseroCDRecordPrivate, 1);
 }
 
+static void
+brasero_cdrecord_stop_real (BraseroCDRecord *cdrecord)
+{
+	brasero_job_set_slave (BRASERO_JOB (cdrecord), NULL);
+
+	if (cdrecord->priv->infs) {
+		g_slist_foreach (cdrecord->priv->infs,
+				 (GFunc) g_remove,
+				 NULL);
+		g_slist_foreach (cdrecord->priv->infs,
+				 (GFunc) g_free,
+				 NULL);
+		g_slist_free (cdrecord->priv->infs);
+		cdrecord->priv->infs = NULL;
+	}
+
+	if (cdrecord->priv->tmpdir) {
+		g_remove (cdrecord->priv->tmpdir);
+		g_free (cdrecord->priv->tmpdir);
+		cdrecord->priv->tmpdir = NULL;
+	}
+}
+
 static void
 brasero_cdrecord_finalize (GObject *object)
 {
 	BraseroCDRecord *cobj;
-	cobj = BRASERO_CD_RECORD(object);
+
+	cobj = BRASERO_CD_RECORD (object);
+
+	brasero_cdrecord_stop_real (cobj);
 
 	if (cobj->priv->drive) {
 		nautilus_burn_drive_unref (cobj->priv->drive);
@@ -230,13 +245,8 @@ brasero_cdrecord_finalize (GObject *object)
 		cobj->priv->track = NULL;
 	}
 
-	if (cobj->priv->inf) {
-		brasero_track_source_free (cobj->priv->inf);
-		cobj->priv->inf = NULL;
-	}
-
-	g_free(cobj->priv);
-	G_OBJECT_CLASS(parent_class)->finalize(object);
+	g_free (cobj->priv);
+	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 BraseroCDRecord *
@@ -250,7 +260,7 @@ brasero_cdrecord_new ()
 }
 
 static BraseroBurnResult
-brasero_cdrecord_stderr_read (BraseroProcess *process, const char *line)
+brasero_cdrecord_stderr_read (BraseroProcess *process, const gchar *line)
 {
 	BraseroCDRecord *cdrecord = BRASERO_CD_RECORD (process);
 
@@ -266,6 +276,12 @@ brasero_cdrecord_stderr_read (BraseroProcess *process, const char *line)
 						BRASERO_BURN_ERROR_MEDIA_NONE,
 						_("There doesn't seem to be a disc in the drive")));
 	}
+	else if (strstr (line, "Input buffer error, aborting") != NULL) {
+		brasero_job_error (BRASERO_JOB (process),
+				   g_error_new (BRASERO_BURN_ERROR,
+						BRASERO_BURN_ERROR_MEDIA_NONE,
+						_("input buffer error")));
+	}
 	else if (strstr (line, "This means that we are checking recorded media.") != NULL) {
 		brasero_job_error (BRASERO_JOB (process),
 				   g_error_new (BRASERO_BURN_ERROR,
@@ -290,10 +306,7 @@ brasero_cdrecord_stderr_read (BraseroProcess *process, const char *line)
 		brasero_job_error (BRASERO_JOB (process),
 				   g_error_new (BRASERO_BURN_ERROR,
 						BRASERO_BURN_ERROR_GENERAL,
-						_("an unknown error occured.")));
-		/* FIXME: future message if overburn is activated.
-		 * a write error occured which was likely due to overburning the 
-		 * disc */
+						_("a write error occured which was likely due to overburning the disc")));
 	}
 	else if (strstr (line, "Inappropriate audio coding")) {
 		brasero_job_error (BRASERO_JOB (process),
@@ -353,89 +366,60 @@ brasero_cdrecord_stderr_read (BraseroProcess *process, const char *line)
 						_("the cd information could not be written")));
 	}
 	else if (strstr (line, "Cannot fixate disk")) {
-		/* FIXME: replace error message with
-		 * "the disc could not be closed" */
 		brasero_job_error (BRASERO_JOB (process),
 				   g_error_new (BRASERO_BURN_ERROR,
 						BRASERO_BURN_ERROR_GENERAL,
-						_("internal error")));
+						_("the disc could not be closed")));
+	}
+	else if (strstr (line, "Bad audio track size")) {
+		brasero_job_error (BRASERO_JOB (process),
+				   g_error_new (BRASERO_BURN_ERROR,
+						BRASERO_BURN_ERROR_GENERAL,
+						_("the audio tracks are too short or not a multiple of 2352")));
 	}
 
 	return BRASERO_BURN_OK;
 }
 
-static void
-brasero_cdrecord_check_fifo (BraseroCDRecord *cdrecord, int buf)
-{
-	BraseroJob *slave;
-	gint64 rate = (cdrecord->priv->speed + 1) * CDR_SPEED;
-
-	slave = brasero_job_get_slave (BRASERO_JOB (cdrecord));
-	if (!slave)
-		return;
-
-	if (cdrecord->priv->fifo < 50) {
-		/* we try to raise the rate */
-		rate = (cdrecord->priv->speed * 2) * CDR_SPEED;
-	}
-	else if (cdrecord->priv->fifo > 95) {
-		/* we try to lower the rate */
-		if (buf < 90)
-			rate = (cdrecord->priv->speed * 1.5) * CDR_SPEED;
-	}
-
-	brasero_job_debug_message (BRASERO_JOB (cdrecord),
-				   "setting rate to %" G_GINT64_FORMAT,
-				   rate);
-	brasero_job_set_rate (slave, rate);
-}
-
 static void
 brasero_cdrecord_compute (BraseroCDRecord *cdrecord,
-			  int mb_written,
-			  int mb_total,
-			  gdouble speed,
-			  int track_num)
+			  gint mb_written,
+			  gint mb_total,
+			  gint track_num)
 {
-	double percent;
-	gint64 bytes;
+	gboolean track_num_changed = FALSE;
+	gchar *action_string;
 	gint64 this_remain;
+	gint64 bytes;
 	gint64 total;
-	long secs;
 
 	if (cdrecord->priv->tracks_total_bytes > 0)
 		total = cdrecord->priv->tracks_total_bytes;
 	else
 		total = mb_total * 1048576;
 
+	BRASERO_JOB_TASK_SET_TOTAL (cdrecord, total);
+
 	if (track_num > cdrecord->priv->current_track_num) {
+		track_num_changed = TRUE;
 		cdrecord->priv->current_track_num = track_num;
 		cdrecord->priv->current_track_end_pos += mb_total * 1048576;
 	}
 
 	this_remain = (mb_total - mb_written) * 1048576;
 	bytes = (total - cdrecord->priv->current_track_end_pos) + this_remain;
-	cdrecord->priv->b_written = total - bytes;
+	BRASERO_JOB_TASK_SET_WRITTEN (cdrecord, total - bytes);
 
-	secs = brasero_burn_common_compute_time_remaining (bytes, (double) speed * CDR_SPEED);
-
-	brasero_job_action_changed (BRASERO_JOB (cdrecord),
-				    BRASERO_BURN_ACTION_WRITING,
-				    FALSE);
-
-	if (cdrecord->priv->tracks_total_bytes > 0) {
-		percent = 0.98 * (1.0 - (double) bytes / (double) cdrecord->priv->tracks_total_bytes);
-	}
-	else {
-		percent = 0.98 * ((double) ((track_num - 1) / (double)cdrecord->priv->track_count)
-			  + ((double)mb_written / mb_total) / (double)cdrecord->priv->track_count);
-	}
-
-	brasero_job_progress_changed (BRASERO_JOB (cdrecord), percent, secs);
+	action_string = g_strdup_printf ("Writing track %02i", track_num);
+	BRASERO_JOB_TASK_SET_ACTION (cdrecord,
+				     BRASERO_BURN_ACTION_WRITING,
+				     action_string,
+				     track_num_changed);
+	g_free (action_string);
 }
 
 static BraseroBurnResult
-brasero_cdrecord_stdout_read (BraseroProcess *process, const char *line)
+brasero_cdrecord_stdout_read (BraseroProcess *process, const gchar *line)
 {
 	guint track;
 	guint speed_1, speed_2;
@@ -446,52 +430,48 @@ brasero_cdrecord_stdout_read (BraseroProcess *process, const char *line)
 
 	if (sscanf (line, "Track %2u: %d of %d MB written (fifo %d%%) [buf %d%%] %d.%dx.",
 		    &track, &mb_written, &mb_total, &fifo, &buf, &speed_1, &speed_2) == 7) {
-		gdouble speed;
+		gdouble current_rate;
 
-		cdrecord->priv->fifo = fifo;
-		cdrecord->priv->cur_speed = speed_1 * CDR_SPEED + speed_2 * CDR_SPEED / 10;
+		current_rate = (gdouble) ((gdouble) speed_1 +
+			       (gdouble) speed_2 / 10.0) *
+			       (gdouble) CDR_SPEED;
+		BRASERO_JOB_TASK_SET_RATE (cdrecord, current_rate);
 
 		cdrecord->priv->current_track_written = mb_written * 1048576;
-		speed = (gdouble) speed_1 + (gdouble) speed_2 / 10.0;
 		brasero_cdrecord_compute (cdrecord,
 					  mb_written,
 					  mb_total,
-					  speed,
 					  track);
 
-		brasero_cdrecord_check_fifo (cdrecord, buf);
+		BRASERO_JOB_TASK_START_PROGRESS (cdrecord, FALSE);
 	} 
 	else if (sscanf (line, "Track %2u:    %d MB written (fifo %d%%) [buf  %d%%]  %d.%dx.",
 			 &track, &mb_written, &fifo, &buf, &speed_1, &speed_2) == 6) {
-		int mb_total;
+		gdouble current_rate;
 
 		/* this line is printed when cdrecord writes on the fly */
-		cdrecord->priv->fifo = fifo;
-		cdrecord->priv->cur_speed = speed_1 * CDR_SPEED + speed_2 * CDR_SPEED / 10;
+		current_rate = (gdouble) ((gdouble) speed_1 +
+			       (gdouble) speed_2 / 10.0) *
+			       (gdouble) CDR_SPEED;
+		BRASERO_JOB_TASK_SET_RATE (cdrecord, current_rate);
+
 		cdrecord->priv->current_track_written = mb_written * 1048576;
 
 		if (cdrecord->priv->track->type == BRASERO_TRACK_SOURCE_IMAGER) {
-			gdouble speed;
-
 			/* we must ask the imager what is the total size */
 			brasero_imager_get_size (BRASERO_IMAGER (cdrecord->priv->track->contents.imager.obj),
 						 &cdrecord->priv->tracks_total_bytes,
 						 FALSE, 
 						 NULL);
+
 			mb_total = cdrecord->priv->tracks_total_bytes / 1048576;
-			speed = (gdouble) speed_1 + (gdouble) speed_2 / 10.0;
 			brasero_cdrecord_compute (cdrecord,
 						  mb_written,
 						  mb_total,
-						  speed,
 						  track);
 		}
-		else
-			brasero_job_action_changed (BRASERO_JOB (cdrecord),
-						    BRASERO_BURN_ACTION_WRITING,
-						    FALSE);
 
-		brasero_cdrecord_check_fifo (cdrecord, buf);
+		BRASERO_JOB_TASK_START_PROGRESS (cdrecord, FALSE);
 	}
 	else if (sscanf (line, "Track %*d: %*s %d MB ", &mb_total) == 1) {
 		if (mb_total > 0) {
@@ -499,17 +479,41 @@ brasero_cdrecord_stdout_read (BraseroProcess *process, const char *line)
 		}
 	}
 	else if (strstr (line, "Sending CUE sheet")) {
-		brasero_job_action_changed (BRASERO_JOB (process),
-					    BRASERO_BURN_ACTION_WRITING_CD_TEXT,
-					    FALSE);
+		gboolean is_audio = FALSE;
+		BraseroTrackSource *source;
+		BraseroImageFormat format;
+		BraseroTrackSourceType type = BRASERO_TRACK_SOURCE_UNKNOWN;
+
+		source = cdrecord->priv->track;
+
+		/* See if we are in an audio case which would mean we're writing
+		 * CD-TEXT */
+		if (source->type == BRASERO_TRACK_SOURCE_IMAGER)
+			brasero_imager_get_track_type (source->contents.imager.obj,
+						       &type,
+						       &format);
+		else
+			type = source->type;
+
+		if (type == BRASERO_TRACK_SOURCE_AUDIO || type == BRASERO_TRACK_SOURCE_INF)
+			is_audio = TRUE;
+
+		BRASERO_JOB_TASK_SET_ACTION (process,
+					     BRASERO_BURN_ACTION_WRITING_CD_TEXT,
+					     is_audio ? NULL:_("Writing cue sheet"),
+					     FALSE);
 	}
 	else if (g_str_has_prefix (line, "Re-load disk and hit <CR>")
-	     ||  g_str_has_prefix (line, "send SIGUSR1 to continue")) {
+	      ||  g_str_has_prefix (line, "send SIGUSR1 to continue")) {
+		BraseroBurnAction action = BRASERO_BURN_ACTION_NONE;
+
+		BRASERO_JOB_TASK_GET_ACTION (process, &action);
+		 
 		/* NOTE: There seems to be a BUG somewhere when writing raw images
 		 * with clone mode. After disc has been written and fixated cdrecord
 		 * asks the media to be reloaded. So we simply ignore this message
 		 * and returns that everything went well. Which is indeed the case */
-		 if (brasero_job_get_current_action (BRASERO_JOB (process)) == BRASERO_BURN_ACTION_FIXATING) {
+		 if (action == BRASERO_BURN_ACTION_FIXATING) {
 			brasero_job_finished (BRASERO_JOB (process));
 			return BRASERO_BURN_OK;
 		 }
@@ -523,22 +527,19 @@ brasero_cdrecord_stdout_read (BraseroProcess *process, const char *line)
 						BRASERO_BURN_ERROR_RELOAD_MEDIA,
 						_("The media needs to be reloaded before being recorded")));
 	}
-	else if (g_str_has_prefix (line, "Fixating...")) {
-		brasero_job_progress_changed (BRASERO_JOB (process), 
-					      0.98,
-					      -1);
-		brasero_job_action_changed (BRASERO_JOB (process),
-					    BRASERO_BURN_ACTION_FIXATING,
-					    FALSE);
+	else if (g_str_has_prefix (line, "Fixating...")
+	      ||  g_str_has_prefix (line, "Writing Leadout...")) {
+		BRASERO_JOB_TASK_SET_ACTION (process,
+					     BRASERO_BURN_ACTION_FIXATING,
+					     NULL,
+					     FALSE);
 	}
 	else if (g_str_has_prefix (line, "Last chance to quit, ")) {
 		brasero_job_set_dangerous (BRASERO_JOB (process), TRUE);
 	}
 	else if (g_str_has_prefix (line, "Blanking PMA, TOC, pregap")
 	      ||  strstr (line, "Blanking entire disk")) {
-		brasero_job_action_changed (BRASERO_JOB (process),
-					    BRASERO_BURN_ACTION_BLANKING,
-					    FALSE);
+		BRASERO_JOB_TASK_START_PROGRESS (cdrecord, FALSE);
 	}
 	else if (strstr (line, "Use tsize= option in SAO mode to specify track size")) {
 		brasero_job_error (BRASERO_JOB (process),
@@ -550,28 +551,287 @@ brasero_cdrecord_stdout_read (BraseroProcess *process, const char *line)
 	return BRASERO_BURN_OK;
 }
 
+static gboolean
+brasero_cdrecord_write_inf (BraseroCDRecord *cdrecord,
+			    GPtrArray *argv,
+			    const gchar *album,
+			    gint index,
+			    gint start,
+			    BraseroSongInfo *info,
+			    GError **error)
+{
+	gint fd;
+	gint size;
+	gchar *path;
+	gchar *string;
+	gint b_written;
+	gchar buffer [128];
+
+	/* NOTE: about the .inf files: they should have the exact same path
+	 * but the ending suffix file is replaced by inf:
+	 * example : /path/to/file.mp3 => /path/to/file.inf */
+	if (info->path) {
+		gchar *dot, *separator;
+
+		dot = strrchr (info->path, '.');
+		separator = strrchr (info->path, G_DIR_SEPARATOR);
+
+		if (dot && dot > separator)
+			path = g_strdup_printf ("%.*s.inf",
+						dot - info->path,
+						info->path);
+		else
+			path = g_strdup_printf ("%s.inf",
+						info->path);
+	}
+	else
+		path = g_strdup_printf ("%s/Track%02i.inf",
+					cdrecord->priv->tmpdir,
+					index);
+
+	fd = open (path, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
+	if (fd < 0)
+		goto error;
+
+	BRASERO_JOB_LOG (cdrecord, "writing inf (%s)", path);
+
+	strcpy (buffer, "# created by brasero\n");
+	size = strlen (buffer);
+	b_written = write (fd, buffer, size);
+	if (b_written != size)
+		goto error;
+
+	strcpy (buffer, "MCN=\t\n");
+	size = strlen (buffer);
+	b_written = write (fd, buffer, size);
+	if (b_written != size)
+		goto error;
+
+	if (info->isrc > 0)
+		string = g_strdup_printf ("ISRC=\t%i\n", info->isrc);
+	else
+		string = g_strdup ("ISRC=\t\n");
+	size = strlen (string);
+	b_written = write (fd, string, size);
+	g_free (string);
+	if (b_written != size)
+		goto error;
+
+	strcpy (buffer, "Albumperformer=\t\n");
+	size = strlen (buffer);
+	b_written = write (fd, buffer, size);
+	if (b_written != size)
+		goto error;
+
+	if (album)
+		string = g_strdup_printf ("Albumtitle=\t%s\n", album);
+	else
+		string = strdup ("Albumtitle=\t\n");
+	size = strlen (string);
+	b_written = write (fd, string, size);
+	g_free (string);
+	if (b_written != size)
+		goto error;
+
+	if (info->artist)
+		string = g_strdup_printf ("Performer=\t%s\n", info->artist);
+	else
+		string = strdup ("Performer=\t\n");
+	size = strlen (string);
+	b_written = write (fd, string, size);
+	g_free (string);
+	if (b_written != size)
+		goto error;
+
+	if (info->composer)
+		string = g_strdup_printf ("Composer=\t%s\n", info->composer);
+	else
+		string = strdup ("Composer=\t\n");
+	size = strlen (string);
+	b_written = write (fd, string, size);
+	g_free (string);
+	if (b_written != size)
+		goto error;
+
+	if (info->title)
+		string = g_strdup_printf ("Tracktitle=\t%s\n", info->title);
+	else
+		string = strdup ("Tracktitle=\t\n");
+	size = strlen (string);
+	b_written = write (fd, string, size);
+	g_free (string);
+	if (b_written != size)
+		goto error;
+
+	string = g_strdup_printf ("Tracknumber=\t%i\n", index);
+	size = strlen (string);
+	b_written = write (fd, string, size);
+	g_free (string);
+	if (b_written != size)
+		goto error;
+
+	string = g_strdup_printf ("Trackstart=\t%i\n", start);
+	size = strlen (string);
+	b_written = write (fd, string, size);
+	g_free (string);
+	if (b_written != size)
+		goto error;
+
+	string = g_strdup_printf ("Tracklength=\t%i, 0\n", info->sectors);
+	size = strlen (string);
+	b_written = write (fd, string, size);
+	g_free (string);
+	if (b_written != size)
+		goto error;
+
+	strcpy (buffer, "Pre-emphasis=\tyes\n");
+	size = strlen (buffer);
+	b_written = write (fd, buffer, size);
+	if (b_written != size)
+		goto error;
+
+	strcpy (buffer, "Channels=\t2\n");
+	size = strlen (buffer);
+	b_written = write (fd, buffer, size);
+	if (b_written != size)
+		goto error;
+
+	strcpy (buffer, "Copy_permitted=\tyes\n");
+	size = strlen (buffer);
+	b_written = write (fd, buffer, size);
+	if (b_written != size)
+		goto error;
+
+	strcpy (buffer, "Endianess=\tlittle\n");
+	size = strlen (buffer);
+	b_written = write (fd, buffer, size);
+	if (b_written != size)
+		goto error;
+
+	strcpy (buffer, "Index=\t\t0\n");
+	size = strlen (buffer);
+	b_written = write (fd, buffer, size);
+	if (b_written != size)
+		goto error;
+
+	string = g_strdup_printf ("Index0=\t\t%i\n", info->sectors);
+	size = strlen (string);
+	b_written = write (fd, string, size);
+	if (b_written != size)
+		goto error;
+
+	close (fd);
+
+	cdrecord->priv->infs = g_slist_prepend (cdrecord->priv->infs,
+						g_strdup (path));
+
+	if (info->path) {
+		g_ptr_array_add (argv, g_strdup (info->path));
+		g_free (path);
+	}
+	else
+		g_ptr_array_add (argv, path);
+
+	return BRASERO_BURN_OK;
+
+
+error:
+	g_remove (path);
+	g_free (path);
+
+	g_set_error (error,
+		     BRASERO_BURN_ERROR,
+		     BRASERO_BURN_ERROR_GENERAL,
+		     _("the inf file can't be written : %s"), 
+		     strerror (errno));
+
+	return BRASERO_BURN_ERR;
+}
+
+static BraseroBurnResult
+brasero_cdrecord_write_infs (BraseroCDRecord *cdrecord,
+			     GPtrArray *argv,
+			     const BraseroTrackSource *infs,
+			     GError **error)
+{
+	GSList *iter;
+	gchar *album;
+	gint index;
+	gint start;
+
+	if (cdrecord->priv->infs) {
+		g_slist_foreach (cdrecord->priv->infs, (GFunc) g_remove, NULL);
+
+		g_slist_foreach (cdrecord->priv->infs, (GFunc) g_free, NULL);
+		g_slist_free (cdrecord->priv->infs);
+		cdrecord->priv->infs = NULL;
+	}
+
+	if (!cdrecord->priv->tmpdir) {
+		cdrecord->priv->tmpdir = g_strdup_printf ("%s/" BRASERO_BURN_TMP_FILE_NAME,
+							  g_get_tmp_dir ());
+
+		if (!mkdtemp (cdrecord->priv->tmpdir)) {
+			g_set_error (error,
+				     BRASERO_BURN_ERROR,
+				     BRASERO_BURN_ERROR_GENERAL,
+				     _("a temporary directory couldn't be created"));
+			return BRASERO_BURN_ERR;
+		}
+	}
+
+	album = infs->contents.audio.album;
+	index = 0;
+	start = 0;
+
+	for (iter = infs->contents.audio.infos; iter; iter = iter->next) {
+		BraseroSongInfo *info;
+		BraseroBurnResult result;
+
+		info = iter->data;
+		result = brasero_cdrecord_write_inf (cdrecord,
+						     argv,
+						     album,
+						     index,
+						     start,
+						     info,
+						     error);
+
+		if (result != BRASERO_BURN_OK)
+			return result;
+
+		index ++;
+		start += info->sectors;
+	}
+
+	cdrecord->priv->infs = g_slist_reverse (cdrecord->priv->infs);
+	return BRASERO_BURN_OK;
+}
+
 static BraseroBurnResult
 brasero_cdrecord_set_argv_record (BraseroCDRecord *cdrecord,
 				  GPtrArray *argv, 
 				  GError **error)
 {
+	BraseroTrackSource *source;
+
 	if (!cdrecord->priv->track)
-		return BRASERO_BURN_NOT_READY;
+		BRASERO_JOB_NOT_READY (cdrecord);
+
+	source = cdrecord->priv->track;
 
 	if (cdrecord->priv->immediate) {
 		g_ptr_array_add (argv, g_strdup ("-immed"));
 		g_ptr_array_add (argv, g_strdup_printf ("minbuf=%i", cdrecord->priv->minbuf));
 	}
 
-        if (cdrecord->priv->speed > 0) {
-		char *speed_str;
+        if (cdrecord->priv->rate > 0) {
+		gchar *speed_str;
 
-		speed_str = g_strdup_printf ("speed=%d", cdrecord->priv->speed);
+		speed_str = g_strdup_printf ("speed=%d", cdrecord->priv->rate);
 		g_ptr_array_add (argv, speed_str);
 	}
 
-	if (cdrecord->priv->dao)
-		g_ptr_array_add (argv, g_strdup ("-dao"));
 	if (cdrecord->priv->overburn)
 		g_ptr_array_add (argv, g_strdup ("-overburn"));
 	if (cdrecord->priv->burnproof)
@@ -579,8 +839,9 @@ brasero_cdrecord_set_argv_record (BraseroCDRecord *cdrecord,
 	if (cdrecord->priv->multi)
 		g_ptr_array_add (argv, g_strdup ("-multi"));
 
-	if (cdrecord->priv->track->type == BRASERO_TRACK_SOURCE_IMAGER) {
+	if (source->type == BRASERO_TRACK_SOURCE_IMAGER) {
 		BraseroTrackSourceType track_type;
+		BraseroImageFormat format;
 		BraseroBurnResult result;
 		BraseroImager *imager;
 		int buffer_size;
@@ -589,7 +850,9 @@ brasero_cdrecord_set_argv_record (BraseroCDRecord *cdrecord,
 		imager = cdrecord->priv->track->contents.imager.obj;
 
 		/* we need to know what is the type of the track (audio / data) */
-		result = brasero_imager_get_track_type (imager, &track_type);
+		result = brasero_imager_get_track_type (imager,
+							&track_type,
+							&format);
 		if (result != BRASERO_BURN_OK) {
 			g_set_error (error,
 				     BRASERO_BURN_ERROR,
@@ -619,161 +882,205 @@ brasero_cdrecord_set_argv_record (BraseroCDRecord *cdrecord,
 
 		g_ptr_array_add (argv, g_strdup_printf ("fs=%im", buffer_size));
 
-		if (track_type == BRASERO_TRACK_SOURCE_ISO
-		||  track_type == BRASERO_TRACK_SOURCE_ISO_JOLIET) {
-			g_ptr_array_add (argv, g_strdup_printf ("tsize=%Lis", sectors));
+		brasero_job_set_slave (BRASERO_JOB (cdrecord), BRASERO_JOB (imager));
+		brasero_job_set_run_slave (BRASERO_JOB (cdrecord), TRUE);
 
-			g_ptr_array_add (argv, g_strdup ("-data"));
-			g_ptr_array_add (argv, g_strdup ("-nopad"));
-			g_ptr_array_add (argv, g_strdup ("-"));
+		if (track_type == BRASERO_TRACK_SOURCE_IMAGE) {
+			if ((format & BRASERO_IMAGE_FORMAT_ISO)) {
+				g_ptr_array_add (argv, g_strdup_printf ("tsize=%Lis", sectors));
 
-			brasero_job_set_slave (BRASERO_JOB (cdrecord), BRASERO_JOB (imager));
-			brasero_job_set_relay_slave_signals (BRASERO_JOB (cdrecord), FALSE);
-		}
-		else if (track_type == BRASERO_TRACK_SOURCE_RAW) {
-			g_ptr_array_add (argv, g_strdup ("fs=16m"));
-			g_ptr_array_add (argv, g_strdup ("-raw96r"));
-			g_ptr_array_add (argv, g_strdup ("-clone"));
+				if (cdrecord->priv->dao)
+					g_ptr_array_add (argv, g_strdup ("-dao"));
 
-			/* we need to generate the toc first */
-			if (result != BRASERO_BURN_OK)
-				return result;
+				g_ptr_array_add (argv, g_strdup ("-data"));
+				g_ptr_array_add (argv, g_strdup ("-nopad"));
+				g_ptr_array_add (argv, g_strdup ("-"));
+			}
+#if 0
+			else if ((format & BRASERO_IMAGE_FORMAT_CLONE)) {
+				g_ptr_array_add (argv, g_strdup_printf ("tsize=%Lis", sectors));
 
-			brasero_job_set_slave (BRASERO_JOB (cdrecord), BRASERO_JOB (imager));
-			brasero_job_set_relay_slave_signals (BRASERO_JOB (cdrecord), FALSE);
+				g_ptr_array_add (argv, g_strdup ("-raw96r"));
+				g_ptr_array_add (argv, g_strdup ("-clone"));
+				g_ptr_array_add (argv, g_strdup ("-"));
+
+				/* we need to generate the toc first */
+
+				if (result != BRASERO_BURN_OK)
+					return result;
+			} 
+#endif
+			else
+				BRASERO_JOB_NOT_SUPPORTED (cdrecord);;
 		}
 		else if (track_type == BRASERO_TRACK_SOURCE_AUDIO) {
-			GSList *iter;
+			BraseroTrackSource *infs;
 
 			/* we need to get the inf first */
 			result = brasero_imager_set_output_type (imager,
 								 BRASERO_TRACK_SOURCE_INF,
+								 BRASERO_IMAGE_FORMAT_NONE,
 								 error);
 			if (result != BRASERO_BURN_OK)
 				return result;
 
-			brasero_job_set_slave (BRASERO_JOB (cdrecord), BRASERO_JOB (imager));
 			brasero_job_set_relay_slave_signals (BRASERO_JOB (cdrecord), TRUE);
-
 			result = brasero_imager_get_track (imager,
-							   &cdrecord->priv->inf,
+							   &infs,
 							   error);
 
-			brasero_job_set_relay_slave_signals (BRASERO_JOB (cdrecord), FALSE);
-
 			if (result != BRASERO_BURN_OK)
 				return result;
 	
 			result = brasero_imager_set_output_type (imager,
 								 BRASERO_TRACK_SOURCE_AUDIO,
+								 BRASERO_IMAGE_FORMAT_NONE,
 								 error);
 			if (result != BRASERO_BURN_OK)
 				return result;
 
 			/* now we set the rate of the slave slightly above the speed */
 			brasero_job_set_rate (BRASERO_JOB (imager),
-					      (cdrecord->priv->speed + 1) * CDR_SPEED);
+					      (cdrecord->priv->rate + 1) * CDR_SPEED);
 
 			/* now set the rest of the arguments */
 			g_ptr_array_add (argv, g_strdup ("-dao"));
+			g_ptr_array_add (argv, g_strdup ("-swab"));
 			g_ptr_array_add (argv, g_strdup ("-audio"));
 			g_ptr_array_add (argv, g_strdup ("-useinfo"));
 			g_ptr_array_add (argv, g_strdup ("-text"));
 
-			for (iter = cdrecord->priv->inf->contents.inf.files; iter; iter = iter->next) {
-				gchar *arg;
-	
-				arg = g_strdup (iter->data);
-				g_ptr_array_add (argv, arg);
-			}
+			result = brasero_cdrecord_write_infs (cdrecord,
+							      argv,
+							      infs,
+							      error);
+			brasero_track_source_free (infs);
+
+			if (result != BRASERO_BURN_OK)
+				return result;
 		}
 		else
-			return BRASERO_BURN_NOT_SUPPORTED;
+			BRASERO_JOB_NOT_SUPPORTED (cdrecord);
 
-		brasero_job_set_run_slave (BRASERO_JOB (cdrecord), TRUE);
+		brasero_job_set_relay_slave_signals (BRASERO_JOB (cdrecord), FALSE);
 	}
-	else if (cdrecord->priv->track->type == BRASERO_TRACK_SOURCE_AUDIO) {
-		GSList *iter;
+	else if (source->type == BRASERO_TRACK_SOURCE_AUDIO) {
+		BraseroBurnResult result;
 
 		/* CD-text cannot be written in tao mode (which is the default) */
-		g_ptr_array_add (argv, g_strdup ("-dao"));
+		if (cdrecord->priv->dao)
+			g_ptr_array_add (argv, g_strdup ("-dao"));
 
 		g_ptr_array_add (argv, g_strdup ("fs=16m"));
 		g_ptr_array_add (argv, g_strdup ("-audio"));
+		g_ptr_array_add (argv, g_strdup ("-swab"));
 		g_ptr_array_add (argv, g_strdup ("-pad"));
+	
 		g_ptr_array_add (argv, g_strdup ("-useinfo"));
 		g_ptr_array_add (argv, g_strdup ("-text"));
 
-		for (iter = cdrecord->priv->track->contents.audio.files; iter; iter = iter->next) {
-			gchar *arg;
+		result = brasero_cdrecord_write_infs (cdrecord,
+						      argv,
+						      source,
+						      error);
 
-			arg = g_strdup (iter->data);
-			g_ptr_array_add (argv, arg);
+		if (result != BRASERO_BURN_OK)
+			return result;
+
+		brasero_job_set_run_slave (BRASERO_JOB (cdrecord), FALSE);
+	}
+	else if (source->type == BRASERO_TRACK_SOURCE_IMAGE) {
+		if (source->format == BRASERO_IMAGE_FORMAT_NONE) {
+			gchar *image_path;
+
+			image_path = brasero_track_source_get_image_localpath (cdrecord->priv->track);
+			if (!image_path)
+				return BRASERO_BURN_ERR;
+
+			if (cdrecord->priv->dao)
+				g_ptr_array_add (argv, g_strdup ("-dao"));
+
+			g_ptr_array_add (argv, g_strdup ("fs=16m"));
+			g_ptr_array_add (argv, g_strdup ("-data"));
+			g_ptr_array_add (argv, g_strdup ("-nopad"));
+			g_ptr_array_add (argv, image_path);
+
+			brasero_job_set_run_slave (BRASERO_JOB (cdrecord), FALSE);
 		}
+		else if (source->format & BRASERO_IMAGE_FORMAT_ISO) {
+			gchar *isopath;
 
-		brasero_job_set_run_slave (BRASERO_JOB (cdrecord), FALSE);
-	}
-	else if (cdrecord->priv->track->type == BRASERO_TRACK_SOURCE_ISO
-	      ||  cdrecord->priv->track->type == BRASERO_TRACK_SOURCE_ISO_JOLIET) {
-		gchar *isopath;
+			isopath = brasero_track_source_get_image_localpath (cdrecord->priv->track);
+			if (!isopath)
+				return BRASERO_BURN_ERR;
 
-		isopath = brasero_track_source_get_iso_localpath (cdrecord->priv->track);
-		if (!isopath)
-			return BRASERO_BURN_ERR;
+			if (cdrecord->priv->dao)
+				g_ptr_array_add (argv, g_strdup ("-dao"));
 
-		g_ptr_array_add (argv, g_strdup ("fs=16m"));
-		g_ptr_array_add (argv, g_strdup ("-data"));
-		g_ptr_array_add (argv, g_strdup ("-nopad"));
-		g_ptr_array_add (argv, isopath);
+			g_ptr_array_add (argv, g_strdup ("fs=16m"));
+			g_ptr_array_add (argv, g_strdup ("-data"));
+			g_ptr_array_add (argv, g_strdup ("-nopad"));
+			g_ptr_array_add (argv, isopath);
 
-		brasero_job_set_run_slave (BRASERO_JOB (cdrecord), FALSE);
-	}
-	else if (cdrecord->priv->track->type == BRASERO_TRACK_SOURCE_RAW) {
-		gchar *rawpath;
+			brasero_job_set_run_slave (BRASERO_JOB (cdrecord), FALSE);
+		}
+		else if (source->format & BRASERO_IMAGE_FORMAT_CLONE) {
+			gchar *rawpath;
 
-		rawpath = brasero_track_source_get_raw_localpath (cdrecord->priv->track);
-		if (!rawpath)
-			return BRASERO_BURN_ERR;
+			rawpath = brasero_track_source_get_raw_localpath (cdrecord->priv->track);
+			if (!rawpath)
+				return BRASERO_BURN_ERR;
 
-		g_ptr_array_add (argv, g_strdup ("fs=16m"));
-		g_ptr_array_add (argv, g_strdup ("-raw96r"));
-		g_ptr_array_add (argv, g_strdup ("-clone"));
-		g_ptr_array_add (argv, rawpath);
+			g_ptr_array_add (argv, g_strdup ("fs=16m"));
+			g_ptr_array_add (argv, g_strdup ("-raw96r"));
+			g_ptr_array_add (argv, g_strdup ("-clone"));
+			g_ptr_array_add (argv, rawpath);
 
-		brasero_job_set_run_slave (BRASERO_JOB (cdrecord), FALSE);
-	}
-	else if (cdrecord->priv->track->type == BRASERO_TRACK_SOURCE_CUE) {
-		gchar *cue_str;
-		gchar *cuepath;
+			brasero_job_set_run_slave (BRASERO_JOB (cdrecord), FALSE);
+		}
+		else if (source->format & BRASERO_IMAGE_FORMAT_CUE) {
+			gchar *cue_str;
+			gchar *cuepath;
 
-		g_ptr_array_add (argv, g_strdup ("fs=16m"));
-		g_ptr_array_add (argv, g_strdup ("-dao"));
+			if (cdrecord->priv->dao)
+				g_ptr_array_add (argv, g_strdup ("-dao"));
 
-		cuepath = brasero_track_source_get_cue_localpath (cdrecord->priv->track);
-		if (!cuepath)
-			return BRASERO_BURN_ERR;
+			g_ptr_array_add (argv, g_strdup ("fs=16m"));
 
-		cue_str = g_strdup_printf ("cuefile=%s", cuepath);
-		g_free (cuepath);
+			cuepath = brasero_track_source_get_cue_localpath (cdrecord->priv->track);
+			if (!cuepath)
+				return BRASERO_BURN_ERR;
 
-		g_ptr_array_add (argv, cue_str);
+			cue_str = g_strdup_printf ("cuefile=%s", cuepath);
+			g_free (cuepath);
 
-		brasero_job_set_run_slave (BRASERO_JOB (cdrecord), FALSE);
+			g_ptr_array_add (argv, cue_str);
+
+			brasero_job_set_run_slave (BRASERO_JOB (cdrecord), FALSE);
+		}
 	}
 	else
-		return BRASERO_BURN_NOT_SUPPORTED;
+		BRASERO_JOB_NOT_SUPPORTED (cdrecord);
 
+	BRASERO_JOB_TASK_SET_ACTION (cdrecord,
+				     BRASERO_BURN_ACTION_PREPARING,
+				     NULL,
+				     FALSE);
 	return BRASERO_BURN_OK;
 }
 
 static BraseroBurnResult
 brasero_cdrecord_set_argv_blank (BraseroCDRecord *cdrecord, GPtrArray *argv)
 {
-	char *blank_str;
+	gchar *blank_str;
 
 	blank_str = g_strdup_printf ("blank=%s", cdrecord->priv->blank_fast ? "fast" : "all");
 	g_ptr_array_add (argv, blank_str);
 
+	BRASERO_JOB_TASK_SET_ACTION (cdrecord,
+				     BRASERO_BURN_ACTION_ERASING,
+				     NULL,
+				     FALSE);
 	return BRASERO_BURN_OK;
 }
 
@@ -785,15 +1092,15 @@ brasero_cdrecord_set_argv (BraseroProcess *process,
 {
 	BraseroCDRecord *cdrecord;
 	BraseroBurnResult result;
-	char *dev_str;
+	gchar *dev_str;
 
 	cdrecord = BRASERO_CD_RECORD (process);
 
 	if (has_master)
-		return BRASERO_BURN_NOT_SUPPORTED;
+		BRASERO_JOB_NOT_SUPPORTED (cdrecord);
 
 	if (!cdrecord->priv->drive)
-		return BRASERO_BURN_NOT_READY;
+		BRASERO_JOB_NOT_READY (cdrecord);
 
 	g_ptr_array_add (argv, g_strdup ("cdrecord"));
 	g_ptr_array_add (argv, g_strdup ("-v"));
@@ -814,18 +1121,24 @@ brasero_cdrecord_set_argv (BraseroProcess *process,
 	else if (cdrecord->priv->action == BRASERO_CD_RECORD_ACTION_BLANK)
 		result = brasero_cdrecord_set_argv_blank (cdrecord, argv);
 	else
-		return BRASERO_BURN_NOT_READY;
-
-	if (result == BRASERO_BURN_OK || result == BRASERO_BURN_CANCEL) {
-		brasero_job_action_changed (BRASERO_JOB (cdrecord),
-					    BRASERO_BURN_ACTION_PREPARING,
-					    FALSE);
-		brasero_job_progress_changed (BRASERO_JOB (cdrecord), 0, -1);
-	}
+		BRASERO_JOB_NOT_READY (cdrecord);
 
 	return result;	
 }
 
+static BraseroBurnResult
+brasero_cdrecord_post (BraseroProcess *process,
+		       BraseroBurnResult retval)
+{
+	BraseroCDRecord *cdrecord;
+
+	cdrecord = BRASERO_CD_RECORD (process);
+
+	brasero_cdrecord_stop_real (cdrecord);
+
+	return BRASERO_BURN_OK;
+}
+
 static BraseroBurnResult
 brasero_cdrecord_set_drive (BraseroRecorder *recorder,
 			    NautilusBurnDrive *drive,
@@ -834,11 +1147,11 @@ brasero_cdrecord_set_drive (BraseroRecorder *recorder,
 	BraseroCDRecord *cdrecord;
 	NautilusBurnMediaType media;
 
+	cdrecord = BRASERO_CD_RECORD (recorder);
+
 	media = nautilus_burn_drive_get_media_type (drive);
 	if (media > NAUTILUS_BURN_MEDIA_TYPE_CDRW)
-		return BRASERO_BURN_NOT_SUPPORTED;
-
-	cdrecord = BRASERO_CD_RECORD (recorder);
+		BRASERO_JOB_NOT_SUPPORTED (cdrecord);
 
 	if (cdrecord->priv->drive) {
 		nautilus_burn_drive_unref (cdrecord->priv->drive);
@@ -860,17 +1173,21 @@ brasero_cdrecord_set_source (BraseroJob *job,
 
 	cdrecord = BRASERO_CD_RECORD (job);
 
-	if (track->type != BRASERO_TRACK_SOURCE_AUDIO
-	&&  track->type != BRASERO_TRACK_SOURCE_CUE
-	&&  track->type != BRASERO_TRACK_SOURCE_RAW
-	&&  track->type != BRASERO_TRACK_SOURCE_ISO
-	&&  track->type != BRASERO_TRACK_SOURCE_ISO_JOLIET
-	&&  track->type != BRASERO_TRACK_SOURCE_IMAGER)
-		return BRASERO_BURN_NOT_SUPPORTED;
+	if (track->type != BRASERO_TRACK_SOURCE_IMAGE
+	&&  track->type != BRASERO_TRACK_SOURCE_IMAGER
+	&&  track->type != BRASERO_TRACK_SOURCE_AUDIO
+	&&  track->type != BRASERO_TRACK_SOURCE_INF)
+		BRASERO_JOB_NOT_SUPPORTED (cdrecord);
 
-	if (cdrecord->priv->inf) {
-		brasero_track_source_free (cdrecord->priv->inf);
-		cdrecord->priv->inf = NULL;
+	if (cdrecord->priv->infs) {
+		g_slist_foreach (cdrecord->priv->infs,
+				 (GFunc) g_remove,
+				 NULL);
+		g_slist_foreach (cdrecord->priv->infs,
+				 (GFunc) g_free,
+				 NULL);
+		g_slist_free (cdrecord->priv->infs);
+		cdrecord->priv->infs = NULL;
 	}
 
 	if (cdrecord->priv->track)
@@ -911,7 +1228,7 @@ brasero_cdrecord_set_rate (BraseroJob *job,
 		return BRASERO_BURN_RUNNING;
 
 	cdrecord = BRASERO_CD_RECORD (job);
-	cdrecord->priv->speed = speed / CDR_SPEED;
+	cdrecord->priv->rate = speed / CDR_SPEED;
 	return BRASERO_BURN_OK;
 }
 
@@ -936,10 +1253,6 @@ brasero_cdrecord_blank (BraseroRecorder *recorder,
 	result = brasero_job_run (BRASERO_JOB (cdrecord), error);
 	cdrecord->priv->action = BRASERO_CD_RECORD_ACTION_NONE;
 
-	brasero_job_action_changed (BRASERO_JOB (cdrecord),
-				    BRASERO_BURN_ACTION_NONE,
-				    FALSE);
-
 	return result;	
 }
 
@@ -953,82 +1266,15 @@ brasero_cdrecord_record (BraseroRecorder *recorder,
 	cdrecord = BRASERO_CD_RECORD (recorder);
 
 	if (!cdrecord->priv->track)
-		return BRASERO_BURN_NOT_READY;
+		BRASERO_JOB_NOT_READY (cdrecord);
 
 	cdrecord->priv->action = BRASERO_CD_RECORD_ACTION_RECORD;
-
-	/* set as slave if track is an imager (on the fly burning) */
-	if (cdrecord->priv->track->type == BRASERO_TRACK_SOURCE_IMAGER) {
-		BraseroJob *slave;
-
-		slave = BRASERO_JOB (cdrecord->priv->track->contents.imager.obj);
-		brasero_job_set_slave (BRASERO_JOB (cdrecord), slave);
-		brasero_job_set_relay_slave_signals (BRASERO_JOB (cdrecord), FALSE);
-		brasero_job_set_run_slave (BRASERO_JOB (cdrecord), TRUE);
-	}
-	else
-		brasero_job_set_run_slave (BRASERO_JOB (cdrecord), FALSE);
-
 	result = brasero_job_run (BRASERO_JOB (cdrecord), error);
-	brasero_job_set_slave (BRASERO_JOB (cdrecord), NULL);
 	cdrecord->priv->action = BRASERO_CD_RECORD_ACTION_NONE;
+
 	return result;				
 }
 
-static BraseroBurnResult
-brasero_cdrecord_get_rate (BraseroJob *job,
-			   gint64 *rate)
-{
-	BraseroCDRecord *cdrecord;
-
-	cdrecord = BRASERO_CD_RECORD (job);
-
-	if (rate)
-		*rate = cdrecord->priv->cur_speed;
-
-	return BRASERO_BURN_OK;
-}
-
-static BraseroBurnResult
-brasero_cdrecord_get_written (BraseroJob *job,
-			      gint64 *written)
-{
-	BraseroCDRecord *cdrecord;
-
-	cdrecord = BRASERO_CD_RECORD (job);
-
-	if (written)
-		*written = cdrecord->priv->b_written;
-
-	return BRASERO_BURN_OK;
-}
-
-static BraseroBurnResult
-brasero_cdrecord_get_fifo (BraseroRecorder *recorder,
-			   gint *fifo)
-{
-	BraseroCDRecord *cdrecord;
-
-	cdrecord = BRASERO_CD_RECORD (recorder);
-
-	if (fifo)
-		*fifo = cdrecord->priv->fifo;
-
-	return BRASERO_BURN_OK;
-}
-
-static BraseroBurnResult
-brasero_cdrecord_get_action_string (BraseroJob *job,
-				    BraseroBurnAction action,
-				    char **string)
-{
-	job = brasero_job_get_slave (job);
-	if (!job)
-		return BRASERO_BURN_NOT_SUPPORTED;
-
-	return brasero_job_get_action_string (job, action, string);
-}
-
 void
 brasero_cdrecord_set_immediate (BraseroCDRecord *cdrecord, gint minbuf)
 {
