@@ -60,6 +60,7 @@ brasero_growisofs_set_source (BraseroJob *job,
 static BraseroBurnResult
 brasero_growisofs_set_output_type (BraseroImager *imager,
 				   BraseroTrackSourceType type,
+				   BraseroImageFormat format,
 				   GError **error);
 static BraseroBurnResult
 brasero_growisofs_set_append (BraseroImager *imager,
@@ -84,9 +85,6 @@ brasero_growisofs_set_argv (BraseroProcess *process,
 			    GPtrArray *array,
 			    gboolean has_master,
 			    GError **error);
-static BraseroBurnResult
-brasero_growisofs_post (BraseroProcess *process,
-			BraseroBurnResult retval);
 			
 /* Recording part */
 static BraseroBurnResult
@@ -99,7 +97,7 @@ brasero_growisofs_set_flags (BraseroRecorder *recorder,
 			     GError **error);
 static BraseroBurnResult
 brasero_growisofs_set_rate (BraseroJob *job,
-			     gint64 speed);
+			     gint64 rate);
 
 static BraseroBurnResult
 brasero_growisofs_record (BraseroRecorder *recorder,
@@ -108,21 +106,6 @@ static BraseroBurnResult
 brasero_growisofs_blank (BraseroRecorder *recorder,
 			 GError **error);
 
-static BraseroBurnResult
-brasero_growisofs_get_rate (BraseroJob *job,
-			    gint64 *rate);
-static BraseroBurnResult
-brasero_growisofs_get_written (BraseroJob *job,
-			       gint64 *written);
-static BraseroBurnResult
-brasero_growisofs_get_fifo (BraseroRecorder *recorder,
-			    gint *fifo);
-
-static BraseroBurnResult
-brasero_growisofs_get_action_string (BraseroJob *job,
-				     BraseroBurnAction action,
-				     char **string);
-
 typedef enum {
 	BRASERO_GROWISOFS_ACTION_NONE,
 	BRASERO_GROWISOFS_ACTION_RECORD,
@@ -132,17 +115,11 @@ typedef enum {
 
 struct BraseroGrowisofsPrivate {
 	BraseroBurnCaps *caps;
-	BraseroTrackSourceType track_type;
+	BraseroImageFormat image_format;
 	BraseroGrowisofsAction action;
 
 	NautilusBurnDrive *drive;
-	int speed;
-
-	int cur_speed;
-	int fifo;
-	gint64 bytes_written;
-	gdouble elapsed;
-	GTimer *timer;
+	gint rate;
 
 	BraseroTrackSource *source;
 
@@ -214,16 +191,12 @@ brasero_growisofs_class_init (BraseroGrowisofsClass *klass)
 	parent_class = g_type_class_peek_parent(klass);
 	object_class->finalize = brasero_growisofs_finalize;
 
-	job_class->get_action_string = brasero_growisofs_get_action_string;
-	job_class->get_written = brasero_growisofs_get_written;
 	job_class->set_source = brasero_growisofs_set_source;
 	job_class->set_rate = brasero_growisofs_set_rate;
-	job_class->get_rate = brasero_growisofs_get_rate;
 
 	process_class->stdout_func = brasero_growisofs_read_stdout;
 	process_class->stderr_func = brasero_growisofs_read_stderr;
 	process_class->set_argv = brasero_growisofs_set_argv;
-	process_class->post = brasero_growisofs_post;
 }
 
 static void
@@ -239,7 +212,6 @@ brasero_growisofs_iface_init_record (BraseroRecorderIFace *iface)
 {
 	iface->set_drive = brasero_growisofs_set_drive;
 	iface->set_flags = brasero_growisofs_set_flags;
-	iface->get_fifo = brasero_growisofs_get_fifo;
 	iface->record = brasero_growisofs_record;
 	iface->blank = brasero_growisofs_blank;
 }
@@ -305,6 +277,7 @@ brasero_growisofs_new ()
 static BraseroBurnResult
 brasero_growisofs_set_output_type (BraseroImager *imager,
 				   BraseroTrackSourceType type,
+				   BraseroImageFormat format,
 				   GError **error)
 {
 	BraseroGrowisofs *growisofs;
@@ -312,11 +285,13 @@ brasero_growisofs_set_output_type (BraseroImager *imager,
 	growisofs = BRASERO_GROWISOFS (imager);
 
 	if (type != BRASERO_TRACK_SOURCE_DEFAULT
-	&&  type != BRASERO_TRACK_SOURCE_ISO
-	&&  type != BRASERO_TRACK_SOURCE_ISO_JOLIET)
-		return BRASERO_BURN_NOT_SUPPORTED;
+	&&  type != BRASERO_TRACK_SOURCE_IMAGE)
+		BRASERO_JOB_NOT_SUPPORTED (growisofs);
 
-	growisofs->priv->track_type = type;
+	if (!(format & BRASERO_IMAGE_FORMAT_ISO))
+		BRASERO_JOB_NOT_SUPPORTED (growisofs);
+
+	growisofs->priv->image_format = format;
 
 	return BRASERO_BURN_OK;
 }
@@ -330,9 +305,6 @@ brasero_growisofs_get_track (BraseroGrowisofs *growisofs,
 	BraseroBurnCaps *caps;
 	BraseroImager *imager;
 
-	/* we need to download all non local files first
-	* and make a list of excluded and graft points */
-
 	/* ask BurnCaps to create an object to get GRAFTS */
 	caps = brasero_burn_caps_get_default ();
 	result = brasero_burn_caps_create_imager (caps,
@@ -363,6 +335,7 @@ brasero_growisofs_get_track (BraseroGrowisofs *growisofs,
 
 	result = brasero_imager_set_output_type (imager,
 						 BRASERO_TRACK_SOURCE_GRAFTS,
+						 source->format,
 						 error);
 	if (result != BRASERO_BURN_OK)
 		return result;
@@ -400,10 +373,18 @@ brasero_growisofs_set_source (BraseroJob *job,
 
 	if (source->type != BRASERO_TRACK_SOURCE_DATA
 	&&  source->type != BRASERO_TRACK_SOURCE_GRAFTS
-	&&  source->type != BRASERO_TRACK_SOURCE_ISO
-	&&  source->type != BRASERO_TRACK_SOURCE_ISO_JOLIET
-	&&  source->type != BRASERO_TRACK_SOURCE_IMAGER)
-		return BRASERO_BURN_NOT_SUPPORTED;
+	&&  source->type != BRASERO_TRACK_SOURCE_IMAGER
+	&&  source->type != BRASERO_TRACK_SOURCE_IMAGE)
+		BRASERO_JOB_NOT_SUPPORTED (growisofs);
+
+	if (source->type == BRASERO_TRACK_SOURCE_IMAGE) {
+		if (source->format != BRASERO_IMAGE_FORMAT_NONE
+		&& (source->format & BRASERO_IMAGE_FORMAT_ISO) == 0)
+			BRASERO_JOB_NOT_SUPPORTED (growisofs);
+	}
+	else if (source->type != BRASERO_TRACK_SOURCE_IMAGER
+	     && !(source->format & BRASERO_IMAGE_FORMAT_ISO))
+		BRASERO_JOB_NOT_SUPPORTED (growisofs);
 
 	if (source->type == BRASERO_TRACK_SOURCE_DATA)
 		result = brasero_growisofs_get_track (growisofs, source, error);
@@ -452,10 +433,10 @@ brasero_growisofs_get_size (BraseroImager *imager,
 	growisofs = BRASERO_GROWISOFS (imager);
 
 	if (!growisofs->priv->source)
-		return BRASERO_BURN_NOT_READY;
+		BRASERO_JOB_NOT_READY (growisofs);
 
 	if (growisofs->priv->source->type != BRASERO_TRACK_SOURCE_GRAFTS)
-		return BRASERO_BURN_NOT_SUPPORTED;
+		BRASERO_JOB_NOT_SUPPORTED (growisofs);
 
 	if (!growisofs->priv->sectors_num) {
 		if (brasero_job_is_running (BRASERO_JOB (imager)))
@@ -491,22 +472,10 @@ brasero_growisofs_record (BraseroRecorder *recorder,
 	growisofs = BRASERO_GROWISOFS (recorder);
 
 	if (!growisofs->priv->drive)
-		return BRASERO_BURN_NOT_READY;
+		BRASERO_JOB_NOT_READY (growisofs);
 
 	if (!growisofs->priv->source)
-		return BRASERO_BURN_NOT_READY;
-
-	/* set as slave if track is an imager (on the fly burning) */
-	if (growisofs->priv->source->type == BRASERO_TRACK_SOURCE_IMAGER) {
-		BraseroJob *slave;
-
-		slave = BRASERO_JOB (growisofs->priv->source->contents.imager.obj);
-		brasero_job_set_slave (BRASERO_JOB (growisofs), slave);
-		brasero_job_set_relay_slave_signals (BRASERO_JOB (growisofs), FALSE);
-		brasero_job_set_run_slave (BRASERO_JOB (growisofs), TRUE);
-	}
-	else
-		brasero_job_set_run_slave (BRASERO_JOB (growisofs), FALSE);
+		BRASERO_JOB_NOT_READY (growisofs);
 
 	growisofs->priv->action = BRASERO_GROWISOFS_ACTION_RECORD;
 	result = brasero_job_run (BRASERO_JOB (growisofs), error);
@@ -526,17 +495,20 @@ brasero_growisofs_blank (BraseroRecorder *recorder,
 	growisofs = BRASERO_GROWISOFS (recorder);
 
 	if (!growisofs->priv->drive)
-		return BRASERO_BURN_NOT_READY;
+		BRASERO_JOB_NOT_READY (growisofs);
 
 	media = nautilus_burn_drive_get_media_type (growisofs->priv->drive);
 
-	if (media <= NAUTILUS_BURN_MEDIA_TYPE_CDRW)
-		return BRASERO_BURN_NOT_SUPPORTED;
+	if (media != NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_RW)
+		BRASERO_JOB_NOT_SUPPORTED (growisofs);
 
-	/* There is no need to format RW+ in a fast way */
-        if (media == NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_RW
-	&&  growisofs->priv->fast_blank)
-		return BRASERO_BURN_OK;
+	/* There is no way to format RW+ or RW- in restricted overwrite mode in a fast way */
+	/* FIXME: actually there is: just reformat them (with dvd+rw-format -force) */
+	/* FIXME: we should use the same thing for DVD+RW and DVD-RW in restricted
+	 * overwrite mode */
+        if (media != NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_RW
+	||  growisofs->priv->fast_blank)
+		BRASERO_JOB_NOT_SUPPORTED (growisofs);
 
 	/* if we have a slave we don't want it to run */
 	brasero_job_set_run_slave (BRASERO_JOB (recorder), FALSE);
@@ -595,53 +567,7 @@ brasero_growisofs_set_rate (BraseroJob *job,
 		return BRASERO_BURN_RUNNING;
 
 	growisofs = BRASERO_GROWISOFS (job);
-	growisofs->priv->speed = rate / DVD_SPEED;
-
-	return BRASERO_BURN_OK;
-}
-
-static BraseroBurnResult
-brasero_growisofs_get_rate (BraseroJob *job, gint64 *rate)
-{
-	BraseroGrowisofs *growisofs;
-
-	growisofs = BRASERO_GROWISOFS (job);
-	if (growisofs->priv->action != BRASERO_GROWISOFS_ACTION_RECORD)
-		return BRASERO_BURN_NOT_RUNNING;
-
-	if (rate)
-		*rate = growisofs->priv->cur_speed;
-
-	return BRASERO_BURN_OK;
-}
-
-static BraseroBurnResult
-brasero_growisofs_get_written (BraseroJob *job, gint64 *written)
-{
-	BraseroGrowisofs *growisofs;
-
-	growisofs = BRASERO_GROWISOFS (job);
-	if (growisofs->priv->action != BRASERO_GROWISOFS_ACTION_RECORD)
-		return BRASERO_BURN_NOT_RUNNING;
-
-	if (written)
-		*written = growisofs->priv->bytes_written;
-
-	return BRASERO_BURN_OK;
-}
-
-static BraseroBurnResult
-brasero_growisofs_get_fifo (BraseroRecorder *recorder, gint *fifo)
-{
-	BraseroGrowisofs *growisofs;
-
-	growisofs = BRASERO_GROWISOFS (recorder);
-
-	if (growisofs->priv->action != BRASERO_GROWISOFS_ACTION_RECORD)
-		return BRASERO_BURN_NOT_RUNNING;
-
-	if (fifo)
-		*fifo = growisofs->priv->fifo;
+	growisofs->priv->rate = rate / DVD_SPEED;
 
 	return BRASERO_BURN_OK;
 }
@@ -659,38 +585,27 @@ brasero_growisofs_read_stdout (BraseroProcess *process, const char *line)
 
 	if (sscanf (line, "%10lld/%lld ( %2d.%1d%%) @%2d.%1dx, remaining %*d:%*d",
 		    &b_written, &b_total, &perc_1, &perc_2, &speed_1, &speed_2) == 6) {
-		gdouble percent;
-		gdouble speed;
-		glong secs;
+		BRASERO_JOB_TASK_SET_WRITTEN (growisofs, b_written);
+		BRASERO_JOB_TASK_SET_TOTAL (growisofs, b_total);
+		BRASERO_JOB_TASK_SET_RATE (growisofs, (gdouble) (speed_1 * 10 + speed_2) / 10.0 * (gdouble) DVD_SPEED);
 
-		speed = (gdouble) ((gdouble) speed_1 + ((gdouble) speed_2 / (gdouble) 10.0));
-		growisofs->priv->cur_speed = speed * DVD_SPEED;
-		growisofs->priv->bytes_written = b_written;
-
-		if (growisofs->priv->action == BRASERO_GROWISOFS_ACTION_BLANK)
-			brasero_job_action_changed (BRASERO_JOB (process),
-						    BRASERO_BURN_ACTION_BLANKING,
-						    FALSE);
-		else
-			brasero_job_action_changed (BRASERO_JOB (process),
-						    BRASERO_BURN_ACTION_WRITING,
-						    FALSE);
-
-		percent = (gdouble) ((gdouble) perc_1 + ((gdouble) perc_2 / (gdouble) 10.0)) / (gdouble) 100.0;
-		secs = brasero_burn_common_compute_time_remaining (b_total - b_written, (gdouble) speed * (gdouble) DVD_SPEED);
-
-		brasero_job_progress_changed (BRASERO_JOB (process), 
-					      percent * 0.98,
-					      secs);
-	}
-	else if (strstr (line, "About to execute") || strstr (line, "Executing")) {
-		if (growisofs->priv->action != BRASERO_GROWISOFS_ACTION_GET_SIZE) {
-			brasero_job_action_changed (BRASERO_JOB (process),
-						    BRASERO_BURN_ACTION_PREPARING,
-						    FALSE);
-			brasero_job_set_dangerous (BRASERO_JOB (process), TRUE);
+		if (growisofs->priv->action == BRASERO_GROWISOFS_ACTION_BLANK) {
+			BRASERO_JOB_TASK_SET_ACTION (growisofs,
+						     BRASERO_BURN_ACTION_ERASING,
+						     NULL,
+						     FALSE);
 		}
+		else {
+			BRASERO_JOB_TASK_SET_ACTION (growisofs,
+						     BRASERO_BURN_ACTION_WRITING,
+						     NULL,
+						     FALSE);
+		}
+
+		BRASERO_JOB_TASK_START_PROGRESS (growisofs, FALSE);
 	}
+	else if (strstr (line, "About to execute") || strstr (line, "Executing"))
+		brasero_job_set_dangerous (BRASERO_JOB (process), TRUE);
 
 	return BRASERO_BURN_OK;
 }
@@ -704,58 +619,24 @@ brasero_growisofs_read_stderr (BraseroProcess *process, const char *line)
 	growisofs = BRASERO_GROWISOFS (process);
 
 	if (sscanf (line, " %2d.%1d%% done, estimate finish", &perc_1, &perc_2) == 2) {
-		long secs = -1;
-		gdouble percent;
+		gdouble fraction;
+		gint64 written, total;
 
-		percent = (gdouble) ((gdouble) perc_1 + ((gdouble) perc_2 / (gdouble) 10.0)) / (gdouble) 100.0;
+		fraction = (gdouble) ((gdouble) perc_1 +
+			   ((gdouble) perc_2 / (gdouble) 10.0)) /
+			   (gdouble) 100.0;
 
-		if (growisofs->priv->sectors_num) {
-			gulong micro;
-			gint64 b_total;
-			gdouble elapsed;
-			gint64 b_written;
-			gdouble raw_speed;
-			gdouble average_speed;
+		total = growisofs->priv->sectors_num * 2048;
+		written = total * fraction;
 
-			b_total = growisofs->priv->sectors_num * 2048;
-			b_written = b_total * percent;
-
-			/* calculate the elapsed time */
-			elapsed = g_timer_elapsed (growisofs->priv->timer, &micro);
-			g_timer_start (growisofs->priv->timer);
-			elapsed += (gdouble) micro / (gdouble) 1000000.0;
-			growisofs->priv->elapsed += elapsed;
-
-			/* calculate average speed and time remaining */
-			average_speed = (gdouble) b_written / growisofs->priv->elapsed;
-			secs = brasero_burn_common_compute_time_remaining (b_total - b_written, average_speed);
-
-			/* calculate current speed */
-			raw_speed = (gdouble) ((gdouble) b_written - (gdouble) growisofs->priv->bytes_written) / elapsed;
-			growisofs->priv->cur_speed = raw_speed;
-			growisofs->priv->bytes_written = b_written;
-		}
-
-		brasero_job_action_changed (BRASERO_JOB (process),
-					    BRASERO_BURN_ACTION_WRITING,
-					    FALSE);
-		brasero_job_progress_changed (BRASERO_JOB (process), 
-					      percent * 0.98,
-					      secs);
-	}
-	else if (strstr (line, "restarting DVD+RW format")) {
-		brasero_job_action_changed (BRASERO_JOB (process),
-					    BRASERO_BURN_ACTION_BLANKING,
-					    FALSE);
-	}
-	else if (strstr (line, "\"Current Write Speed\" is ")) {
-		brasero_job_action_changed (BRASERO_JOB (process),
-					    BRASERO_BURN_ACTION_PREPARING,
-					    FALSE);
-	}
-	else if (strstr (line, "Writing:   The File(s)")) {
-		growisofs->priv->timer = g_timer_new ();
-		growisofs->priv->elapsed = 0.0;
+		BRASERO_JOB_TASK_SET_TOTAL (growisofs, total);
+		BRASERO_JOB_TASK_SET_WRITTEN (growisofs, written);
+		BRASERO_JOB_TASK_SET_PROGRESS (growisofs, fraction);
+		BRASERO_JOB_TASK_SET_ACTION (growisofs,
+					     BRASERO_BURN_ACTION_WRITING,
+					     NULL,
+					     FALSE);
+		BRASERO_JOB_TASK_START_PROGRESS (growisofs, FALSE);
 	}
 	else if (strstr (line, "Total extents scheduled to be written = ")) {
 		BraseroGrowisofs *growisofs;
@@ -764,19 +645,24 @@ brasero_growisofs_read_stderr (BraseroProcess *process, const char *line)
 
 		line += strlen ("Total extents scheduled to be written = ");
 		growisofs->priv->sectors_num = strtoll (line, NULL, 10);
+		BRASERO_JOB_TASK_SET_TOTAL (growisofs, growisofs->priv->sectors_num * 2048);
 	}
-	else if (strstr (line, "unsupported MMC profile")
-	      || (strstr (line, "already carries isofs") && strstr (line, "FATAL:"))) {
-		/* This is not supposed to happen since we checked for the cd
-		   type before starting, but we try to handle it anyway, since mmc
-		   profiling can fail. */
+	else if (strstr (line, "flushing cache") != NULL) {
+		BRASERO_JOB_TASK_SET_PROGRESS (growisofs, 1.0);
+		BRASERO_JOB_TASK_SET_WRITTEN (growisofs, growisofs->priv->sectors_num * 2048);
+		BRASERO_JOB_TASK_SET_ACTION (growisofs,
+					     BRASERO_BURN_ACTION_FIXATING,
+					     NULL,
+					     FALSE);
+	}
+	else if (strstr (line, "already carries isofs") && strstr (line, "FATAL:")) {
 		brasero_job_error (BRASERO_JOB (process), 
 				   g_error_new (BRASERO_BURN_ERROR,
 						BRASERO_BURN_ERROR_MEDIA_NOT_WRITABLE,
 						_("The disc is already burnt")));
 	}
-	else if (strstr (line, "unable to open") || strstr (line, "unable to stat")) {
-		/* This fits the "open6" and "open"-like messages */
+	else if (strstr (line, "unable to open")
+	     ||  strstr (line, "unable to stat")) {
 		brasero_job_error (BRASERO_JOB (process), 
 				   g_error_new (BRASERO_BURN_ERROR,
 						BRASERO_BURN_ERROR_BUSY_DRIVE,
@@ -800,15 +686,6 @@ brasero_growisofs_read_stderr (BraseroProcess *process, const char *line)
 						BRASERO_BURN_ERROR_GENERAL,
 						_("The files selected did not fit on the CD")));
 	}
-	else if (strstr (line, "flushing cache") != NULL) {
-		brasero_job_progress_changed (BRASERO_JOB (process), 
-					      0.98,
-					      -1);
-		brasero_job_action_changed (BRASERO_JOB (process),
-					    BRASERO_BURN_ACTION_FIXATING,
-					    FALSE);
-		brasero_job_set_dangerous (BRASERO_JOB (process), FALSE);
-	}
 	else if (strstr (line, "unable to proceed with recording: unable to unmount")) {
 		brasero_job_error (BRASERO_JOB (process),
 				   g_error_new_literal (BRASERO_BURN_ERROR,
@@ -842,19 +719,24 @@ brasero_growisofs_set_mkisofs_argv (BraseroGrowisofs *growisofs,
 				    GPtrArray *argv,
 				    GError **error)
 {
-	BraseroTrackSourceType type;
+	BraseroImageFormat format;
+	BraseroTrackSource *source;
 
 	g_ptr_array_add (argv, g_strdup ("-r"));
+	source = growisofs->priv->source;
 
-	if (growisofs->priv->track_type == BRASERO_TRACK_SOURCE_DEFAULT)
-		type = brasero_burn_caps_get_imager_default_target (growisofs->priv->caps,
-								    growisofs->priv->source);
+	if (growisofs->priv->image_format == BRASERO_IMAGE_FORMAT_ANY)
+		format = brasero_burn_caps_get_imager_default_format (growisofs->priv->caps,
+								      source);
 	else
-		type = growisofs->priv->track_type;
+		format = growisofs->priv->image_format;
 
-	if (type == BRASERO_TRACK_SOURCE_ISO_JOLIET)
+	if (format & BRASERO_IMAGE_FORMAT_JOLIET)
 		g_ptr_array_add (argv, g_strdup ("-J"));
 
+	if (format & BRASERO_IMAGE_FORMAT_VIDEO)
+		g_ptr_array_add (argv, g_strdup ("-dvd-video"));
+
 	if (growisofs->priv->use_utf8) {
 		g_ptr_array_add (argv, g_strdup ("-input-charset"));
 		g_ptr_array_add (argv, g_strdup ("utf8"));
@@ -866,7 +748,7 @@ brasero_growisofs_set_mkisofs_argv (BraseroGrowisofs *growisofs,
 	g_ptr_array_add (argv, g_strdup ("-path-list"));
 	g_ptr_array_add (argv, g_strdup (growisofs->priv->source->contents.grafts.grafts_path));
 
-	if (growisofs->priv->source->contents.grafts.excluded_path) {
+	if (source->contents.grafts.excluded_path) {
 		g_ptr_array_add (argv, g_strdup ("-exclude-list"));
 		g_ptr_array_add (argv, g_strdup (growisofs->priv->source->contents.grafts.excluded_path));
 	}
@@ -902,18 +784,28 @@ brasero_growisofs_set_mkisofs_argv (BraseroGrowisofs *growisofs,
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * Some info about use-the-force-luke options
+ * dry-run => stops after invoking mkisofs
+ * no_tty => avoids fatal error if an isofs exists and an image is piped
+ *  	  => skip the five seconds waiting
+ * 
+ */
 static BraseroBurnResult
 brasero_growisofs_set_argv_record (BraseroGrowisofs *growisofs,
 				   GPtrArray *argv,
 				   GError **error)
 {
 	BraseroBurnResult result;
+	BraseroTrackSource *source;
 
 	if (!growisofs->priv->drive)
-		return BRASERO_BURN_NOT_READY;
+		BRASERO_JOB_NOT_READY (growisofs);
 
 	if (!growisofs->priv->source)
-		return BRASERO_BURN_NOT_READY;
+		BRASERO_JOB_NOT_READY (growisofs);
+
+	source = growisofs->priv->source;
 
 	/* This seems to help to eject tray after burning (at least with mine) */
 	g_ptr_array_add (argv, g_strdup ("-use-the-force-luke=notray"));
@@ -925,11 +817,12 @@ brasero_growisofs_set_argv_record (BraseroGrowisofs *growisofs,
 	if (growisofs->priv->dao)
 		g_ptr_array_add (argv, g_strdup ("-use-the-force-luke=dao"));
 
-	if (growisofs->priv->speed > 0)
-		g_ptr_array_add (argv, g_strdup_printf ("-speed=%d", growisofs->priv->speed));
+	if (growisofs->priv->rate > 0)
+		g_ptr_array_add (argv, g_strdup_printf ("-speed=%d", growisofs->priv->rate));
 
-	/* dvd-compat closes the discs and could cause
-	 * problems if multi session is required */
+	/* dvd-compat closes the discs and could cause problems if multi session
+	* is required. NOTE: it doesn't work with DVD+RW and DVD-RW in restricted
+	* overwrite mode. */
 	if (!growisofs->priv->multi)
 		g_ptr_array_add (argv, g_strdup ("-dvd-compat"));
 
@@ -937,13 +830,11 @@ brasero_growisofs_set_argv_record (BraseroGrowisofs *growisofs,
 	 * a list of grafts. The image can't come through stdin or an already 
 	 * made image */
 	if (growisofs->priv->merge) {
-		if (growisofs->priv->source->type != BRASERO_TRACK_SOURCE_GRAFTS) {
-			g_set_error (error, 
-				     BRASERO_BURN_ERROR,
-				     BRASERO_BURN_ERROR_GENERAL,
-				     _("only graft points can be appended in multisession mode"));
-			return BRASERO_BURN_NOT_SUPPORTED;
-		}
+		if (source->type != BRASERO_TRACK_SOURCE_GRAFTS)
+			BRASERO_JOB_NOT_SUPPORTED (growisofs);
+
+		if (growisofs->priv->sectors_num)
+			g_ptr_array_add (argv, g_strdup_printf ("-use-the-force-luke=tracksize:%"G_GINT64_FORMAT, growisofs->priv->sectors_num));
 
 		g_ptr_array_add (argv, g_strdup ("-M"));
 		if (NCB_DRIVE_GET_DEVICE (growisofs->priv->drive))
@@ -952,12 +843,8 @@ brasero_growisofs_set_argv_record (BraseroGrowisofs *growisofs,
 			return BRASERO_BURN_ERR;
 		
 		/* this can only happen if source->type == BRASERO_TRACK_SOURCE_GRAFTS */
-		if (growisofs->priv->action == BRASERO_GROWISOFS_ACTION_GET_SIZE) {
+		if (growisofs->priv->action == BRASERO_GROWISOFS_ACTION_GET_SIZE)
 			g_ptr_array_add (argv, g_strdup ("-dry-run"));
-			brasero_job_action_changed (BRASERO_JOB (growisofs),
-						    BRASERO_BURN_ACTION_GETTING_SIZE,
-						    FALSE);
-		}
 
 		result = brasero_growisofs_set_mkisofs_argv (growisofs, 
 							     argv,
@@ -971,25 +858,18 @@ brasero_growisofs_set_argv_record (BraseroGrowisofs *growisofs,
 		if (!growisofs->priv->multi)
 			g_ptr_array_add (argv, g_strdup ("-use-the-force-luke=tty"));
 
-		if (growisofs->priv->source->type == BRASERO_TRACK_SOURCE_IMAGER) {
+		if (source->type == BRASERO_TRACK_SOURCE_IMAGER) {
+			BraseroTrackSourceType track_type;
+			BraseroImageFormat format;
 			BraseroImager *imager;
 			gint64 sectors;
 
 			imager = growisofs->priv->source->contents.imager.obj;
 
-			/* in this case this must be an iso : check that */
-			result = brasero_imager_set_output_type (imager,
-								 BRASERO_TRACK_SOURCE_ISO,
-								 error);
-
-			if (result == BRASERO_BURN_NOT_SUPPORTED) {
-				if (!error)
-					g_set_error (error,
-						     BRASERO_BURN_ERROR,
-						     BRASERO_BURN_ERROR_GENERAL,
-						     _("imager can't create iso9660 images"));
-				return BRASERO_BURN_ERR;
-			}
+			/* we need to know what is the type of the track */
+			result = brasero_imager_get_track_type (imager,
+								&track_type,
+								&format);
 
 			if (result != BRASERO_BURN_OK) {
 				if (!error)
@@ -1000,6 +880,15 @@ brasero_growisofs_set_argv_record (BraseroGrowisofs *growisofs,
 				return BRASERO_BURN_ERR;
 			}
 
+			if (track_type != BRASERO_TRACK_SOURCE_IMAGE
+			|| !(format & BRASERO_IMAGE_FORMAT_ISO)) {
+				g_set_error (error,
+					     BRASERO_BURN_ERROR,
+					     BRASERO_BURN_ERROR_GENERAL,
+					     _("imager can't create iso9660 images"));
+				return BRASERO_BURN_ERR;
+			}
+
 			/* ask the size */
 			result = brasero_imager_get_size (imager, &sectors, TRUE, error);
 			if (result != BRASERO_BURN_OK) {
@@ -1034,40 +923,42 @@ brasero_growisofs_set_argv_record (BraseroGrowisofs *growisofs,
 			brasero_job_set_relay_slave_signals (BRASERO_JOB (growisofs), FALSE);
 			brasero_job_set_run_slave (BRASERO_JOB (growisofs), TRUE);
 		}
-		else if (growisofs->priv->source->type == BRASERO_TRACK_SOURCE_ISO
-		      ||  growisofs->priv->source->type == BRASERO_TRACK_SOURCE_ISO_JOLIET) {
+		else if (source->type == BRASERO_TRACK_SOURCE_IMAGE) {
 			gchar *localpath;
 
-			localpath = brasero_track_source_get_iso_localpath (growisofs->priv->source);
+			if (source->format != BRASERO_IMAGE_FORMAT_NONE
+			&& (source->format & BRASERO_IMAGE_FORMAT_ISO) == 0)
+				BRASERO_JOB_NOT_SUPPORTED (growisofs);
+
+			if (growisofs->priv->sectors_num)
+				g_ptr_array_add (argv, g_strdup_printf ("-use-the-force-luke=tracksize:%"G_GINT64_FORMAT, growisofs->priv->sectors_num));
+
+			localpath = brasero_track_source_get_image_localpath (source);
 			if (!localpath) {
-				/* FIXME!
 				g_set_error (error,
 					     BRASERO_BURN_ERROR,
-					     BRASERO_BURN_ERROR,
-					     _("the image is not local")); */
+					     BRASERO_BURN_ERROR_GENERAL,
+					     _("the image is not stored locally"));
 				return BRASERO_BURN_ERR;
 			}
 
-			/* FIXME: should we use DAO ? */
 			g_ptr_array_add (argv, g_strdup ("-Z"));
 			g_ptr_array_add (argv, g_strdup_printf ("%s=%s",
 								NCB_DRIVE_GET_DEVICE (growisofs->priv->drive),
 								localpath));
 			g_free (localpath);
-
 			brasero_job_set_run_slave (BRASERO_JOB (growisofs), FALSE);
 		}
-		else if (growisofs->priv->source->type == BRASERO_TRACK_SOURCE_GRAFTS) {
+		else if (source->type == BRASERO_TRACK_SOURCE_GRAFTS) {
+			if (growisofs->priv->sectors_num)
+				g_ptr_array_add (argv, g_strdup_printf ("-use-the-force-luke=tracksize:%"G_GINT64_FORMAT, growisofs->priv->sectors_num));
+
 			g_ptr_array_add (argv, g_strdup ("-Z"));
 			g_ptr_array_add (argv, g_strdup (NCB_DRIVE_GET_DEVICE (growisofs->priv->drive)));
 
 			/* this can only happen if source->type == BRASERO_TRACK_SOURCE_GRAFTS */
-			if (growisofs->priv->action == BRASERO_GROWISOFS_ACTION_GET_SIZE) {
+			if (growisofs->priv->action == BRASERO_GROWISOFS_ACTION_GET_SIZE)
 				g_ptr_array_add (argv, g_strdup ("-dry-run"));
-				brasero_job_action_changed (BRASERO_JOB (growisofs),
-							     BRASERO_BURN_ACTION_GETTING_SIZE,
-							     FALSE);
-			}
 
 			result = brasero_growisofs_set_mkisofs_argv (growisofs, 
 								     argv,
@@ -1078,7 +969,20 @@ brasero_growisofs_set_argv_record (BraseroGrowisofs *growisofs,
 			brasero_job_set_run_slave (BRASERO_JOB (growisofs), FALSE);
 		}
 		else
-			return BRASERO_BURN_NOT_SUPPORTED;
+			BRASERO_JOB_NOT_SUPPORTED (growisofs);
+	}
+
+	if (growisofs->priv->action == BRASERO_GROWISOFS_ACTION_GET_SIZE) {
+		BRASERO_JOB_TASK_SET_ACTION (growisofs,
+					     BRASERO_BURN_ACTION_GETTING_SIZE,
+					     NULL,
+					     FALSE);
+	}
+	else {
+		BRASERO_JOB_TASK_SET_ACTION (growisofs,
+					     BRASERO_BURN_ACTION_PREPARING,
+					     NULL,
+					     FALSE);
 	}
 
 	return BRASERO_BURN_OK;
@@ -1094,12 +998,26 @@ brasero_growisofs_set_argv_blank (BraseroGrowisofs *growisofs,
 							NCB_DRIVE_GET_DEVICE (growisofs->priv->drive),
 							"/dev/zero"));
 
+		/* That should fix a problem where when the DVD had an isofs
+		 * growisofs warned that it had an isofs already on the disc */
+		g_ptr_array_add (argv, g_strdup ("-use-the-force-luke=tty"));
+
+		/* set a decent speed since from growisofs point of view this
+		 * is still writing. Set 4x and growisofs will adapt the speed
+		 * anyway if disc or drive can't be used at this speed. */
+		g_ptr_array_add (argv, g_strdup_printf ("-speed=%d", 4));
+
 		if (growisofs->priv->dummy)
 			g_ptr_array_add (argv, g_strdup ("-use-the-force-luke=dummy"));
+
+		BRASERO_JOB_TASK_SET_ACTION (growisofs,
+					     BRASERO_BURN_ACTION_ERASING,
+					     NULL,
+					     FALSE);
+		BRASERO_JOB_TASK_START_PROGRESS (growisofs, FALSE);
 	}
 	else
-		brasero_job_debug_message (BRASERO_JOB (growisofs),
-					   "Skipping fast blank for already formatted DVD+RW media\n");
+		BRASERO_JOB_LOG (growisofs,"skipping fast blank for already formatted DVD+RW media");
 
 	/* we never want any slave to be started */
 	return BRASERO_BURN_OK;
@@ -1117,7 +1035,7 @@ brasero_growisofs_set_argv (BraseroProcess *process,
 	growisofs = BRASERO_GROWISOFS (process);
 
 	if (has_master)
-		return BRASERO_BURN_NOT_SUPPORTED;
+		BRASERO_JOB_NOT_SUPPORTED (growisofs);
 
 	g_ptr_array_add (argv, g_strdup ("growisofs"));
 
@@ -1132,34 +1050,7 @@ brasero_growisofs_set_argv (BraseroProcess *process,
 	else if (growisofs->priv->action == BRASERO_GROWISOFS_ACTION_BLANK)
 		result = brasero_growisofs_set_argv_blank (growisofs, argv);
 	else
-		return BRASERO_BURN_NOT_READY;
+		BRASERO_JOB_NOT_READY (growisofs);
 
 	return result;
 }
-
-static BraseroBurnResult
-brasero_growisofs_post (BraseroProcess *process, BraseroBurnResult retval)
-{
-	BraseroGrowisofs *growisofs;
-
-	growisofs = BRASERO_GROWISOFS (process);
-
-	if (growisofs->priv->timer) {
-		g_timer_destroy (growisofs->priv->timer);
-		growisofs->priv->timer = NULL;
-	}
-
-	return BRASERO_BURN_OK;
-}
-
-static BraseroBurnResult
-brasero_growisofs_get_action_string (BraseroJob *job,
-				     BraseroBurnAction action,
-				     char **string)
-{
-	job = brasero_job_get_slave (job);
-	if (!job)
-		return BRASERO_BURN_NOT_SUPPORTED;
-
-	return brasero_job_get_action_string (job, action, string);
-}
