@@ -42,6 +42,8 @@
 #include "burn-imager.h"
 #include "burn-local-image.h"
 #include "burn-common.h"
+#include "burn-sum.h"
+#include "burn-md5.h"
 
 static void brasero_local_image_class_init (BraseroLocalImageClass *klass);
 static void brasero_local_image_init (BraseroLocalImage *sp);
@@ -62,17 +64,8 @@ brasero_local_image_start (BraseroJob *job,
 
 static BraseroBurnResult
 brasero_local_image_stop (BraseroJob *job,
-			  BraseroBurnResult retval);
-
-static BraseroBurnResult
-brasero_local_image_get_action_string (BraseroJob *job,
-				       BraseroBurnAction action,
-				       gchar **string);
-static BraseroBurnResult
-brasero_local_image_get_size (BraseroImager *imager,
-			      gint64 *size,
-			      gboolean sectors,
-			      GError **error);
+			  BraseroBurnResult retval,
+			  GError **error);
 
 static BraseroBurnResult
 brasero_local_image_get_track (BraseroImager *imager,
@@ -93,14 +86,9 @@ struct _BraseroLocalImagePrivate {
 	GHashTable *nonlocals;
 
 	GnomeVFSAsyncHandle *xfer_handle;
-	gint64 current_bytes_copied;
-	gint64 current_copy_size;
-	GTimer *timer;
-	gint report_id;
-	GSList *rates;
-
 	GSList *downloaded;
 
+	gint check_integrity:1;
 	gint overwrite:1;
 	gint clean:1;
 };
@@ -158,7 +146,6 @@ brasero_local_image_get_type ()
 static void
 brasero_local_image_iface_init_image (BraseroImagerIFace *iface)
 {
-	iface->get_size = brasero_local_image_get_size;
 	iface->get_track = brasero_local_image_get_track;
 	iface->set_output = brasero_local_image_set_output;
 }
@@ -172,7 +159,6 @@ brasero_local_image_class_init (BraseroLocalImageClass *klass)
 	parent_class = g_type_class_peek_parent (klass);
 	object_class->finalize = brasero_local_image_finalize;
 
-	job_class->get_action_string = brasero_local_image_get_action_string;
 	job_class->set_source = brasero_local_image_set_source;
 	job_class->start = brasero_local_image_start;
 	job_class->stop = brasero_local_image_stop;
@@ -253,7 +239,7 @@ brasero_local_image_set_source (BraseroJob *job,
 	||  source->type == BRASERO_TRACK_SOURCE_IMAGER
 	||  source->type == BRASERO_TRACK_SOURCE_DISC
 	||  source->type == BRASERO_TRACK_SOURCE_INF)
-		return BRASERO_BURN_NOT_SUPPORTED;
+		BRASERO_JOB_NOT_SUPPORTED (image);
 
 	/* we accept whatever source */
 	image->priv->source = brasero_track_source_copy (source);
@@ -267,18 +253,23 @@ brasero_local_image_start (BraseroJob *job,
 			   int *fd_out,
 			   GError **error)
 {
+	BraseroLocalImage *image;
+
 	/* we can't pipe or be piped */
+	image = BRASERO_LOCAL_IMAGE (job);
+
 	g_return_val_if_fail (BRASERO_IS_LOCAL_IMAGE (job), BRASERO_BURN_ERR);
 	
 	if (fd_in != -1 || fd_out)
-		return BRASERO_BURN_NOT_SUPPORTED;
+		BRASERO_JOB_NOT_SUPPORTED (image);
 
 	return BRASERO_BURN_OK;
 }
 
 static BraseroBurnResult
 brasero_local_image_stop (BraseroJob *job,
-			  BraseroBurnResult retval)
+			  BraseroBurnResult retval,
+			  GError **error)
 {
 	BraseroLocalImage *image = BRASERO_LOCAL_IMAGE (job);
 
@@ -287,69 +278,6 @@ brasero_local_image_stop (BraseroJob *job,
 		image->priv->xfer_handle = NULL;
 	}
 
-	if (image->priv->report_id) {
-		g_source_remove (image->priv->report_id);
-		image->priv->report_id = 0;
-	}
-
-	if (image->priv->timer) {
-		g_timer_destroy (image->priv->timer);
-		image->priv->timer = NULL;
-	}
-
-	if (image->priv->rates) {
-		g_slist_free (image->priv->rates);
-		image->priv->rates = NULL;
-	}
-
-	return BRASERO_BURN_OK;
-}
-
-static BraseroBurnResult
-brasero_local_image_get_action_string (BraseroJob *job,
-				       BraseroBurnAction action,
-				       gchar **string)
-{
-	BraseroLocalImage *image = BRASERO_LOCAL_IMAGE (job);
-
-	if (action == BRASERO_BURN_ACTION_FILE_COPY) {
-		char *name;
-		
-		name = g_path_get_basename (image->priv->current_download);
-		*string = g_strdup_printf (_("Copying \"%s\""), name);
-		g_free (name);
-	}
-	else {
-		const char *tmp;
-
-		tmp = brasero_burn_action_to_string (action);
-		*string = g_strdup (tmp);
-	}
-
-	return BRASERO_BURN_OK;
-}
-
-static BraseroBurnResult
-brasero_local_image_get_size (BraseroImager *imager,
-			      gint64 *size,
-			      gboolean sectors,
-			      GError **error)
-{
-	BraseroLocalImage *image;
-
-	image = BRASERO_LOCAL_IMAGE (imager);
-
-	if (!brasero_job_is_running (BRASERO_JOB (image)))
-		return BRASERO_BURN_NOT_RUNNING;
-
-	if (!image->priv->current_copy_size)
-		return BRASERO_BURN_NOT_READY;
-
-	if (sectors)
-		*size = image->priv->current_copy_size / 2048;
-	else
-		*size = image->priv->current_copy_size;
-
 	return BRASERO_BURN_OK;
 }
 
@@ -377,37 +305,6 @@ brasero_local_image_xfer_async_cb (GnomeVFSAsyncHandle *handle,
 	return TRUE;
 }
 
-static gboolean
-brasero_local_image_report_progress (BraseroLocalImage *image)
-{
-	glong remaining = -1;
-	gdouble fraction;
-	gdouble elapsed;
-
-	elapsed = g_timer_elapsed (image->priv->timer, NULL);
-	if (elapsed <= 2.0)
-		return TRUE;
-
-	fraction = (gdouble) image->priv->current_bytes_copied /
-		   (gdouble) image->priv->current_copy_size;
-
-	if (fraction <= 1.0 && fraction >= 0.0) {
-		gdouble rate;
-
-		rate = image->priv->current_bytes_copied / elapsed;
-		rate = brasero_burn_common_get_average_rate (&image->priv->rates, rate);
-
-		remaining = (gdouble) image->priv->current_copy_size / rate - elapsed;
-	}
-	else
-		fraction = -1.0;
-
-	brasero_job_progress_changed (BRASERO_JOB (image),
-				      fraction,
-				      remaining);
-	return TRUE;
-}
-
 /* This one is for progress reporting */
 static gint
 brasero_local_image_xfer_cb (GnomeVFSXferProgressInfo *info,
@@ -416,19 +313,13 @@ brasero_local_image_xfer_cb (GnomeVFSXferProgressInfo *info,
 	if (!image->priv->xfer_handle)
 		return FALSE;
 
-	if (!image->priv->timer) {
-		image->priv->timer = g_timer_new ();
-		image->priv->report_id = g_timeout_add (200,
-							(GSourceFunc) brasero_local_image_report_progress,
-							image);
-		return TRUE;
-	}
+	BRASERO_JOB_TASK_START_PROGRESS (image, FALSE);
 
 	if (info->file_size)
-		image->priv->current_copy_size = info->bytes_total;
+		BRASERO_JOB_TASK_SET_TOTAL (image, info->bytes_total);
 
 	if (info->bytes_copied)
-		image->priv->current_bytes_copied = info->total_bytes_copied;
+		BRASERO_JOB_TASK_SET_WRITTEN (image, info->total_bytes_copied);
 
 	return TRUE;
 }
@@ -445,11 +336,12 @@ brasero_local_image_download_uri (BraseroLocalImage *image,
 	GnomeVFSURI *vfsuri;
 	GnomeVFSResult res;
 	gchar *localpath_tmp;
-	gchar *escaped_uri;
 	gchar *uri_dest;
+	gchar *string;
+	gchar *name;
 
 	/* generate a unique name */
-	localpath_tmp = g_strconcat (image->priv->tmpdir, "/XXXXXX", NULL);
+	localpath_tmp = g_strconcat (image->priv->tmpdir, "/"BRASERO_BURN_TMP_FILE_NAME, NULL);
 	if (mkstemp (localpath_tmp) == -1) {
 		g_free (localpath_tmp);
 		g_set_error (error,
@@ -464,14 +356,10 @@ brasero_local_image_download_uri (BraseroLocalImage *image,
 	g_free (localpath_tmp);
 
 	/* start the thing */
-	escaped_uri = gnome_vfs_escape_host_and_path_string (uri_src);
-	vfsuri = gnome_vfs_uri_new (escaped_uri);
-	g_free (escaped_uri);
+	vfsuri = gnome_vfs_uri_new (uri_src);
 	src_list = g_list_append (NULL, vfsuri);
 
-	escaped_uri = gnome_vfs_escape_host_and_path_string (uri_dest);
-	tmpuri = gnome_vfs_uri_new (escaped_uri);
-	g_free (escaped_uri);
+	tmpuri = gnome_vfs_uri_new (uri_dest);
 	dest_list = g_list_append (NULL, tmpuri);
 	
 	res = gnome_vfs_async_xfer (&image->priv->xfer_handle,
@@ -504,21 +392,24 @@ brasero_local_image_download_uri (BraseroLocalImage *image,
 
 	/* start the job */
 	image->priv->current_download = uri_src;
-	brasero_job_action_changed (BRASERO_JOB (image),
-				    BRASERO_BURN_ACTION_FILE_COPY,
-				    TRUE);
-	brasero_job_progress_changed (BRASERO_JOB (image), -1.0, -1);
+
+	BRASERO_GET_BASENAME_FOR_DISPLAY (image->priv->current_download, name);
+	string = g_strdup_printf (_("Copying \"%s\""), name);
+	g_free (name);
+
+	BRASERO_JOB_TASK_SET_ACTION (BRASERO_JOB (image),
+				     BRASERO_BURN_ACTION_FILE_COPY,
+				     string,
+				     TRUE);
+	g_free (string);
 
 	result = brasero_job_run (BRASERO_JOB (image), error);
-	image->priv->current_download = NULL;
-	image->priv->current_copy_size = 0;
 
 	/* finished: even if it fails we return the path so that we can clean it */
 	*localuri = uri_dest;
 	if (result != BRASERO_BURN_OK)
 		return result;
 
-	brasero_job_progress_changed (BRASERO_JOB (image), 1.0, -1);
 	return BRASERO_BURN_OK;
 }
 
@@ -554,6 +445,100 @@ _foreach_non_local_cb (const gchar *uri,
 	data->list = g_slist_prepend (data->list, (gchar *) uri);
 }
 
+static BraseroBurnResult
+brasero_local_image_check_image_integrity (BraseroLocalImage *image,
+					   const gchar *image_local_uri,
+					   const gchar *image_uri)
+{
+	gint read;
+	FILE *file;
+	gchar *sumuri;
+	gchar *localuri;
+	gchar *localpath;
+	BraseroJob *slave;
+	BraseroBurnResult result;
+	BraseroTrackSource source;
+	BraseroTrackSource *md5_track;
+	gchar file_checksum [MD5_STRING_LEN + 1];
+	gchar image_checksum [MD5_STRING_LEN + 1];
+	
+	/* we try *.md5 *.md5.asc */
+	sumuri = g_strdup_printf ("%s.md5", image_uri);
+	result = brasero_local_image_download_uri (image,
+						   sumuri,
+						   &localuri,
+						   NULL);
+	g_free (sumuri);
+
+	if (result != BRASERO_BURN_OK)
+		return BRASERO_BURN_OK;
+
+	localpath = gnome_vfs_get_local_path_from_uri (localuri);
+	g_free (localuri);
+
+	/* get the file_checksum from the md5 file */
+	file = fopen (localpath, "r");
+
+	/* that way the file will be removed when fclose is called if it 
+	 * succeeded and remove it anyway if fopen failed */
+	g_remove (localpath);
+
+	if (!file)
+		return BRASERO_BURN_OK;
+
+	read = fread (file_checksum, 1, 32, file);
+	fclose (file);
+
+	if (read != 32)
+		return BRASERO_BURN_OK;
+
+	file_checksum [32] = '\0';
+
+	/* generate a checksum for the image we just downloaded */
+	source.type = BRASERO_TRACK_SOURCE_IMAGE;
+	source.format = BRASERO_IMAGE_FORMAT_ANY;
+	source.contents.image.image = (gchar*) image_local_uri;
+	source.contents.image.toc = NULL;
+
+	slave = BRASERO_JOB (g_object_new (BRASERO_TYPE_BURN_SUM, NULL));
+
+	brasero_job_set_slave (BRASERO_JOB (image), slave);
+	g_object_unref (slave);
+
+	result = brasero_job_set_source (slave, &source, NULL);
+	if (result != BRASERO_BURN_OK) {
+		brasero_job_set_slave (BRASERO_JOB (image), NULL);
+		return BRASERO_BURN_OK;
+	}
+
+	result = brasero_imager_set_output_type (BRASERO_IMAGER (slave),
+						 BRASERO_TRACK_SOURCE_SUM,
+						 BRASERO_IMAGE_FORMAT_NONE,
+						 NULL);
+	if (result != BRASERO_BURN_OK) {
+		brasero_job_set_slave (BRASERO_JOB (image), NULL);
+		return BRASERO_BURN_OK;
+	}
+
+	brasero_job_set_relay_slave_signals (BRASERO_JOB (image), TRUE);
+	result = brasero_imager_get_track (BRASERO_IMAGER (slave),
+					   &md5_track,
+					   NULL);
+
+	brasero_job_set_slave (BRASERO_JOB (image), NULL);
+
+	if (result != BRASERO_BURN_OK)
+		return BRASERO_BURN_OK;
+
+	brasero_md5_string (&md5_track->contents.sum.md5, image_checksum);
+	brasero_track_source_free (md5_track);
+
+	if (strcmp (image_checksum, file_checksum))
+		return BRASERO_BURN_RETRY;
+
+	return BRASERO_BURN_OK;
+}
+
 static BraseroBurnResult
 brasero_local_image_download_non_local (BraseroLocalImage *image,
 					GError **error)
@@ -597,7 +582,6 @@ brasero_local_image_download_non_local (BraseroLocalImage *image,
 		if (g_str_has_prefix (uri, "burn://")) {
 			GnomeVFSHandle *handle = NULL;
 			GnomeVFSResult res;
-			gchar *escaped_uri;
 
 			/* these files are local files so we can check their
 			 * readability and that they are not symlinks nor
@@ -608,10 +592,7 @@ brasero_local_image_download_non_local (BraseroLocalImage *image,
 			if (result != BRASERO_BURN_OK)
 				return result;
 
-			escaped_uri = gnome_vfs_escape_host_and_path_string (uri);
-			res = gnome_vfs_open (&handle, escaped_uri, GNOME_VFS_OPEN_READ);
-			g_free (escaped_uri);
-
+			res = gnome_vfs_open (&handle, uri, GNOME_VFS_OPEN_READ);
 			if (res != GNOME_VFS_OK || !handle)
 				return BRASERO_BURN_ERR;
 
@@ -631,10 +612,28 @@ brasero_local_image_download_non_local (BraseroLocalImage *image,
 			}
 		}
 		else {
-			result = brasero_local_image_download_uri (image,
-								   uri,
-								   &localuri,
-								   error);
+			gint retries = 0;
+
+			localuri = NULL;
+			result = BRASERO_BURN_RETRY;
+
+			while (result == BRASERO_BURN_RETRY && retries < 2) {
+				if (localuri) {
+					g_remove (localuri);
+					g_free (localuri);
+					retries ++;	
+				}
+
+				result = brasero_local_image_download_uri (image,
+									   uri,
+									   &localuri,
+									   error);
+
+				if (result == BRASERO_BURN_OK && image->priv->check_integrity)
+					result = brasero_local_image_check_image_integrity (image,
+											    localuri,
+											    uri);
+			}
 
 			/* add newuri to a list to clean it at finalize time 
 			 * env if it's false, just to make sure. */
@@ -659,6 +658,9 @@ brasero_local_image_translate_uri (BraseroLocalImage *image,
 	gchar *newuri;
 	gchar *parent;
 
+	if (uri == NULL)
+		return NULL;
+
 	/* see if it is a local file */
 	if (g_str_has_prefix (uri, "file://"))
 		return uri;
@@ -737,33 +739,21 @@ brasero_local_image_get_track (BraseroImager *imager,
 		}
 		break;
 
-	case BRASERO_TRACK_SOURCE_AUDIO:
-		for (iter = retval->contents.audio.files; iter; iter = iter->next)
-			ADD_IF_NON_LOCAL (image, iter->data);
-		break;
-
 	case BRASERO_TRACK_SOURCE_GRAFTS:
 		ADD_IF_NON_LOCAL (image, retval->contents.grafts.excluded_path);
 		ADD_IF_NON_LOCAL (image, retval->contents.grafts.grafts_path);
 		break;
 
-	case BRASERO_TRACK_SOURCE_CUE:
-		ADD_IF_NON_LOCAL (image, retval->contents.cue.image);
-		ADD_IF_NON_LOCAL (image, retval->contents.cue.toc);
-		break;
-
-	case BRASERO_TRACK_SOURCE_ISO:
-	case BRASERO_TRACK_SOURCE_ISO_JOLIET:
-		ADD_IF_NON_LOCAL (image, retval->contents.iso.image);
-		break;
-
-	case BRASERO_TRACK_SOURCE_RAW:
-		ADD_IF_NON_LOCAL (image, retval->contents.raw.image);
-		ADD_IF_NON_LOCAL (image, retval->contents.raw.toc);
+	case BRASERO_TRACK_SOURCE_IMAGE:
+		/* This is an image. See if there is any md5 sum sitting in the
+		 * same directory to check our download integrity */
+		ADD_IF_NON_LOCAL (image, retval->contents.image.image);
+		ADD_IF_NON_LOCAL (image, retval->contents.image.toc);
+		image->priv->check_integrity = 1;
 		break;
 
 	default:
-		return BRASERO_BURN_NOT_SUPPORTED;
+		BRASERO_JOB_NOT_SUPPORTED (image);
 	}
 
 	/* see if there is anything to download */
@@ -807,12 +797,6 @@ brasero_local_image_get_track (BraseroImager *imager,
 									iter->data);
 		break;
 
-	case BRASERO_TRACK_SOURCE_AUDIO:
-		for (iter = retval->contents.audio.files; iter; iter = iter->next)
-			iter->data = brasero_local_image_translate_uri (image,
-									iter->data);
-		break;
-
 	case BRASERO_TRACK_SOURCE_GRAFTS:
 		retval->contents.grafts.excluded_path = brasero_local_image_translate_uri (image,
 											   retval->contents.grafts.excluded_path);
@@ -820,28 +804,15 @@ brasero_local_image_get_track (BraseroImager *imager,
 											 retval->contents.grafts.grafts_path);
 		break;
 
-	case BRASERO_TRACK_SOURCE_CUE:
-		retval->contents.cue.image = brasero_local_image_translate_uri (image,
-										retval->contents.cue.image);
-		retval->contents.cue.toc = brasero_local_image_translate_uri (image,
-									      retval->contents.cue.toc);
-		break;
-
-	case BRASERO_TRACK_SOURCE_ISO:
-	case BRASERO_TRACK_SOURCE_ISO_JOLIET:
-		retval->contents.iso.image = brasero_local_image_translate_uri (image,
-										retval->contents.iso.image);
-		break;
-
-	case BRASERO_TRACK_SOURCE_RAW:
-		retval->contents.raw.image = brasero_local_image_translate_uri (image,
-										retval->contents.raw.image);
-		retval->contents.raw.toc = brasero_local_image_translate_uri (image,
-									      retval->contents.raw.toc);
+	case BRASERO_TRACK_SOURCE_IMAGE:
+		retval->contents.image.image = brasero_local_image_translate_uri (image,
+										  retval->contents.image.image);
+		retval->contents.image.toc = brasero_local_image_translate_uri (image,
+										retval->contents.image.toc);
 		break;
 
 	default:
-		return BRASERO_BURN_NOT_SUPPORTED;
+		BRASERO_JOB_NOT_SUPPORTED (image);
 	}
 
 	*source = retval;
