@@ -62,8 +62,6 @@ brasero_mkisofs_set_argv (BraseroProcess *process,
 			  GPtrArray *argv,
 			  gboolean has_master,
 			  GError **error);
-static BraseroBurnResult
-brasero_mkisofs_post (BraseroProcess *process, BraseroBurnResult retval);
 
 static BraseroBurnResult
 brasero_mkisofs_set_source (BraseroJob *job,
@@ -83,6 +81,7 @@ brasero_mkisofs_set_output (BraseroImager *imager,
 static BraseroBurnResult
 brasero_mkisofs_set_output_type (BraseroImager *imager,
 				 BraseroTrackSourceType type,
+				 BraseroImageFormat format,
 				 GError **error);
 static BraseroBurnResult
 brasero_mkisofs_get_track (BraseroImager *imager,
@@ -90,21 +89,14 @@ brasero_mkisofs_get_track (BraseroImager *imager,
 			   GError **error);
 static BraseroBurnResult
 brasero_mkisofs_get_track_type (BraseroImager *imager,
-				BraseroTrackSourceType *type);
+				BraseroTrackSourceType *type,
+				BraseroImageFormat *format);
 static BraseroBurnResult
 brasero_mkisofs_get_size (BraseroImager *imager,
 			  gint64 *size,
 			  gboolean sectors,
 			  GError **error);
 
-static BraseroBurnResult
-brasero_mkisofs_get_action_string (BraseroJob *job,
-				   BraseroBurnAction action,
-				   char **string);
-static BraseroBurnResult
-brasero_mkisofs_get_written (BraseroJob *job,
-			     gint64 *written);
-
 typedef enum {
 	BRASERO_MKISOFS_ACTION_NONE,
 	BRASERO_MKISOFS_ACTION_GET_SIZE,
@@ -114,12 +106,9 @@ typedef enum {
 struct BraseroMkisofsPrivate {
 	BraseroBurnCaps *caps;
 	BraseroMkisofsAction action;
-	BraseroTrackSourceType track_type;
-
-	GTimer *timer;
+	BraseroImageFormat image_format;
 
 	gint64 sectors_num;
-	gint64 b_written;
 
         NautilusBurnDrive *drive;
 	gchar *output;
@@ -182,14 +171,11 @@ brasero_mkisofs_class_init (BraseroMkisofsClass *klass)
 	parent_class = g_type_class_peek_parent(klass);
 	object_class->finalize = brasero_mkisofs_finalize;
 
-	job_class->get_action_string = brasero_mkisofs_get_action_string;
-	job_class->get_written = brasero_mkisofs_get_written;
 	job_class->set_source = brasero_mkisofs_set_source;
 
 	process_class->stdout_func = brasero_mkisofs_read_stdout;
 	process_class->stderr_func = brasero_mkisofs_read_stderr;
 	process_class->set_argv = brasero_mkisofs_set_argv;
-	process_class->post = brasero_mkisofs_post;
 }
 
 static void
@@ -206,7 +192,7 @@ brasero_mkisofs_iface_init_image (BraseroImagerIFace *iface)
 static void
 brasero_mkisofs_init (BraseroMkisofs *obj)
 {
-	char *standard_error;
+	gchar *standard_error;
 	gboolean res;
 
 	obj->priv = g_new0(BraseroMkisofsPrivate, 1);
@@ -225,7 +211,7 @@ brasero_mkisofs_init (BraseroMkisofs *obj)
 	else
 		obj->priv->use_utf8 = FALSE;
 
-	obj->priv->track_type = BRASERO_TRACK_SOURCE_DEFAULT;
+	obj->priv->image_format = BRASERO_IMAGE_FORMAT_NONE;
 
 	g_free (standard_error);
 }
@@ -279,15 +265,16 @@ brasero_mkisofs_new ()
 }
 
 static BraseroBurnResult
-brasero_mkisofs_read_isosize (BraseroMkisofs *mkisofs, const char *line)
+brasero_mkisofs_read_isosize (BraseroMkisofs *mkisofs, const gchar *line)
 {
 	/* mkisofs reports blocks of 2048 bytes */
 	mkisofs->priv->sectors_num = strtoll (line, NULL, 10);
+
 	return BRASERO_BURN_OK;
 }
 
 static BraseroBurnResult
-brasero_mkisofs_read_stdout (BraseroProcess *process, const char *line)
+brasero_mkisofs_read_stdout (BraseroProcess *process, const gchar *line)
 {
 	BraseroMkisofs *mkisofs;
 
@@ -300,38 +287,30 @@ brasero_mkisofs_read_stdout (BraseroProcess *process, const char *line)
 }
 
 static BraseroBurnResult
-brasero_mkisofs_read_stderr (BraseroProcess *process, const char *line)
+brasero_mkisofs_read_stderr (BraseroProcess *process, const gchar *line)
 {
-	char fraction_str [7];
+	gchar fraction_str [7];
+	BraseroMkisofs *mkisofs;
+
+	mkisofs = BRASERO_MKISOFS (process);
 
 	if (strstr (line, "estimate finish")
 	&&  sscanf (line, "%6c%% done, estimate finish", fraction_str) == 1) {
 		gdouble fraction;
-		BraseroMkisofs *mkisofs;
+		gint64 written;
 	
-		mkisofs = BRASERO_MKISOFS (process);
+		fraction = g_strtod (fraction_str, NULL) / (gdouble) 100.0;
+		written = mkisofs->priv->sectors_num * 2048 * fraction;
 
-		fraction = g_strtod (fraction_str, NULL);
-		if (mkisofs->priv->timer) {
-			long secs = -1;
-			gdouble elapsed;
-
-			mkisofs->priv->b_written = mkisofs->priv->sectors_num * 2048 * fraction / 100;
-			elapsed = g_timer_elapsed (mkisofs->priv->timer, NULL);
-			secs = 100.0 * elapsed / fraction - elapsed;
-
-			brasero_job_progress_changed (BRASERO_JOB (process),
-						      fraction / 100.0,
-						      secs);
-		}
-		else
-			mkisofs->priv->timer = g_timer_new ();
+		BRASERO_JOB_TASK_SET_PROGRESS (mkisofs, fraction);
+		BRASERO_JOB_TASK_SET_WRITTEN (mkisofs, written);
+		BRASERO_JOB_TASK_START_PROGRESS (process, FALSE);
 	}
-	else if (!strncmp (line, "Total translation table size", 28)) {
-		brasero_job_progress_changed (BRASERO_JOB (process), 1.0, -1);
-	}
-	else if (!strncmp (line, "Total extents scheduled to be written", 37)) {
-		brasero_job_progress_changed (BRASERO_JOB (process), 1.0, -1);
+	else if (strstr (line, "Input/output error. Read error on old image")) {
+		brasero_job_error (BRASERO_JOB (process), 
+				   g_error_new_literal (BRASERO_BURN_ERROR,
+							BRASERO_BURN_ERROR_GENERAL,
+							_("the old image couldn't be read")));
 	}
 	else if (strstr (line, "Unable to sort directory")) {
 		brasero_job_error (BRASERO_JOB (process), 
@@ -339,6 +318,9 @@ brasero_mkisofs_read_stderr (BraseroProcess *process, const char *line)
 							BRASERO_BURN_ERROR_GENERAL,
 							_("the image can't be created")));
 	}
+	else if (strstr (line, "have the same joliet name")) {
+		/* we keep the name of the files in case we need to rerun */
+	}
 	else if (strstr (line, "Joliet tree sort failed.")) {
 		brasero_job_error (BRASERO_JOB (process), 
 				   g_error_new_literal (BRASERO_BURN_ERROR,
@@ -482,6 +464,7 @@ brasero_mkisofs_set_output (BraseroImager *imager,
 static BraseroBurnResult
 brasero_mkisofs_set_output_type (BraseroImager *imager,
 				 BraseroTrackSourceType type,
+				 BraseroImageFormat format,
 				 GError **error)
 {
 	BraseroMkisofs *mkisofs;
@@ -489,14 +472,16 @@ brasero_mkisofs_set_output_type (BraseroImager *imager,
 	mkisofs = BRASERO_MKISOFS (imager);
 
 	if (type != BRASERO_TRACK_SOURCE_DEFAULT
-	&&  type != BRASERO_TRACK_SOURCE_ISO
-	&&  type != BRASERO_TRACK_SOURCE_ISO_JOLIET)
-		return BRASERO_BURN_NOT_SUPPORTED;
+	&&  type != BRASERO_TRACK_SOURCE_IMAGE)
+		BRASERO_JOB_NOT_SUPPORTED (mkisofs);
 
-	if (mkisofs->priv->track_type == type)
+	if (!(format & (BRASERO_IMAGE_FORMAT_ISO | BRASERO_IMAGE_FORMAT_VIDEO)))
+		BRASERO_JOB_NOT_SUPPORTED (mkisofs)
+	 
+	if (mkisofs->priv->image_format == format)
 		return BRASERO_BURN_OK;
 
-	mkisofs->priv->track_type = type;
+	mkisofs->priv->image_format = format;
 	if (mkisofs->priv->output) {
 		if ((mkisofs->priv->iso_ready || mkisofs->priv->iso_joliet_ready)
 		&&  mkisofs->priv->clean)
@@ -511,23 +496,29 @@ brasero_mkisofs_set_output_type (BraseroImager *imager,
 
 static BraseroBurnResult
 brasero_mkisofs_get_track_type (BraseroImager *imager,
-				BraseroTrackSourceType *type)
+				BraseroTrackSourceType *type,
+				BraseroImageFormat *format)
 {
 	BraseroMkisofs *mkisofs;
-	BraseroTrackSourceType retval;
+	BraseroImageFormat retval;
 
 	mkisofs = BRASERO_MKISOFS (imager);
 
 	if (!mkisofs->priv->source)
-		return BRASERO_BURN_NOT_READY;
+		BRASERO_JOB_NOT_READY (mkisofs);
 
-	if (mkisofs->priv->track_type == BRASERO_TRACK_SOURCE_DEFAULT)
-		retval = brasero_burn_caps_get_imager_default_target (mkisofs->priv->caps,
+	if (type)
+		*type = BRASERO_TRACK_SOURCE_IMAGE;
+
+	if (mkisofs->priv->image_format == BRASERO_IMAGE_FORMAT_ANY)
+		retval = brasero_burn_caps_get_imager_default_format (mkisofs->priv->caps,
 								      mkisofs->priv->source);
 	else
-		retval = mkisofs->priv->track_type;
+		retval = mkisofs->priv->image_format;
+
+	if (format)
+		*format = retval;
 
-	*type = retval;
 	return BRASERO_BURN_OK;
 }
 
@@ -572,6 +563,7 @@ brasero_mkisofs_get_grafts (BraseroMkisofs *mkisofs,
 
 	result = brasero_imager_set_output_type (imager,
 						 BRASERO_TRACK_SOURCE_GRAFTS,
+						 source->format,
 						 error);
 	if (result != BRASERO_BURN_OK)
 		return result;
@@ -610,7 +602,7 @@ brasero_mkisofs_set_source (BraseroJob *job,
 
 	if (source->type != BRASERO_TRACK_SOURCE_GRAFTS
 	&&  source->type != BRASERO_TRACK_SOURCE_DATA)
-		return BRASERO_BURN_NOT_SUPPORTED;
+		BRASERO_JOB_NOT_SUPPORTED (mkisofs);
 
 	if (source->type == BRASERO_TRACK_SOURCE_DATA)
 		result = brasero_mkisofs_get_grafts (mkisofs, source, error);
@@ -627,24 +619,27 @@ brasero_mkisofs_set_argv_image (BraseroMkisofs *mkisofs,
 				GError **error)
 {
 	BraseroBurnResult result;
-	BraseroTrackSourceType type;
+	BraseroImageFormat format;
 
 	if (!mkisofs->priv->source
 	||   mkisofs->priv->source->type != BRASERO_TRACK_SOURCE_GRAFTS)
-		return BRASERO_BURN_NOT_READY;
+		BRASERO_JOB_NOT_READY (mkisofs);
 
 	/* set argv */
 	g_ptr_array_add (argv, g_strdup ("-r"));
 
-	if (mkisofs->priv->track_type == BRASERO_TRACK_SOURCE_DEFAULT)
-		type = brasero_burn_caps_get_imager_default_target (mkisofs->priv->caps,
-								    mkisofs->priv->source);
+	if (mkisofs->priv->image_format == BRASERO_IMAGE_FORMAT_ANY)
+		format = brasero_burn_caps_get_imager_default_format (mkisofs->priv->caps,
+								      mkisofs->priv->source);
 	else
-		type = mkisofs->priv->track_type;
+		format = mkisofs->priv->image_format;
 
-	if (type == BRASERO_TRACK_SOURCE_ISO_JOLIET)
+	if ((format & BRASERO_IMAGE_FORMAT_JOLIET))
 		g_ptr_array_add (argv, g_strdup ("-J"));
 
+	if ((format & BRASERO_IMAGE_FORMAT_VIDEO))
+		g_ptr_array_add (argv, g_strdup ("-dvd-video"));
+
 	g_ptr_array_add (argv, g_strdup ("-graft-points"));
 	g_ptr_array_add (argv, g_strdup ("-D"));	// This is dangerous the manual says but apparently it works well
 
@@ -657,11 +652,14 @@ brasero_mkisofs_set_argv_image (BraseroMkisofs *mkisofs,
 	}
 
 	if (mkisofs->priv->action == BRASERO_MKISOFS_ACTION_GET_SIZE) {
-		g_ptr_array_add (argv, g_strdup ("-q"));
+		g_ptr_array_add (argv, g_strdup ("-quiet"));
 		g_ptr_array_add (argv, g_strdup ("-print-size"));
-		brasero_job_action_changed (BRASERO_JOB (mkisofs),
-					    BRASERO_BURN_ACTION_GETTING_SIZE,
-					    FALSE);
+
+		BRASERO_JOB_TASK_SET_ACTION (mkisofs,
+					     BRASERO_BURN_ACTION_GETTING_SIZE,
+					     NULL,
+					     FALSE);
+		BRASERO_JOB_TASK_START_PROGRESS (mkisofs, FALSE);
 		return BRASERO_BURN_OK;
 	}
 
@@ -688,6 +686,8 @@ brasero_mkisofs_set_argv_image (BraseroMkisofs *mkisofs,
 	if (!has_master) {
 		/* see if an output was given otherwise create a temp one */
 		result = brasero_burn_common_check_output (&mkisofs->priv->output,
+							   BRASERO_IMAGE_FORMAT_ISO,
+							   TRUE,
 							   mkisofs->priv->overwrite,
 							   NULL,
 							   error);
@@ -699,10 +699,11 @@ brasero_mkisofs_set_argv_image (BraseroMkisofs *mkisofs,
 	}
 
 	if (mkisofs->priv->drive) {
-		char *startpoint = NULL;
-		char *command = NULL;
+		gchar *startpoint = NULL;
+		gchar *command = NULL;
+		gboolean has_audio;
 		gboolean res;
-		int tmp;
+		gint tmp;
 
 		command = g_strdup_printf ("cdrecord -msinfo dev=%s",
 					   NCB_DRIVE_GET_DEVICE (mkisofs->priv->drive));
@@ -725,8 +726,14 @@ brasero_mkisofs_set_argv_image (BraseroMkisofs *mkisofs,
 		g_ptr_array_add (argv, g_strdup ("-C"));
 		g_ptr_array_add (argv, startpoint);
 
-		if (mkisofs->priv->merge) {
-		        char *dev_str = NULL;
+		NCB_DRIVE_MEDIA_GET_TYPE (mkisofs->priv->drive,
+					  NULL,
+					  NULL,
+					  NULL,
+					  &has_audio);
+
+		if (mkisofs->priv->merge && !has_audio) {
+		        gchar *dev_str = NULL;
 
 			g_ptr_array_add (argv, g_strdup ("-M"));
 
@@ -735,11 +742,14 @@ brasero_mkisofs_set_argv_image (BraseroMkisofs *mkisofs,
 		}
 	}
 
-	brasero_job_action_changed (BRASERO_JOB (mkisofs), 
-				    BRASERO_BURN_ACTION_CREATING_IMAGE,
-				    FALSE);
-	brasero_job_progress_changed (BRASERO_JOB (mkisofs),
-				      0.0, -1);
+	BRASERO_JOB_TASK_SET_ACTION (mkisofs, 
+				     BRASERO_BURN_ACTION_CREATING_IMAGE,
+				     NULL,
+				     FALSE);
+
+	if (mkisofs->priv->sectors_num)
+		BRASERO_JOB_TASK_SET_TOTAL (mkisofs, mkisofs->priv->sectors_num * 2048);
+
 	return BRASERO_BURN_OK;
 }
 
@@ -768,7 +778,7 @@ brasero_mkisofs_set_argv (BraseroProcess *process,
 	else if (has_master)
 		result = brasero_mkisofs_set_argv_image (mkisofs, argv, has_master, error);
 	else
-		return BRASERO_BURN_NOT_READY;
+		BRASERO_JOB_NOT_READY (mkisofs);
 
 	if (result != BRASERO_BURN_OK)
 		return result;
@@ -780,43 +790,28 @@ brasero_mkisofs_set_argv (BraseroProcess *process,
 	return BRASERO_BURN_OK;
 }
 
-static BraseroBurnResult
-brasero_mkisofs_post (BraseroProcess *process, BraseroBurnResult retval)
-{
-	BraseroMkisofs *mkisofs;
-
-	mkisofs = BRASERO_MKISOFS (process);
-
-	if (mkisofs->priv->timer) {
-		g_timer_destroy (mkisofs->priv->timer);
-		mkisofs->priv->timer = NULL;
-	}
-
-	return BRASERO_BURN_OK;
-}
-
 static BraseroBurnResult
 brasero_mkisofs_get_track (BraseroImager *imager,
 			   BraseroTrackSource **track,
 			   GError **error)
 {
 	BraseroMkisofs *mkisofs;
+	BraseroImageFormat format;
 	BraseroTrackSource *retval;
-	BraseroTrackSourceType type;
 
 	mkisofs = BRASERO_MKISOFS (imager);
 
-	if (mkisofs->priv->track_type == BRASERO_TRACK_SOURCE_UNKNOWN)
-		return BRASERO_BURN_NOT_READY;
+	if (mkisofs->priv->image_format == BRASERO_IMAGE_FORMAT_NONE)
+		BRASERO_JOB_NOT_READY (mkisofs);
 
-	if (mkisofs->priv->track_type == BRASERO_TRACK_SOURCE_DEFAULT)
-		type = brasero_burn_caps_get_imager_default_target (mkisofs->priv->caps,
-								    mkisofs->priv->source);
+	if (mkisofs->priv->image_format == BRASERO_IMAGE_FORMAT_ANY)
+		format = brasero_burn_caps_get_imager_default_format (mkisofs->priv->caps,
+								      mkisofs->priv->source);
 	else
-		type = mkisofs->priv->track_type;
+		format = mkisofs->priv->image_format;
 
-	if ((!mkisofs->priv->iso_ready && type != BRASERO_TRACK_SOURCE_ISO_JOLIET)
-	||  (!mkisofs->priv->iso_joliet_ready && type == BRASERO_TRACK_SOURCE_ISO_JOLIET)){
+	if ((!mkisofs->priv->iso_ready && !(format & BRASERO_IMAGE_FORMAT_JOLIET))
+	||  (!mkisofs->priv->iso_joliet_ready && (format & BRASERO_IMAGE_FORMAT_JOLIET))){
 		BraseroBurnResult result;
 
 		mkisofs->priv->action = BRASERO_MKISOFS_ACTION_GET_IMAGE;
@@ -826,17 +821,16 @@ brasero_mkisofs_get_track (BraseroImager *imager,
 		if (result != BRASERO_BURN_OK)
 			return result;
 
-		mkisofs->priv->b_written = mkisofs->priv->sectors_num * 2048;
-
-		if (type == BRASERO_TRACK_SOURCE_ISO_JOLIET)
+		if ((format & BRASERO_IMAGE_FORMAT_JOLIET))
 			mkisofs->priv->iso_joliet_ready = 1;
 		else
 			mkisofs->priv->iso_ready = 1;
 	}
 
 	retval = g_new0 (BraseroTrackSource, 1);
-	retval->type = type;
-	retval->contents.iso.image = g_strdup_printf ("file://%s", mkisofs->priv->output);
+	retval->type = BRASERO_TRACK_SOURCE_IMAGE;
+	retval->format = format;
+	retval->contents.image.image = g_strdup_printf ("file://%s", mkisofs->priv->output);
 
 	*track = retval;
 
@@ -859,7 +853,7 @@ brasero_mkisofs_get_size (BraseroImager *imager,
 
 		slave = brasero_job_get_slave (BRASERO_JOB (mkisofs));
 		if (!slave)
-			return BRASERO_BURN_NOT_READY;
+			BRASERO_JOB_NOT_READY (mkisofs);
 
 		return brasero_imager_get_size (BRASERO_IMAGER (slave), size, sectors, error);
 	}
@@ -883,26 +877,3 @@ brasero_mkisofs_get_size (BraseroImager *imager,
 
 	return result;
 }
-static BraseroBurnResult
-brasero_mkisofs_get_action_string (BraseroJob *job,
-				   BraseroBurnAction action,
-				   char **string)
-{
-	job = brasero_job_get_slave (job);
-	if (!job)
-		return BRASERO_BURN_NOT_SUPPORTED;
-
-	return brasero_job_get_action_string (job, action, string);
-}
-
-static BraseroBurnResult
-brasero_mkisofs_get_written (BraseroJob *job,
-			     gint64 *written)
-{
-	BraseroMkisofs *mkisofs;
-
-	mkisofs = BRASERO_MKISOFS (job);
-	*written = mkisofs->priv->b_written;
-
-	return BRASERO_BURN_OK;
-}
