@@ -22,6 +22,10 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include <sys/types.h>
 #include <signal.h>
 #include <unistd.h>
@@ -33,6 +37,7 @@
 
 #include <glib.h>
 #include <glib-object.h>
+#include <glib/gi18n-lib.h>
 
 #include "burn-basics.h"
 #include "burn-process.h"
@@ -47,17 +52,21 @@ brasero_process_pre_init (BraseroJob *job,
 			  gboolean has_master,
 			  GError **error);
 static BraseroBurnResult
-brasero_process_start (BraseroJob *job, int in_fd, int *out_fd, GError **error);
+brasero_process_start (BraseroJob *job,
+		       int in_fd,
+		       int *out_fd,
+		       GError **error);
 static BraseroBurnResult
 brasero_process_stop (BraseroJob *job,
-		      BraseroBurnResult retval);
+		      BraseroBurnResult retval,
+		      GError **error);
 enum {
 	BRASERO_CHANNEL_STDOUT	= 0,
 	BRASERO_CHANNEL_STDERR
 };
 
-static const char *debug_prefixes [] = {	"stdout (%s): %s",
-						"stderr (%s): %s",
+static const gchar *debug_prefixes [] = {	"stdout: %s",
+						"stderr: %s",
 						NULL };
 
 typedef BraseroBurnResult	(*BraseroProcessReadFunc)	(BraseroProcess *process,
@@ -166,7 +175,8 @@ brasero_process_finalize (GObject *object)
 	}
 
 	if (cobj->priv->argv) {
-		g_ptr_array_free (cobj->priv->argv, TRUE);
+		g_strfreev ((gchar**) cobj->priv->argv->pdata);
+		g_ptr_array_free (cobj->priv->argv, FALSE);
 		cobj->priv->argv = NULL;
 	}
 
@@ -187,16 +197,16 @@ brasero_process_pre_init (BraseroJob *job,
 	if (process->priv->pid)
 		return BRASERO_BURN_RUNNING;
 
-	if (!klass->set_argv) {
-		brasero_job_debug_message (BRASERO_JOB (process), "object doesn't implement set_argv.");
-		return BRASERO_BURN_NOT_SUPPORTED;
+	if (!klass->set_argv)
+		BRASERO_JOB_NOT_SUPPORTED (process);
+
+	BRASERO_JOB_LOG (process, "getting varg");
+
+	if (process->priv->argv) {
+		g_strfreev ((gchar**) process->priv->argv->pdata);
+		g_ptr_array_free (process->priv->argv, FALSE);
 	}
 
-	brasero_job_debug_message (BRASERO_JOB (process), "getting varg.");
-
-	if (process->priv->argv)
-		g_ptr_array_free (process->priv->argv, TRUE);
-
 	process->priv->argv = g_ptr_array_new ();
 	result = klass->set_argv (process,
 				  process->priv->argv,
@@ -204,16 +214,15 @@ brasero_process_pre_init (BraseroJob *job,
 				  error);
 	g_ptr_array_add (process->priv->argv, NULL);
 
-	brasero_job_debug_message (BRASERO_JOB (process),
-				   "got varg for %s.",
-				   G_OBJECT_TYPE_NAME (process));
+	BRASERO_JOB_LOG (process, "got varg:");
 
 	for (i = 0; process->priv->argv->pdata [i]; i++)
-		brasero_job_debug_message (BRASERO_JOB (process),
-					   process->priv->argv->pdata [i]);
+		BRASERO_JOB_LOG_ARG (process,
+				     process->priv->argv->pdata [i]);
 
 	if (result != BRASERO_BURN_OK) {
-		g_ptr_array_free (process->priv->argv, TRUE);
+		g_strfreev ((gchar**) process->priv->argv->pdata);
+		g_ptr_array_free (process->priv->argv, FALSE);
 		process->priv->argv = NULL;
 		return result;
 	}
@@ -232,6 +241,9 @@ brasero_process_read (BraseroProcess *process,
 	GIOStatus status;
 	BraseroBurnResult result = BRASERO_BURN_OK;
 
+	if (!channel)
+		return FALSE;
+
 	if (channel_type == BRASERO_CHANNEL_STDERR)
 		buffer = process->priv->err_buffer;
 	else
@@ -239,7 +251,7 @@ brasero_process_read (BraseroProcess *process,
 
 	if (condition & G_IO_IN) {
 		gsize term;
-		char *line = NULL;
+		gchar *line = NULL;
 
 		status = g_io_channel_read_line (channel,
 						 &line,
@@ -248,7 +260,7 @@ brasero_process_read (BraseroProcess *process,
 						 NULL);
 
 		if (status == G_IO_STATUS_AGAIN) {
-			char character;
+			gchar character;
 			/* line is NULL since there wasn't any character ending the line and so it wasn't
 			 * read at all.
 			 * some processes (like cdrecord/cdrdao) produce an extra character sometimes */
@@ -266,10 +278,9 @@ brasero_process_read (BraseroProcess *process,
 				case '\r':
 				case '\xe2':
 				case '\0':
-					brasero_job_debug_message (BRASERO_JOB (process),
-								   debug_prefixes [channel_type],
-								   process->priv->argv->pdata [0],
-								   buffer->str);
+					BRASERO_JOB_LOG (process,
+							 debug_prefixes [channel_type],
+							 buffer->str);
 
 					if (read)
 						result = read (process, buffer->str);
@@ -302,10 +313,9 @@ brasero_process_read (BraseroProcess *process,
 			g_string_append (buffer, line);
 			g_free (line);
 
-			brasero_job_debug_message (BRASERO_JOB (process),
-						   debug_prefixes [channel_type],
-						   process->priv->argv->pdata [0],
-						   buffer->str);
+			BRASERO_JOB_LOG (process,
+					 debug_prefixes [channel_type],
+					 buffer->str);
 
 			if (read)
 				result = read (process, buffer->str);
@@ -326,19 +336,19 @@ brasero_process_read (BraseroProcess *process,
 				return FALSE;
 		}
 		else if (status == G_IO_STATUS_EOF) {
-			brasero_job_debug_message (BRASERO_JOB (process), 
-						   debug_prefixes [channel_type],
-						   process->priv->argv->pdata [0],
-						   "EOF");
+			BRASERO_JOB_LOG (process, 
+					 debug_prefixes [channel_type],
+					 "EOF");
 			return FALSE;
 		}
+		else
+			return FALSE;
 	}
 	else if (condition & G_IO_HUP) {
 		/* only handle the HUP when we have read all available lines of output */
-		brasero_job_debug_message (BRASERO_JOB (process),
-					   debug_prefixes [channel_type],
-					   process->priv->argv->pdata [0],
-					   "HUP");
+		BRASERO_JOB_LOG (process,
+				 debug_prefixes [channel_type],
+				 "HUP");
 		return FALSE;
 	}
 
@@ -375,8 +385,8 @@ brasero_process_read_stderr (GIOChannel *source,
 	process->priv->err_buffer = NULL;
 	
 	if (process->priv->pid
-	&&  !process->priv->io_err
-	&&  !process->priv->io_out)
+	&& !process->priv->io_err
+	&& !process->priv->io_out)
 		brasero_job_finished (BRASERO_JOB (process));
 
 	return FALSE;
@@ -405,15 +415,17 @@ brasero_process_read_stdout (GIOChannel *source,
 
 	process->priv->io_out = 0;
 
-	g_io_channel_unref (process->priv->std_out);
-	process->priv->std_out = NULL;
+	if (process->priv->std_out) {
+		g_io_channel_unref (process->priv->std_out);
+		process->priv->std_out = NULL;
+	}
 
 	g_string_free (process->priv->out_buffer, TRUE);
 	process->priv->out_buffer = NULL;
 
 	if (process->priv->pid
-	&&  !process->priv->io_err
-	&&  !process->priv->io_out)
+	&& !process->priv->io_err
+	&& !process->priv->io_out)
 		brasero_job_finished (BRASERO_JOB (process));
 
 	return FALSE;
@@ -431,21 +443,48 @@ brasero_process_setup_stdin (gpointer data)
 		g_warning ("Dup2 failed\n");
 }
 
+static GIOChannel *
+brasero_process_setup_channel (BraseroProcess *process,
+			       int pipe,
+			       gint *watch,
+			       GIOFunc function)
+{
+	GIOChannel *channel;
+
+	fcntl (pipe, F_SETFL, O_NONBLOCK);
+	channel = g_io_channel_unix_new (pipe);
+	g_io_channel_set_flags (channel,
+				g_io_channel_get_flags (channel) | G_IO_FLAG_NONBLOCK,
+				NULL);
+	g_io_channel_set_encoding (channel, NULL, NULL);
+	*watch = g_io_add_watch (channel,
+				(G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_NVAL),
+				 function,
+				 process);
+
+	g_io_channel_set_close_on_unref (channel, TRUE);
+	return channel;
+}
+
 static BraseroBurnResult
 brasero_process_start (BraseroJob *job, int in_fd, int *out_fd, GError **error)
 {
 	BraseroProcess *process = BRASERO_PROCESS (job);
 	int stdout_pipe, stderr_pipe;
-	GIOChannel *channel;
+	/* that's to make sure programs are not translated */
+	gchar *envp [] = {	"LANG=C",
+				"LANGUAGE=C"
+				"LC_ALL=C",
+				NULL};
 
 	if (process->priv->pid)
 		return BRASERO_BURN_RUNNING;
 
-	brasero_job_debug_message (BRASERO_JOB (process), "launching command: ");
+	BRASERO_JOB_LOG (process, "launching command");
 
 	if (!g_spawn_async_with_pipes (NULL,
-				       (char **) process->priv->argv->pdata,
-				       NULL,
+				       (gchar **) process->priv->argv->pdata,
+				       (gchar **) envp,
 				       G_SPAWN_SEARCH_PATH,
 				       brasero_process_setup_stdin,
 				       GINT_TO_POINTER (in_fd),
@@ -457,32 +496,17 @@ brasero_process_start (BraseroJob *job, int in_fd, int *out_fd, GError **error)
 		return BRASERO_BURN_ERR;
 
 	/* error channel */
-	fcntl (stderr_pipe, F_SETFL, O_NONBLOCK);
-	channel = g_io_channel_unix_new (stderr_pipe);
-	g_io_channel_set_flags (channel,
-				g_io_channel_get_flags (channel) | G_IO_FLAG_NONBLOCK,
-				NULL);
-	g_io_channel_set_encoding (channel, NULL, NULL);
-	process->priv->io_err = g_io_add_watch (channel,
-						(G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_NVAL),
-						(GIOFunc) brasero_process_read_stderr,
-						process);
-	process->priv->std_error = channel;
+	process->priv->std_error = brasero_process_setup_channel (process,
+								  stderr_pipe,
+								  &process->priv->io_err,
+								  (GIOFunc) brasero_process_read_stderr);
 
 	/* we only watch stdout coming from the last object in the queue */
-	if (!out_fd) {
-		fcntl (stdout_pipe, F_SETFL, O_NONBLOCK);
-		channel = g_io_channel_unix_new (stdout_pipe);
-		g_io_channel_set_flags (channel,
-					g_io_channel_get_flags (channel) | G_IO_FLAG_NONBLOCK,
-					NULL);
-		g_io_channel_set_encoding (channel, NULL, NULL);
-		process->priv->io_out = g_io_add_watch (channel,
-							(G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_NVAL),
-							(GIOFunc) brasero_process_read_stdout,
-							process);
-		process->priv->std_out = channel;
-	}
+	if (!out_fd)
+		process->priv->std_out = brasero_process_setup_channel (process,
+									stdout_pipe,
+									&process->priv->io_out,
+									(GIOFunc) brasero_process_read_stdout);
 	else
 		*out_fd = stdout_pipe;
 
@@ -491,7 +515,8 @@ brasero_process_start (BraseroJob *job, int in_fd, int *out_fd, GError **error)
 
 static BraseroBurnResult
 brasero_process_stop (BraseroJob *job,
-		      BraseroBurnResult retval)
+		      BraseroBurnResult retval,
+		      GError **error)
 {
 	BraseroBurnResult result = BRASERO_BURN_OK;
 	BraseroProcessClass *klass;
@@ -510,16 +535,13 @@ brasero_process_stop (BraseroJob *job,
 		process->priv->pid = 0;
 
 		if (kill (pid, SIGQUIT) == -1 && errno != ESRCH) {
-			brasero_job_debug_message (job,
-						   "Job %s couldn't be killed (%s). Trying harder.",
-						   process->priv->argv->pdata [0],
-						   strerror (errno));
+			BRASERO_JOB_LOG (process, 
+					 "process (%s) couldn't be killed: terminating",
+					 strerror (errno));
 			kill (pid, SIGKILL);
 		}
 		else
-			brasero_job_debug_message (job,
-						   "Job %s got killed",
-						   process->priv->argv->pdata [0]);
+			BRASERO_JOB_LOG (process, "got killed");
 
 		g_spawn_close_pid (pid);
 	}
@@ -542,8 +564,13 @@ brasero_process_stop (BraseroJob *job,
 						     BRASERO_CHANNEL_STDOUT,
 						     klass->stdout_func));
 		}
-		g_io_channel_unref (process->priv->std_out);
-		process->priv->std_out = NULL;
+
+	    	/* NOTE: we already checked if process->priv->std_out wasn't 
+		 * NULL but brasero_process_read could have closed it */
+	    	if (process->priv->std_out) {
+			g_io_channel_unref (process->priv->std_out);
+			process->priv->std_out = NULL;
+		}
 	}
 
 	if (process->priv->out_buffer) {
@@ -568,8 +595,13 @@ brasero_process_stop (BraseroJob *job,
 						     BRASERO_CHANNEL_STDERR,
 						     klass->stderr_func));
 		}
-		g_io_channel_unref (process->priv->std_error);
-		process->priv->std_error = NULL;
+
+	    	/* NOTE: we already checked if process->priv->std_out wasn't 
+		 * NULL but brasero_process_read could have closed it */
+	    	if (process->priv->std_error) {
+			g_io_channel_unref (process->priv->std_error);
+			process->priv->std_error = NULL;
+		}
 	}
 
 	if (process->priv->err_buffer) {
@@ -578,7 +610,8 @@ brasero_process_stop (BraseroJob *job,
 	}
 
 	if (process->priv->argv) {
-		g_ptr_array_free (process->priv->argv, TRUE);
+		g_strfreev ((gchar**) process->priv->argv->pdata);
+		g_ptr_array_free (process->priv->argv, FALSE);
 		process->priv->argv = NULL;
 	}
 
