@@ -42,6 +42,7 @@
 #include "burn-process.h"
 #include "burn-job.h"
 #include "burn-caps.h"
+#include "burn-iso9660.h"
 #include "brasero-ncb.h"
 
 static void brasero_readcd_class_init (BraseroReadcdClass *klass);
@@ -56,8 +57,6 @@ brasero_readcd_set_argv (BraseroProcess *process,
 			 GPtrArray *argv,
 			 gboolean has_master,
 			 GError **error);
-static BraseroBurnResult
-brasero_readcd_post (BraseroProcess *process, BraseroBurnResult retval);
 
 static BraseroBurnResult
 brasero_readcd_set_source (BraseroJob *job,
@@ -72,6 +71,7 @@ brasero_readcd_set_output (BraseroImager *imager,
 static BraseroBurnResult
 brasero_readcd_set_output_type (BraseroImager *imager,
 				BraseroTrackSourceType type,
+				BraseroImageFormat format,
 				GError **error);
 static BraseroBurnResult
 brasero_readcd_get_track (BraseroImager *imager,
@@ -83,16 +83,10 @@ brasero_readcd_get_size (BraseroImager *imager,
 			 gboolean sectors,
 			 GError **error);
 
-static BraseroBurnResult
-brasero_readcd_get_rate (BraseroJob *job,
-			 gint64 *rate);
-static BraseroBurnResult
-brasero_readcd_get_written (BraseroJob *job,
-			    gint64 *written);
-
 static BraseroBurnResult
 brasero_readcd_get_track_type (BraseroImager *imager,
-			       BraseroTrackSourceType *track_type);
+			       BraseroTrackSourceType *track_type,
+			       BraseroImageFormat *format);
 
 typedef enum {
 	BRASERO_READCD_ACTION_IMAGING,
@@ -102,26 +96,21 @@ typedef enum {
 
 struct BraseroReadcdPrivate {
 	BraseroReadcdAction action;
-	GTimer *timer;
-
 	BraseroBurnCaps *caps;
 
-	BraseroTrackSourceType track_type;
+	BraseroImageFormat image_format;
 	BraseroTrackSource *source;
 
 	gint64 sectors_num;
-	gint64 current_sector;
-	gint64 current_rate;
-	int start;
 
-	char *toc;
-	char *output;
+	gchar *toc;
+	gchar *output;
 
-	int overwrite:1;
-	int clean:1;
+	gint overwrite:1;
+	gint clean:1;
 
-	int track_ready:1;
-	int is_DVD:1;
+	gint track_ready:1;
+	gint is_DVD:1;
 };
 
 static GObjectClass *parent_class = NULL;
@@ -174,13 +163,10 @@ brasero_readcd_class_init (BraseroReadcdClass *klass)
 	parent_class = g_type_class_peek_parent(klass);
 	object_class->finalize = brasero_readcd_finalize;
 
-	job_class->get_rate = brasero_readcd_get_rate;
 	job_class->set_source = brasero_readcd_set_source;
-	job_class->get_written = brasero_readcd_get_written;
 
 	process_class->stderr_func = brasero_readcd_read_stderr;
 	process_class->set_argv = brasero_readcd_set_argv;
-	process_class->post = brasero_readcd_post;
 }
 
 static void
@@ -199,7 +185,6 @@ brasero_readcd_init (BraseroReadcd *obj)
 {
 	obj->priv = g_new0 (BraseroReadcdPrivate, 1);
 	obj->priv->caps = brasero_burn_caps_get_default ();
-	obj->priv->track_type = BRASERO_TRACK_SOURCE_DEFAULT;
 }
 
 static void
@@ -211,11 +196,6 @@ brasero_readcd_finalize (GObject *object)
 	g_object_unref (cobj->priv->caps);
 	cobj->priv->caps = NULL;
 
-	if (cobj->priv->timer) {
-		g_timer_destroy (cobj->priv->timer);
-		cobj->priv->timer = NULL;
-	}
-
 	if (cobj->priv->toc) {
 		if (cobj->priv->clean)
 			g_remove (cobj->priv->toc);
@@ -248,15 +228,15 @@ brasero_readcd_get_track (BraseroImager *imager,
 {
 	BraseroReadcd *readcd;
 	BraseroTrackSource *retval;
-	BraseroTrackSourceType target;
+	BraseroImageFormat format;
 
 	readcd = BRASERO_READCD (imager);
 
 	if (!readcd->priv->source)
-		return BRASERO_BURN_NOT_READY;
+		BRASERO_JOB_NOT_READY (readcd);
 
-	if (readcd->priv->track_type == BRASERO_TRACK_SOURCE_UNKNOWN)
-		return BRASERO_BURN_NOT_READY;
+	if (readcd->priv->image_format == BRASERO_IMAGE_FORMAT_NONE)
+		BRASERO_JOB_NOT_READY (readcd);
 
 	if (!readcd->priv->track_ready) {
 		NautilusBurnDrive *drive;
@@ -270,7 +250,6 @@ brasero_readcd_get_track (BraseroImager *imager,
 		else
 			readcd->priv->is_DVD = FALSE;
 
-		readcd->priv->current_sector = 0;
 		readcd->priv->action = BRASERO_READCD_ACTION_IMAGING;
 		result = brasero_job_run (BRASERO_JOB (readcd), error);
 		readcd->priv->action = BRASERO_READCD_ACTION_NONE;
@@ -278,7 +257,6 @@ brasero_readcd_get_track (BraseroImager *imager,
 		if (result != BRASERO_BURN_OK)
 			return result;
 
-		readcd->priv->current_sector = readcd->priv->sectors_num;
 		readcd->priv->track_ready = 1;
 	}
 
@@ -286,21 +264,19 @@ brasero_readcd_get_track (BraseroImager *imager,
 	retval = g_new0 (BraseroTrackSource, 1);
 
 	/* the disc is the same as when we called set_argv so is the target */
-	target = readcd->priv->track_type;
-	if (target == BRASERO_TRACK_SOURCE_DEFAULT)
-		target = brasero_burn_caps_get_imager_default_target (readcd->priv->caps,
+	format = readcd->priv->image_format;
+	if (format == BRASERO_IMAGE_FORMAT_ANY)
+		format = brasero_burn_caps_get_imager_default_format (readcd->priv->caps,
 								      readcd->priv->source);
-	else
-		target = readcd->priv->track_type;
 
-	retval->type = target;
-	if (target == BRASERO_TRACK_SOURCE_ISO
-	||  target == BRASERO_TRACK_SOURCE_ISO_JOLIET) {
-		retval->contents.iso.image = g_strdup_printf ("file://%s", readcd->priv->output);
+	retval->type = BRASERO_TRACK_SOURCE_IMAGE;
+	retval->format = format;
+	if (format & BRASERO_IMAGE_FORMAT_ISO) {
+		retval->contents.image.image = g_strdup_printf ("file://%s", readcd->priv->output);
 	}
-	else if (target == BRASERO_TRACK_SOURCE_RAW) {
-		retval->contents.raw.toc = g_strdup_printf ("file://%s", readcd->priv->toc);
-		retval->contents.raw.image = g_strdup_printf ("file://%s", readcd->priv->output);
+	else if (format & BRASERO_IMAGE_FORMAT_CLONE) {
+		retval->contents.image.toc = g_strdup_printf ("file://%s", readcd->priv->toc);
+		retval->contents.image.image = g_strdup_printf ("file://%s", readcd->priv->output);
 	}
 
 	*track = retval;
@@ -315,15 +291,36 @@ brasero_readcd_get_size (BraseroImager *imager,
 			 GError **error)
 {
 	BraseroReadcd *readcd;
-	BraseroTrackSourceType target;
+	BraseroImageFormat format;
 	BraseroBurnResult result = BRASERO_BURN_OK;
 
 	readcd = BRASERO_READCD (imager);
 
 	if (!readcd->priv->source)
-		return BRASERO_BURN_NOT_READY;
+		BRASERO_JOB_NOT_READY (readcd);
 
-	if (readcd->priv->sectors_num == 0) {
+	format = readcd->priv->image_format;
+	if (format == BRASERO_IMAGE_FORMAT_ANY)
+		format = brasero_burn_caps_get_imager_default_format (readcd->priv->caps,
+								      readcd->priv->source);
+
+	/* if iso is required then just read the primary vol descriptor */
+	if (format == BRASERO_IMAGE_FORMAT_ISO && !readcd->priv->sectors_num) {
+		gboolean res;
+		gint nb_blocks = 0;
+		NautilusBurnDrive *drive;
+
+		drive = readcd->priv->source->contents.drive.disc;
+		res = brasero_volume_get_size (NCB_DRIVE_GET_DEVICE (drive),
+					       &nb_blocks,
+					       error);
+		if (!res)
+			return BRASERO_BURN_ERR;
+
+		readcd->priv->sectors_num = nb_blocks;
+	}
+
+	if (!readcd->priv->sectors_num) {
 		if (brasero_job_is_running (BRASERO_JOB (imager)))
 			return BRASERO_BURN_RUNNING;
 
@@ -335,74 +332,37 @@ brasero_readcd_get_size (BraseroImager *imager,
 			return result;
 	}
 
-	target = readcd->priv->track_type;
-	if (target == BRASERO_TRACK_SOURCE_DEFAULT)
-		target = brasero_burn_caps_get_imager_default_target (readcd->priv->caps,
-								      readcd->priv->source);
-
 	if (sectors)
 		*size = readcd->priv->sectors_num;
-	else if (target == BRASERO_TRACK_SOURCE_ISO
-	      ||  target == BRASERO_TRACK_SOURCE_ISO_JOLIET)
+	else if (format & BRASERO_IMAGE_FORMAT_ISO)
 		*size = readcd->priv->sectors_num * 2048;
-	else	/* when used with -clone option */
+	else if (format & BRASERO_IMAGE_FORMAT_CLONE)
 		*size = readcd->priv->sectors_num * 2448;
+	else
+		BRASERO_JOB_NOT_SUPPORTED (readcd);
 
 	return result;
 }
 
-static BraseroBurnResult
-brasero_readcd_get_rate (BraseroJob *job,
-			 gint64 *rate)
-{
-	BraseroReadcd *readcd;
-
-	readcd = BRASERO_READCD (job);
-
-	if (!readcd->priv->current_rate)
-		return BRASERO_BURN_NOT_READY;
-
-	if (rate)
-		*rate = readcd->priv->current_rate;
-
-	return BRASERO_BURN_OK;
-}
-
-static BraseroBurnResult
-brasero_readcd_get_written (BraseroJob *job,
-			    gint64 *written)
-{
-	BraseroReadcd *readcd;
-
-	readcd = BRASERO_READCD (job);
-
-	if (written) {
-		BraseroTrackSourceType target;
-
-		target = readcd->priv->track_type;
-		if (target == BRASERO_TRACK_SOURCE_DEFAULT)
-			target = brasero_burn_caps_get_imager_default_target (readcd->priv->caps,
-									      readcd->priv->source);
-
-		if (target == BRASERO_TRACK_SOURCE_ISO
-		||  target == BRASERO_TRACK_SOURCE_ISO_JOLIET)
-			*written = readcd->priv->current_sector * 2048;
-		else	/* when used with -clone option */
-			*written = readcd->priv->current_sector * 2448;
-	}
-
-	return BRASERO_BURN_OK;
-}
-
 static BraseroBurnResult
 brasero_readcd_get_track_type (BraseroImager *imager,
-			       BraseroTrackSourceType *track_type)
+			       BraseroTrackSourceType *type,
+			       BraseroImageFormat *format)
 {
 	BraseroReadcd *readcd;
 
 	readcd = BRASERO_READCD (imager);
 
-	*track_type = readcd->priv->track_type;
+	if (type)
+		*type = BRASERO_TRACK_SOURCE_IMAGE;
+
+	if (format) {
+		if (readcd->priv->image_format == BRASERO_IMAGE_FORMAT_ANY)
+			*format = brasero_burn_caps_get_imager_default_format (readcd->priv->caps,
+									       readcd->priv->source);
+		else
+			*format = readcd->priv->image_format;
+	}
 
 	return BRASERO_BURN_OK;
 }
@@ -433,7 +393,7 @@ brasero_readcd_set_source (BraseroJob *job,
 	readcd->priv->track_ready = 0;
 
 	if (source->type != BRASERO_TRACK_SOURCE_DISC)
-		return BRASERO_BURN_NOT_SUPPORTED;
+		BRASERO_JOB_NOT_SUPPORTED (readcd);
 
 	readcd->priv->source = brasero_track_source_copy (source);
 	return BRASERO_BURN_OK;	
@@ -442,14 +402,19 @@ brasero_readcd_set_source (BraseroJob *job,
 static BraseroBurnResult
 brasero_readcd_set_output_type (BraseroImager *imager,
 				BraseroTrackSourceType type,
+				BraseroImageFormat format,
 				GError **error)
 {
 	BraseroReadcd *readcd;
 
 	readcd = BRASERO_READCD (imager);
 
-	if (readcd->priv->track_type == type)
-		return BRASERO_BURN_OK;
+	if (type != BRASERO_TRACK_SOURCE_IMAGE
+	&&  type != BRASERO_TRACK_SOURCE_DEFAULT)
+		BRASERO_JOB_NOT_SUPPORTED (readcd);
+
+	if (!(format & (BRASERO_IMAGE_FORMAT_CLONE | BRASERO_IMAGE_FORMAT_ISO)))
+		BRASERO_JOB_NOT_SUPPORTED (readcd);
 
 	if (readcd->priv->clean && readcd->priv->track_ready) {
 		if (readcd->priv->output)
@@ -465,21 +430,14 @@ brasero_readcd_set_output_type (BraseroImager *imager,
 	/* NOTE: it's not good to call burn_caps here to get the default
 	 * target for the output since the disc might change before we 
 	 * call get track */
-	readcd->priv->track_type = BRASERO_TRACK_SOURCE_UNKNOWN;
-	if (type != BRASERO_TRACK_SOURCE_DEFAULT
-	&&  type != BRASERO_TRACK_SOURCE_RAW
-	&&  type != BRASERO_TRACK_SOURCE_ISO
-	&&  type != BRASERO_TRACK_SOURCE_ISO_JOLIET)
-		return BRASERO_BURN_NOT_SUPPORTED;
-
-	readcd->priv->track_type = type;
+	readcd->priv->image_format = format;
 
 	return BRASERO_BURN_OK;
 }
 
 static BraseroBurnResult
 brasero_readcd_set_output (BraseroImager *imager,
-			   const char *output,
+			   const gchar *output,
 			   gboolean overwrite,
 			   gboolean clean,
 			   GError **error)
@@ -515,72 +473,68 @@ brasero_readcd_set_output (BraseroImager *imager,
 }
 
 static BraseroBurnResult
-brasero_readcd_read_stderr (BraseroProcess *process, const char *line)
+brasero_readcd_read_stderr (BraseroProcess *process, const gchar *line)
 {
 	BraseroReadcd *readcd;
-	char *pos;
+	gchar *pos;
 
 	readcd = BRASERO_READCD (process);
 	if ((pos = strstr (line, "addr:"))) {
-		int sector;
-		double fraction;
-		long remaining = -1;
+		gint sector;
+		gint64 written;
+		BraseroImageFormat format;
 
 		pos += strlen ("addr:");
 		sector = strtoll (pos, NULL, 10);
 
-		/* readcd reports very often when copying a DVD */
-		if (readcd->priv->is_DVD
-		&&  readcd->priv->current_sector + 256 > sector)
-			return BRASERO_BURN_OK;
+		format = readcd->priv->image_format;
+		if (format == BRASERO_IMAGE_FORMAT_ANY)
+			format = brasero_burn_caps_get_imager_default_format (readcd->priv->caps,
+									      readcd->priv->source);
 
-		readcd->priv->current_sector = sector;
-		fraction = (double) sector / readcd->priv->sectors_num;
-
-		if (!readcd->priv->timer && sector > 10) {
-			/* let the speed regulate before keeping track */
-			readcd->priv->timer = g_timer_new ();
-			readcd->priv->start = sector;
-		}
-
-		if (readcd->priv->sectors_num && readcd->priv->timer) {
-			gdouble elapsed;
-			double rate;
-
-			elapsed = g_timer_elapsed (readcd->priv->timer, NULL);
-			rate = (sector - readcd->priv->start) / elapsed;
-
-			if (rate > 0.0) {
-				/* here the ave_rate = sectors / second
-				 * 1 sec = 75 sectors
-				 * speed 1 => 75 / second for a CD */
-				readcd->priv->current_rate = rate * 2352;
-				remaining = (long) ((gdouble) readcd->priv->sectors_num /
-							        rate - (gdouble) elapsed);
-			}
-		}
-
-		if (remaining >= 0)
-			brasero_job_progress_changed (BRASERO_JOB (readcd),
-						      fraction,
-						      remaining);
+		if (format & BRASERO_IMAGE_FORMAT_ISO)
+			written = sector * 2048;
+		else if (format & BRASERO_IMAGE_FORMAT_CLONE)
+			written = sector * 2448;
 		else
-			brasero_job_progress_changed (BRASERO_JOB (readcd),
-						      fraction,
-						      -1);
+			written = sector * 2048;
+
+		BRASERO_JOB_TASK_SET_WRITTEN (readcd, written);
+
+		if (sector > 10)
+			BRASERO_JOB_TASK_START_PROGRESS (process, FALSE);
 	}
 	else if ((pos = strstr (line, "Capacity:"))) {
-		if (readcd->priv->action == BRASERO_READCD_ACTION_GETTING_SIZE) {
+		gint64 total;
+		BraseroImageFormat format;
+
+		if (!readcd->priv->sectors_num) {
 			pos += strlen ("Capacity:");
 			readcd->priv->sectors_num = strtoll (pos, NULL, 10);
+		}
+
+		format = readcd->priv->image_format;
+		if (format == BRASERO_IMAGE_FORMAT_ANY)
+			format = brasero_burn_caps_get_imager_default_format (readcd->priv->caps,
+									      readcd->priv->source);
+
+		if (format & BRASERO_IMAGE_FORMAT_ISO)
+			total = readcd->priv->sectors_num * 2048;
+		else if (format & BRASERO_IMAGE_FORMAT_CLONE)
+			total = readcd->priv->sectors_num * 2448;
+		else
+			total = readcd->priv->sectors_num * 2048;
+
+		BRASERO_JOB_TASK_SET_TOTAL (readcd, total);
+
+		if (readcd->priv->action != BRASERO_READCD_ACTION_GETTING_SIZE) {
+			BRASERO_JOB_TASK_SET_ACTION (readcd,
+						     BRASERO_BURN_ACTION_DRIVE_COPY,
+						     NULL,
+						     FALSE);
+		}
+		else
 			brasero_job_finished (BRASERO_JOB (readcd));
-		}
-		else {
-			readcd->priv->timer = g_timer_new ();
-			brasero_job_action_changed (BRASERO_JOB (readcd),
-						    BRASERO_BURN_ACTION_DRIVE_COPY,
-						    FALSE);
-		}
 	}
 	else if (strstr (line, "Device not ready.")) {
 		brasero_job_error (BRASERO_JOB (readcd),
@@ -613,6 +567,32 @@ brasero_readcd_read_stderr (BraseroProcess *process, const char *line)
 	return BRASERO_BURN_OK;
 }
 
+static BraseroBurnResult
+brasero_readcd_argv_set_iso_boundary (GPtrArray *argv,
+				      NautilusBurnDrive *drive,
+				      GError **error)
+{
+	gboolean res;
+	gint nb_blocks;
+	NautilusBurnMediaType media;
+
+	media = nautilus_burn_drive_get_media_type (drive);
+
+	/* FIXME: we should do the same for DVD-RW restricted ... */
+	if (media != NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_RW)
+		return BRASERO_BURN_OK;
+
+	/* This is to avoid reading till the end of the DVD */
+	res = brasero_volume_get_size (NCB_DRIVE_GET_DEVICE (drive),
+				       &nb_blocks,
+				       NULL);
+	if (!res)
+		nb_blocks = NCB_MEDIA_GET_SIZE (drive) / 2048;
+
+	g_ptr_array_add (argv, g_strdup_printf ("-sectors=0-%i", nb_blocks));
+	return BRASERO_BURN_OK;
+}
+
 static BraseroBurnResult
 brasero_readcd_set_argv (BraseroProcess *process,
 			 GPtrArray *argv,
@@ -620,18 +600,18 @@ brasero_readcd_set_argv (BraseroProcess *process,
 			 GError **error)
 {
 	BraseroBurnResult result = FALSE;
-	BraseroTrackSourceType target;
+	BraseroImageFormat format;
 	NautilusBurnMediaType type;
 	NautilusBurnDrive *drive;
 	BraseroReadcd *readcd;
-	char *outfile_arg;
-	char *dev_str;
+	gchar *outfile_arg;
+	gchar *dev_str;
 
 	readcd = BRASERO_READCD (process);
 	brasero_job_set_run_slave (BRASERO_JOB (readcd), FALSE);
 
 	if (!readcd->priv->source)
-		return BRASERO_BURN_NOT_READY;
+		BRASERO_JOB_NOT_READY (readcd);
 
 	g_ptr_array_add (argv, g_strdup ("readcd"));
 
@@ -643,25 +623,16 @@ brasero_readcd_set_argv (BraseroProcess *process,
 
 	g_ptr_array_add (argv, dev_str);
 
-	if (readcd->priv->action == BRASERO_READCD_ACTION_GETTING_SIZE) {
-		g_ptr_array_add (argv, g_strdup ("-sectors=0-0"));
-		brasero_job_action_changed (BRASERO_JOB (readcd),
-					    BRASERO_BURN_ACTION_GETTING_SIZE,
-					    FALSE);
-		return BRASERO_BURN_OK;
-	}
-
 	g_ptr_array_add (argv, g_strdup ("-nocorr"));
 
 	type = nautilus_burn_drive_get_media_type (drive);
-	target = readcd->priv->track_type;
-	if (target == BRASERO_TRACK_SOURCE_DEFAULT)
-		target = brasero_burn_caps_get_imager_default_target (readcd->priv->caps,
-								      readcd->priv->source);
+	format = readcd->priv->image_format;
 
+	if (format == BRASERO_IMAGE_FORMAT_ANY)
+		format = brasero_burn_caps_get_imager_default_format (readcd->priv->caps,
+								      readcd->priv->source);
 	if (type > NAUTILUS_BURN_MEDIA_TYPE_CDRW 
-	&&  target != BRASERO_TRACK_SOURCE_ISO
-	&&  target != BRASERO_TRACK_SOURCE_ISO_JOLIET) {
+	&& !(format & BRASERO_IMAGE_FORMAT_ISO)) {
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
@@ -669,28 +640,50 @@ brasero_readcd_set_argv (BraseroProcess *process,
 		return BRASERO_BURN_ERR;
 	}
 
-	if (target == BRASERO_TRACK_SOURCE_RAW) {
+	if (format & BRASERO_IMAGE_FORMAT_CLONE) {
 		/* NOTE: with this option the sector size is 2448 
 		 * because it is raw96 (2352+96) otherwise it is 2048  */
 		g_ptr_array_add (argv, g_strdup ("-clone"));
 	}
-	else /* This is for *.iso images to ease the process */
+	else if (format & BRASERO_IMAGE_FORMAT_ISO)
 		g_ptr_array_add (argv, g_strdup ("-noerror"));
+	else
+		BRASERO_JOB_NOT_SUPPORTED (readcd);
+
+	if (readcd->priv->action == BRASERO_READCD_ACTION_GETTING_SIZE) {
+		g_ptr_array_add (argv, g_strdup ("-sectors=0-0"));
+
+		BRASERO_JOB_TASK_SET_ACTION (readcd,
+					     BRASERO_BURN_ACTION_GETTING_SIZE,
+					     NULL,
+					     FALSE);
+		BRASERO_JOB_TASK_START_PROGRESS (readcd, FALSE);
+		return BRASERO_BURN_OK;
+	}
 
 	if (!has_master) {
-		if (target == BRASERO_TRACK_SOURCE_ISO
-		||  target == BRASERO_TRACK_SOURCE_ISO_JOLIET) {
+		if (format & BRASERO_IMAGE_FORMAT_ISO) {
+			result = brasero_readcd_argv_set_iso_boundary (argv, drive, error);
+			if (result != BRASERO_BURN_OK)
+				return result;
+
 			result = brasero_burn_common_check_output (&readcd->priv->output,
+								   BRASERO_IMAGE_FORMAT_ISO,
+								   TRUE,
 								   readcd->priv->overwrite,
 								   NULL,
 								   error);
 		}
-		else if (target == BRASERO_TRACK_SOURCE_RAW) {	
+		else if (format & BRASERO_IMAGE_FORMAT_CLONE) {	
 			result = brasero_burn_common_check_output (&readcd->priv->output,
+								   BRASERO_IMAGE_FORMAT_CLONE,
+								   TRUE,
 								   readcd->priv->overwrite,
 								   &readcd->priv->toc,
 								   error);
 		}
+		else
+			BRASERO_JOB_NOT_SUPPORTED (readcd);
 
 		if (result != BRASERO_BURN_OK)
 			return result;
@@ -698,41 +691,16 @@ brasero_readcd_set_argv (BraseroProcess *process,
 		outfile_arg = g_strdup_printf ("-f=%s", readcd->priv->output);
 		g_ptr_array_add (argv, outfile_arg);
 	}
-	else if (target == BRASERO_TRACK_SOURCE_ISO
-	      ||  target == BRASERO_TRACK_SOURCE_ISO_JOLIET) {
+	else if (format & BRASERO_IMAGE_FORMAT_ISO) {
+		result = brasero_readcd_argv_set_iso_boundary (argv, drive, error);
+		if (result != BRASERO_BURN_OK)
+			return result;
+
 		outfile_arg = g_strdup ("-f=-");
 		g_ptr_array_add (argv, outfile_arg);
 	}
 	else 	/* unfortunately raw images can't be piped out */
-		return BRASERO_BURN_NOT_SUPPORTED;
+		BRASERO_JOB_NOT_SUPPORTED (readcd);
 
 	return BRASERO_BURN_OK;
 }
-
-static BraseroBurnResult
-brasero_readcd_post (BraseroProcess *process, BraseroBurnResult retval)
-{
-	BraseroReadcd *readcd;
-
-	readcd = BRASERO_READCD (process);
-
-	if (readcd->priv->timer) {
-		g_timer_destroy (readcd->priv->timer);
-		readcd->priv->timer = NULL;
-	}
-
-	readcd->priv->current_rate = 0;
-	readcd->priv->start = 0;
-
-	return BRASERO_BURN_OK;
-}
-
-BraseroReadcd *
-brasero_read_cd_new (void)
-{
-	BraseroReadcd *obj;
-
-	obj = BRASERO_READCD (g_object_new (BRASERO_TYPE_READCD, NULL));
-
-	return obj;
-}
