@@ -54,23 +54,17 @@
 #include "utils.h"
 
 
-static void brasero_filtered_dialog_class_init(BraseroFilteredDialogClass *klass);
-static void brasero_filtered_dialog_init(BraseroFilteredDialog *sp);
-static void brasero_filtered_dialog_finalize(GObject *object);
+static void brasero_filtered_dialog_class_init (BraseroFilteredDialogClass *klass);
+static void brasero_filtered_dialog_init (BraseroFilteredDialog *sp);
+static void brasero_filtered_dialog_finalize (GObject *object);
 
 struct BraseroFilteredDialogPrivate {
 	GtkWidget *tree;
 	GtkWidget *restore_hidden;
 	GtkWidget *restore_broken;
-	GtkWidget *restore_joliet;
-
-	GHashTable *joliet_hash;
 
 	int broken_state:1;
 	int hidden_state:1;
-	int joliet_state:1;
-
-	int disable_joliet_warn:1;
 };
 
 enum  {
@@ -103,9 +97,6 @@ static void
 brasero_filtered_dialog_restore_broken_symlink_cb (GtkButton *button,
 						   BraseroFilteredDialog *dialog);
 static void
-brasero_filtered_dialog_restore_joliet_incompat_cb (GtkButton *button,
-						    BraseroFilteredDialog *dialog);
-static void
 brasero_filtered_dialog_row_activated_cb (GtkTreeView *tree,
                                           GtkTreePath *path,
                                           GtkTreeViewColumn *column,
@@ -181,7 +172,7 @@ brasero_filtered_dialog_init (BraseroFilteredDialog *obj)
 	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (obj)->vbox), 10);
 
 	label = gtk_label_new (_("<span weight=\"bold\" size=\"larger\">The following files were removed automatically from the project.</span>"));
-	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+	g_object_set (G_OBJECT (label), "use-markup", TRUE, NULL);
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (obj)->vbox),
 			    label,
@@ -279,14 +270,6 @@ brasero_filtered_dialog_init (BraseroFilteredDialog *obj)
 			  obj);
 	gtk_box_pack_start (GTK_BOX (box), obj->priv->restore_broken, FALSE, FALSE, 0);
 
-	obj->priv->restore_joliet = gtk_button_new_with_label (_("Restore joliet incompatible files"));
-	gtk_widget_set_sensitive (obj->priv->restore_joliet, FALSE);
-	g_signal_connect (G_OBJECT (obj->priv->restore_joliet),
-			  "clicked",
-			  G_CALLBACK (brasero_filtered_dialog_restore_joliet_incompat_cb),
-			  obj);
-	gtk_box_pack_start (GTK_BOX (box), obj->priv->restore_joliet, FALSE, FALSE, 0);
-
 	gtk_box_pack_end (GTK_BOX (GTK_DIALOG (obj)->vbox),
 			  box,
 			  FALSE,
@@ -305,13 +288,11 @@ static void
 brasero_filtered_dialog_finalize (GObject *object)
 {
 	BraseroFilteredDialog *cobj;
-	cobj = BRASERO_FILTERED_DIALOG(object);
-	
-	if (cobj->priv->joliet_hash)
-		g_hash_table_destroy (cobj->priv->joliet_hash);
 
-	g_free(cobj->priv);
-	G_OBJECT_CLASS(parent_class)->finalize(object);
+	cobj = BRASERO_FILTERED_DIALOG (object);
+	g_free (cobj->priv);
+
+	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 GtkWidget *
@@ -330,20 +311,17 @@ brasero_filtered_dialog_add (BraseroFilteredDialog *dialog,
 			     gboolean restored,
 			     BraseroFilterStatus status)
 {
-	gchar *labels [] = { N_("hidden file"),
+	char *labels [] = { N_("hidden file"),
 			    N_("unreadable file"),
 			    N_("broken symlink"),
 			    N_("recursive symlink"),
-			    N_("Joliet standard incompatible"),
 			    NULL };
 	const gchar *stock_id;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
-	gchar *type = NULL;
-
-	if (status != BRASERO_FILTER_UNKNOWN)
-		type = labels [ status - 1 ];
+	gchar *type;
 
+	type = labels [ status - 1 ];
 	if (status == BRASERO_FILTER_UNREADABLE)
 		stock_id = GTK_STOCK_CANCEL;
 	else
@@ -365,9 +343,6 @@ brasero_filtered_dialog_add (BraseroFilteredDialog *dialog,
 
 	if (status == BRASERO_FILTER_BROKEN_SYM)
 		gtk_widget_set_sensitive (dialog->priv->restore_broken, TRUE);
-
-	if (status == BRASERO_FILTER_JOLIET_INCOMPAT)
-		gtk_widget_set_sensitive (dialog->priv->restore_joliet, TRUE);
 }
 
 void
@@ -411,116 +386,6 @@ brasero_filtered_dialog_get_status (BraseroFilteredDialog *dialog,
 	*removed = retval_removed;
 }
 
-static gboolean
-brasero_filtered_dialog_joliet_warning_dialog (BraseroFilteredDialog *dialog)
-{
-	GtkWidget *warn_dialog;
-	GtkWidget *button;
-	int answer;
-
-	if (dialog->priv->disable_joliet_warn)
-		return TRUE;
-
-	warn_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog),
-					      GTK_DIALOG_DESTROY_WITH_PARENT |
-					      GTK_DIALOG_MODAL,
-					      GTK_MESSAGE_WARNING,
-					      GTK_BUTTONS_NONE,
-					      _("Some files don't have a suitable name for a Windows-compatible CD:"));
-
-	gtk_window_set_title (GTK_WINDOW (warn_dialog), _("Miscrosoft compatibility"));
-
-	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (warn_dialog),
-						  _("Do you want to continue with Windows compatibility disabled?"));
-
-	gtk_dialog_add_buttons (GTK_DIALOG (warn_dialog),
-				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-				NULL);
-
-	button = brasero_utils_make_button (_("Continue"), GTK_STOCK_OK);
-	gtk_dialog_add_action_widget (GTK_DIALOG (warn_dialog),
-				      button, GTK_RESPONSE_OK);
-
-	gtk_widget_show_all (warn_dialog);
-	answer = gtk_dialog_run (GTK_DIALOG (warn_dialog));
-	gtk_widget_destroy (warn_dialog);
-	if (answer != GTK_RESPONSE_OK)
-		return FALSE;
-
-	dialog->priv->disable_joliet_warn = 1;
-	return TRUE;
-}
-
-static gboolean
-_foreach_restored_uri_cb (char *uris,
-			  int n_uris,
-			  BraseroFilteredDialog *dialog)
-{
-	if (n_uris > 1) {
-		dialog->priv->disable_joliet_warn = 1;
-		return TRUE;
-	}
-
-	dialog->priv->disable_joliet_warn = 0;
-	return FALSE;
-}
-
-static void
-brasero_filtered_dialog_joliet_item_excluded (BraseroFilteredDialog *dialog,
-					      const char *uri)
-{
-	int n_uris;
-	char *parent;
-
-	parent = g_path_get_dirname (uri);
-	n_uris = GPOINTER_TO_INT (g_hash_table_lookup (dialog->priv->joliet_hash, parent));
-	n_uris --;
-	if (!n_uris) {
-		g_hash_table_remove (dialog->priv->joliet_hash,
-				     parent);
-		g_free (parent);
-	}
-	else 
-		g_hash_table_replace (dialog->priv->joliet_hash,
-				      parent,
-				      GINT_TO_POINTER (n_uris));
-
-	g_hash_table_find (dialog->priv->joliet_hash,
-			   (GHRFunc) _foreach_restored_uri_cb,
-			   dialog);
-}
-
-static gboolean
-brasero_filtered_dialog_joliet_item_restored (BraseroFilteredDialog *dialog,
-					      const char *uri)
-{
-	int n_uris = 0;
-	char *parent;
-
-	parent = g_path_get_dirname (uri);
-
-	if (!dialog->priv->joliet_hash) {
-		dialog->priv->joliet_hash = g_hash_table_new_full (g_str_hash,
-								   g_str_equal,
-								   g_free,
-								   NULL);
-	}
-	else 
-		n_uris = GPOINTER_TO_INT (g_hash_table_lookup (dialog->priv->joliet_hash, parent));
-
-	if (n_uris && !brasero_filtered_dialog_joliet_warning_dialog (dialog)) {
-		g_free (parent);
-		return TRUE;
-	}
-
-	n_uris ++;
-	g_hash_table_replace (dialog->priv->joliet_hash,
-			      parent,
-			      GINT_TO_POINTER (n_uris));
-
-	return FALSE;
-}
-
 static void
 brasero_filtered_dialog_item_state_changed (BraseroFilteredDialog *dialog,
 				            const GtkTreePath *path)
@@ -540,19 +405,10 @@ brasero_filtered_dialog_item_state_changed (BraseroFilteredDialog *dialog,
 			    INCLUDED_COL, &active, -1);
 
 	if (active) { /* (RE) EXCLUDE */
-		if (status == BRASERO_FILTER_JOLIET_INCOMPAT)
-			brasero_filtered_dialog_joliet_item_excluded (dialog, uri);
-
 		gtk_list_store_set (GTK_LIST_STORE (model), &iter,
 				    INCLUDED_COL, FALSE, -1);
 	}
 	else { /* RESTORE */
-		if (status == BRASERO_FILTER_JOLIET_INCOMPAT
-		&&  brasero_filtered_dialog_joliet_item_restored (dialog, uri)) {
-			g_free (uri);
-			return;
-		}
-
 		gtk_list_store_set (GTK_LIST_STORE (model), &iter,
 				    INCLUDED_COL, TRUE, -1);
 	}
@@ -589,16 +445,6 @@ brasero_filtered_dialog_restore_all (BraseroFilteredDialog *dialog,
 				    STATUS_COL, &row_status, -1);
 
 		if (status == row_status) {
-			if (status == BRASERO_FILTER_JOLIET_INCOMPAT) {
-				char *uri;
-
-				gtk_tree_model_get (model, &iter,
-						    URI_COL, &uri, -1);
-				brasero_filtered_dialog_joliet_item_restored (dialog,
-									      uri);
-				g_free (uri);
-			}
-
 			gtk_list_store_set (GTK_LIST_STORE (model), &iter,
 					    INCLUDED_COL, TRUE, -1);
 		}
@@ -661,29 +507,6 @@ brasero_filtered_dialog_restore_broken_symlink_cb (GtkButton *button,
 	}
 }
 
-static void
-brasero_filtered_dialog_restore_joliet_incompat_cb (GtkButton *button,
-						    BraseroFilteredDialog *dialog)
-{
-	if (!dialog->priv->joliet_state) {
-		if (!brasero_filtered_dialog_joliet_warning_dialog (dialog))
-			return;
-
-		gtk_button_set_label (button, _("Exclude joliet incompatible files"));
-		brasero_filtered_dialog_restore_all (dialog, BRASERO_FILTER_JOLIET_INCOMPAT);
-		dialog->priv->joliet_state = 1;
-		
-	}
-	else {
-		gtk_button_set_label (button, _("Restore joliet incompatible files"));
-		brasero_filtered_dialog_exclude_all (dialog, BRASERO_FILTER_JOLIET_INCOMPAT);
-		dialog->priv->joliet_state = 0;
-		dialog->priv->disable_joliet_warn = 0;
-		g_hash_table_destroy (dialog->priv->joliet_hash);
-		dialog->priv->joliet_hash = NULL;
-	}
-}
-
 static void
 brasero_filtered_dialog_row_activated_cb (GtkTreeView *tree,
                                           GtkTreePath *path,
