@@ -66,6 +66,7 @@ struct BraseroMetadataPrivate {
 	guint stop_id;
 
 	gint fast:1;
+	gint started:1;
 	gint complete_at_playing:1;
 };
 
@@ -191,6 +192,7 @@ brasero_metadata_set_property (GObject *obj,
 				      NULL);
 		gst_element_set_state (GST_ELEMENT (meta->priv->pipeline),
 				       GST_STATE_PAUSED);
+		meta->priv->started = 1;
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, prop_id, pspec);
@@ -204,6 +206,77 @@ brasero_metadata_init (BraseroMetadata *obj)
 	obj->priv = g_new0 (BraseroMetadataPrivate, 1);
 }
 
+static void
+brasero_metadata_stop_pipeline (BraseroMetadata *meta)
+{
+	GstStateChangeReturn change;
+
+	if (meta->priv->stop_id) {
+		g_source_remove (meta->priv->stop_id);
+		meta->priv->stop_id = 0;
+	}
+
+	if (meta->priv->watch) {
+		g_source_remove (meta->priv->watch);
+		meta->priv->watch = 0;
+	}
+
+	meta->priv->started = 0;
+
+	if (!meta->priv->pipeline)
+		return;
+
+	/* better to wait for the state change to be completed */
+	change = gst_element_set_state (GST_ELEMENT (meta->priv->pipeline),
+					GST_STATE_READY);
+
+	while (change == GST_STATE_CHANGE_ASYNC) {
+		GstState state;
+		GstState pending;
+
+		change = gst_element_get_state (meta->priv->pipeline,
+						&state,
+						&pending,
+						GST_MSECOND);
+	};
+	
+	change = gst_element_set_state (GST_ELEMENT (meta->priv->pipeline),
+					GST_STATE_NULL);
+
+	while (change == GST_STATE_CHANGE_ASYNC) {
+		GstState state;
+		GstState pending;
+
+		change = gst_element_get_state (meta->priv->pipeline,
+						&state,
+						&pending,
+						GST_MSECOND);
+	}
+
+	if (change == GST_STATE_CHANGE_FAILURE)
+		g_warning ("State change failure\n");
+
+	gst_object_unref (GST_OBJECT (meta->priv->pipeline));
+	meta->priv->pipeline = NULL;
+}
+
+void
+brasero_metadata_cancel (BraseroMetadata *meta)
+{
+	brasero_metadata_stop_pipeline (meta);
+}
+
+static void
+brasero_metadata_stop (BraseroMetadata *meta)
+{
+	if (meta->priv->error) {
+		g_error_free (meta->priv->error);
+		meta->priv->error = NULL;
+	}
+
+	brasero_metadata_stop_pipeline (meta);
+}
+
 static void
 brasero_metadata_finalize (GObject *object)
 {
@@ -211,26 +284,10 @@ brasero_metadata_finalize (GObject *object)
 
 	cobj = BRASERO_METADATA (object);
 
-	if (cobj->priv->stop_id) {
-		g_source_remove (cobj->priv->stop_id);
-		cobj->priv->stop_id = 0;
-	}
+	brasero_metadata_stop (cobj);
 
-	if (cobj->priv->watch) {
-		g_source_remove (cobj->priv->watch);
-		cobj->priv->watch = 0;
-	}
-
-	if (cobj->priv->pipeline) {
-		gst_element_set_state (GST_ELEMENT (cobj->priv->pipeline), GST_STATE_NULL);
-		gst_object_unref (cobj->priv->pipeline);
-		cobj->priv->pipeline = NULL;
-	}
-
-	if (cobj->uri) {
+	if (cobj->uri)
 		g_free (cobj->uri);
-		cobj->uri = NULL;
-	}
 
 	if (cobj->type)
 		g_free (cobj->type);
@@ -247,11 +304,6 @@ brasero_metadata_finalize (GObject *object)
 	if (cobj->musicbrainz_id)
 		g_free (cobj->musicbrainz_id);
 
-	if (cobj->priv->error) {
-		g_error_free (cobj->priv->error);
-		cobj->priv->error = NULL;
-	}
-
 	g_free (cobj->priv);
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -259,7 +311,6 @@ brasero_metadata_finalize (GObject *object)
 static gboolean
 brasero_metadata_create_pipeline (BraseroMetadata *metadata)
 {
-	gchar *uri;
 	GstBus *bus;
 
 	metadata->priv->pipeline = gst_pipeline_new (NULL);
@@ -270,11 +321,9 @@ brasero_metadata_create_pipeline (BraseroMetadata *metadata)
 						   metadata);
 	gst_object_unref (bus);
 
-	uri = gnome_vfs_escape_host_and_path_string (metadata->uri);
 	metadata->priv->source = gst_element_make_from_uri (GST_URI_SRC,
-							    uri,
+							    metadata->uri,
 							    NULL);
-	g_free (uri);
 
 	if (metadata->priv->source == NULL) {
 		metadata->priv->error = g_error_new (BRASERO_ERROR,
@@ -308,22 +357,20 @@ brasero_metadata_create_pipeline (BraseroMetadata *metadata)
 	}
 	gst_bin_add (GST_BIN (metadata->priv->pipeline), metadata->priv->sink);
 
+	gst_element_set_state (GST_ELEMENT (metadata->priv->pipeline), GST_STATE_READY);
 	return TRUE;
 }
 
 BraseroMetadata *
-brasero_metadata_new (const char *uri)
+brasero_metadata_new (const gchar *uri)
 {
 	BraseroMetadata *obj = NULL;
-	char *unescaped_uri;
 
-	unescaped_uri = gnome_vfs_unescape_string_for_display (uri);
-	if (gst_uri_is_valid (unescaped_uri)) {
+	/* escaped URIS are only used by gnome vfs */
+	if (gst_uri_is_valid (uri)) {
 		obj = BRASERO_METADATA (g_object_new (BRASERO_TYPE_METADATA, NULL));
-		obj->uri = unescaped_uri;
+		obj->uri = g_strdup (uri);
 	}
-	else
-		g_free (unescaped_uri);
 
 	return obj;
 }
@@ -337,6 +384,7 @@ brasero_metadata_is_seekable (BraseroMetadata *meta)
 
 	meta->is_seekable = FALSE;
 	query = gst_query_new_seeking (GST_FORMAT_DEFAULT);
+
 	if (!gst_element_query (meta->priv->pipeline, query))
 		goto end;
 
@@ -393,8 +441,8 @@ brasero_metadata_completed (BraseroMetadata *meta, gboolean eos)
 			g_free (meta->type);
 			meta->type = g_strdup ("audio/mpeg");
 		}
+
 		if (!eos
-/* FIXME	&&  !meta->len should we trust the tags ? */
 		&&  !meta->priv->fast
 		&&  !strcmp (meta->type, "audio/mpeg")) {
 			/* we try to go forward. This allows us to avoid reading
@@ -414,6 +462,8 @@ brasero_metadata_completed (BraseroMetadata *meta, gboolean eos)
 			return FALSE;
 		}
 	}
+	else
+		gst_object_unref (typefind);
 
 	query = gst_query_new_duration (format);
 	if (!gst_element_query (GST_ELEMENT (meta->priv->pipeline), query)) {
@@ -422,6 +472,8 @@ brasero_metadata_completed (BraseroMetadata *meta, gboolean eos)
 							 BRASERO_ERROR_GENERAL,
 							 _("this format is not supported by gstreamer"));
 		}
+
+		gst_query_unref (query);
 		goto signal;
 	}
 	gst_query_parse_duration (query, NULL, &duration);
@@ -437,7 +489,7 @@ brasero_metadata_completed (BraseroMetadata *meta, gboolean eos)
 	brasero_metadata_is_seekable (meta);
 
 signal:
-	gst_element_set_state (GST_ELEMENT (meta->priv->pipeline), GST_STATE_NULL);
+	brasero_metadata_stop_pipeline (meta);
 
 	if (!meta->priv->loop
 	||  !g_main_loop_is_running (meta->priv->loop)) {
@@ -496,7 +548,7 @@ foreach_tag (const GstTagList *list,
 }
 
 static gboolean
-brasero_metadata_stop_pipeline_timeout (BraseroMetadata *meta)
+brasero_metadata_pipeline_timeout_cb (BraseroMetadata *meta)
 {
 	GstBus *bus;
 
@@ -530,13 +582,13 @@ brasero_metadata_stop_pipeline_timeout (BraseroMetadata *meta)
 }
 
 static void
-brasero_metadata_stop_pipeline (BraseroMetadata *meta)
+brasero_metadata_pipeline_timeout (BraseroMetadata *meta)
 {
 	if (meta->priv->stop_id)
 		return;
 
 	meta->priv->stop_id = g_timeout_add (1000,
-					     (GSourceFunc) brasero_metadata_stop_pipeline_timeout,
+					     (GSourceFunc) brasero_metadata_pipeline_timeout_cb,
 					     meta);
 }
 
@@ -587,11 +639,11 @@ brasero_metadata_bus_messages (GstBus *bus,
 
 		if (newstate == GST_STATE_PAUSED
 		&& !meta->priv->complete_at_playing)
-			brasero_metadata_stop_pipeline (meta);
+			brasero_metadata_pipeline_timeout (meta);
 
 		if (newstate == GST_STATE_PLAYING
 		&& !meta->priv->complete_at_playing)
-			brasero_metadata_stop_pipeline (meta);
+			brasero_metadata_pipeline_timeout (meta);
 
 		break;
 
@@ -645,7 +697,7 @@ brasero_metadata_get_sync (BraseroMetadata *meta, gboolean fast, GError **error)
 	}
 
 	meta->priv->fast = (fast == TRUE);
-
+	meta->priv->started = 1;
 	gst_element_set_state (GST_ELEMENT (meta->priv->pipeline), GST_STATE_PLAYING);
 
 	meta->priv->loop = g_main_loop_new (NULL, FALSE);
@@ -653,7 +705,7 @@ brasero_metadata_get_sync (BraseroMetadata *meta, gboolean fast, GError **error)
 	g_main_loop_unref (meta->priv->loop);
 	meta->priv->loop = NULL;
 
-	gst_element_set_state (GST_ELEMENT (meta->priv->pipeline), GST_STATE_NULL);
+	brasero_metadata_stop_pipeline (meta);
 	if (meta->priv->error) {
 		if (error) {
 			g_propagate_error (error, meta->priv->error);
@@ -692,24 +744,7 @@ brasero_metadata_get_async (BraseroMetadata *meta, gboolean fast)
 		return FALSE;
 	}
 
+	meta->priv->started = 1;
 	gst_element_set_state (GST_ELEMENT (meta->priv->pipeline), GST_STATE_PLAYING);
 	return TRUE;
 }
-
-void
-brasero_metadata_cancel (BraseroMetadata *meta)
-{
-	if (meta->priv->stop_id)
-		return;
-
-	if (meta->priv->pipeline) {
-		if (meta->priv->watch) {
-			g_source_remove (meta->priv->watch);
-			meta->priv->watch = 0;
-		}
-
-		gst_element_set_state (GST_ELEMENT (meta->priv->pipeline), GST_STATE_NULL);
-		gst_object_unref (GST_OBJECT (meta->priv->pipeline));
-		meta->priv->pipeline = NULL;
-	}
-}
