@@ -40,16 +40,15 @@
 #include <nautilus-burn-drive.h>
 #include <nautilus-burn-drive-selection.h>
 #include <nautilus-burn-recorder.h>
-
-#ifdef NCB_2_15
 #include <nautilus-burn-drive-monitor.h>
-#endif
 
 #include "burn-caps.h"
 #include "recorder-selection.h"
 #include "utils.h"
 #include "burn.h"
+#include "burn-iso9660.h"
 #include "brasero-ncb.h"
+#include "brasero-image-type-chooser.h"
 
 #define ICON_SIZE 48
 
@@ -70,9 +69,11 @@ static void brasero_recorder_selection_button_cb (GtkWidget *button,
 						   BraseroRecorderSelection *selection);
 
 static void brasero_recorder_selection_drive_changed_cb (NautilusBurnDriveSelection *selector, 
-							  const char *device,
+							  NautilusBurnDrive *drive,
 							  BraseroRecorderSelection *selection);
 
+static void brasero_recorder_selection_update_drive_info (BraseroRecorderSelection *selection);
+
 enum {
 	PROP_NONE,
 	PROP_IMAGE,
@@ -89,10 +90,10 @@ static guint brasero_recorder_selection_signals [LAST_SIGNAL] = { 0 };
 
 struct BraseroRecorderSelectionPrivate {
 	/* output properties */
-	char *image_path;
-	BraseroTrackSourceType image_type;
+	gchar *image_path;
+	BraseroImageFormat image_format;
 
-	GtkWidget *image_type_combo;
+	GtkWidget *image_type_widget;
 	GtkWidget *selection;
 	GHashTable *settings;
 	GtkWidget *dialog;
@@ -280,6 +281,7 @@ brasero_recorder_selection_init (BraseroRecorderSelection *obj)
 
 	brasero_recorder_selection_create_prop_button (obj);
 	gtk_box_pack_start (GTK_BOX (obj), box, FALSE, FALSE, 0);
+	gtk_widget_show_all (box);
 
 	box = gtk_hbox_new (FALSE, 12);
 	obj->priv->image = gtk_image_new ();
@@ -291,17 +293,22 @@ brasero_recorder_selection_init (BraseroRecorderSelection *obj)
 	gtk_box_pack_start (GTK_BOX (obj), box, FALSE, FALSE, 0);
 
 	g_signal_connect (obj->priv->selection,
-			  "device-changed",
+			  "drive-changed",
 			  G_CALLBACK (brasero_recorder_selection_drive_changed_cb),
 			  obj);
+
+	obj->priv->image_format = BRASERO_IMAGE_FORMAT_ANY;
+	gtk_widget_show_all (box);
 }
 
 static void
 brasero_recorder_selection_finalize (GObject *object)
 {
 	BraseroRecorderSelection *cobj;
+	NautilusBurnDriveMonitor *monitor;
 
 	cobj = BRASERO_RECORDER_SELECTION (object);
+	monitor = nautilus_burn_get_drive_monitor ();
 
 	if (cobj->priv->image_path) {
 		g_free (cobj->priv->image_path);
@@ -309,13 +316,13 @@ brasero_recorder_selection_finalize (GObject *object)
 	}
 
 	if (cobj->priv->added_signal) {
-		g_signal_handler_disconnect (cobj->priv->drive,
+		g_signal_handler_disconnect (monitor,
 					     cobj->priv->added_signal);
 		cobj->priv->added_signal = 0;
 	}
 
 	if (cobj->priv->removed_signal) {
-		g_signal_handler_disconnect (cobj->priv->drive,
+		g_signal_handler_disconnect (monitor,
 					     cobj->priv->removed_signal);
 		cobj->priv->removed_signal = 0;
 	}
@@ -336,37 +343,37 @@ brasero_recorder_selection_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-static char *
+static gchar *
 brasero_recorder_selection_get_new_image_path (BraseroRecorderSelection *selection)
 {
-	BraseroTrackSourceType image_type;
-	const char *suffixes [] = {".iso",
+	BraseroImageFormat image_format;
+	const gchar *suffixes [] = {".iso",
 				    ".raw",
 				    ".cue",
+				    ".toc",
 				    NULL };
-	const char *suffix;
-	char *path;
-	int i = 0;
+	const gchar *suffix;
+	gchar *path;
+	gint i = 0;
 
-	image_type = selection->priv->image_type;
-	if (image_type == BRASERO_TRACK_SOURCE_DEFAULT)
-		image_type = brasero_burn_caps_get_imager_default_target (selection->priv->caps,
-									  selection->priv->track_source);
+	if (!selection->priv->track_source)
+		return NULL;
 
-	switch (image_type) {
-	case BRASERO_TRACK_SOURCE_ISO:
-	case BRASERO_TRACK_SOURCE_ISO_JOLIET:
+	image_format = selection->priv->image_format;
+	if (image_format == BRASERO_IMAGE_FORMAT_ANY)
+		image_format = brasero_burn_caps_get_imager_default_format (selection->priv->caps,
+									    selection->priv->track_source);
+
+	if (image_format & BRASERO_IMAGE_FORMAT_ISO)
 		suffix = suffixes [0];
-		break;
-	case BRASERO_TRACK_SOURCE_RAW:
+	else if (image_format & BRASERO_IMAGE_FORMAT_CLONE)
 		suffix = suffixes [1];
-		break;
-	case BRASERO_TRACK_SOURCE_CUE:
+	else if (image_format & BRASERO_IMAGE_FORMAT_CUE)
 		suffix = suffixes [2];
-		break;
-	default:
+	else if (image_format & BRASERO_IMAGE_FORMAT_CDRDAO)
+		suffix = suffixes [3];
+	else
 		return NULL;
-	}
 	
 	path = g_strdup_printf ("%s/brasero%s",
 				g_get_home_dir (),
@@ -381,6 +388,7 @@ brasero_recorder_selection_get_new_image_path (BraseroRecorderSelection *selecti
 					suffix);
 		i ++;
 	};
+
 	return path;
 }
 
@@ -393,11 +401,13 @@ brasero_recorder_selection_set_image_properties (BraseroRecorderSelection *selec
 	else
 		props->output_path = g_strdup (selection->priv->image_path);
 
-	props->props.image_type = selection->priv->image_type;
-	brasero_burn_caps_get_default_flags (selection->priv->caps,
-					     selection->priv->track_source,
-					     selection->priv->drive,
-					     &props->flags);
+	props->props.image_format = selection->priv->image_format;
+	brasero_burn_caps_get_flags (selection->priv->caps,
+				     selection->priv->track_source,
+				     selection->priv->drive,
+				     &props->flags,
+				     NULL,
+				     NULL);
 }
 
 static gboolean
@@ -405,31 +415,36 @@ brasero_recorder_selection_set_drive_default_properties (BraseroRecorderSelectio
 							 BraseroDriveProp *props)
 {
 	BraseroBurnFlag default_flags;
+	gint max_rate;
 
-	if (brasero_burn_caps_get_default_flags (selection->priv->caps,
-						 selection->priv->track_source,
-						 selection->priv->drive,
-						 &default_flags) != BRASERO_BURN_OK)
+	if (brasero_burn_caps_get_flags (selection->priv->caps,
+					 selection->priv->track_source,
+					 selection->priv->drive,
+					 &default_flags,
+					 NULL,
+					 NULL) != BRASERO_BURN_OK)
 		return FALSE;
 
-	props->props.drive_speed = nautilus_burn_drive_get_max_speed_write (selection->priv->drive);
-	props->flags = default_flags;
+	props->props.image_format = BRASERO_IMAGE_FORMAT_ANY;
 
+	max_rate = nautilus_burn_drive_get_max_speed_write (selection->priv->drive);
+	if (NAUTILUS_BURN_DRIVE_MEDIA_TYPE_IS_DVD (nautilus_burn_drive_get_media_type (selection->priv->drive)))
+		props->props.drive_speed = NAUTILUS_BURN_DRIVE_DVD_SPEED (max_rate);
+	else
+		props->props.drive_speed = NAUTILUS_BURN_DRIVE_CD_SPEED (max_rate);
+	
+	props->flags = default_flags;
 	return TRUE;
 }
 
 static gboolean
 brasero_recorder_selection_update_info (BraseroRecorderSelection *selection,
-					NautilusBurnMediaType type,
-					gboolean is_rewritable,
-					gboolean has_audio,
-					gboolean has_data,
-					gboolean is_blank)
+					NautilusBurnDrive *drive)
 {
-	char *info;
+	gchar *info;
 	GdkPixbuf *pixbuf = NULL;
 	gboolean can_record = FALSE;
-	char *types [] = { 	NULL,
+	gchar *types [] = { 	NULL,
 				NULL,
 				NULL,
 				"gnome-dev-cdrom",
@@ -443,11 +458,27 @@ brasero_recorder_selection_update_info (BraseroRecorderSelection *selection,
 				"gnome-dev-disc-dvdrw", /* FIXME */
 				"gnome-dev-disc-dvdr-plus" /* FIXME */,
 				NULL };
+	gboolean is_appendable;
+	NautilusBurnMediaType type;
+	gboolean is_rewritable, has_audio, has_data, is_blank;
+
+	if (drive) {
+		type = NCB_DRIVE_MEDIA_GET_TYPE (drive,
+						 &is_blank,
+						 &is_rewritable,
+						 &has_data,
+						 &has_audio);
+		is_appendable = NCB_MEDIA_IS_APPENDABLE (drive);
+	}
+	else {
+		has_audio = has_data = is_blank = is_appendable = is_rewritable = FALSE;
+		type = NAUTILUS_BURN_MEDIA_TYPE_ERROR;
+	}
 
 	selection->priv->media_type = type;
 
 	if (type == NAUTILUS_BURN_MEDIA_TYPE_BUSY) {
-		info = g_strdup (_("<i>The disc is mounted.</i>"));
+		info = g_strdup (_("<i>The disc is busy.</i>"));
 		can_record = TRUE;
 	}
 	else if (type == NAUTILUS_BURN_MEDIA_TYPE_ERROR) {
@@ -456,20 +487,61 @@ brasero_recorder_selection_update_info (BraseroRecorderSelection *selection,
 	else if (type == NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN) {
 		info = g_strdup (_("<i>Unknown type of disc.</i>"));
 	}
-	else if (!is_blank && !is_rewritable) {
+	else if ((!is_appendable && !is_blank && !is_rewritable)
+	     || ((is_appendable && has_audio) && !is_blank && !is_rewritable)) {
 		info = g_strdup_printf (_("The <b>%s</b> is not writable."),
 					nautilus_burn_drive_media_type_get_string (type));
 		pixbuf = brasero_utils_get_icon (types [type], ICON_SIZE);
 	}
-	else if (has_data) {
-		info = g_strdup_printf (_("The <b>%s</b> is ready.\nIt contains data."),
-					nautilus_burn_drive_media_type_get_string (type));
+	else if (has_audio) {
+		if (is_appendable) {
+			gchar *size;
+			GnomeVFSFileSize remaining;
+
+			remaining = NCB_MEDIA_GET_CAPACITY (drive) - NCB_MEDIA_GET_SIZE (drive);
+			size = gnome_vfs_format_file_size_for_display (remaining);
+			info = g_strdup_printf (_("The <b>%s</b> is ready.\nIt contains audio tracks.\nA data session can be added (%s free)."),
+						nautilus_burn_drive_media_type_get_string (type),
+						size);
+			g_free (size);
+		}
+		else
+			info = g_strdup_printf (_("The <b>%s</b> is ready.\nIt contains audio tracks."),
+						nautilus_burn_drive_media_type_get_string (type));
+
 		pixbuf = brasero_utils_get_icon (types [type], ICON_SIZE);
 		can_record = TRUE;
 	}
-	else if (has_audio) {
-		info = g_strdup_printf (_("The <b>%s</b> is ready.\nIt contains audio tracks."),
-					nautilus_burn_drive_media_type_get_string (type));
+	else if (has_data) {
+		if (is_appendable) {
+			gchar *size;
+			GnomeVFSFileSize remaining;
+
+			if (type == NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_RW) {
+				gint num_blocks;
+				gboolean result;
+
+				result = brasero_volume_get_size (NCB_DRIVE_GET_DEVICE (drive),
+								  &num_blocks,
+								  NULL);
+				if (result)
+					remaining = NCB_MEDIA_GET_CAPACITY (drive) - num_blocks * 2048;
+				else
+					remaining = NCB_MEDIA_GET_CAPACITY (drive);
+			}
+			else
+				remaining = NCB_MEDIA_GET_CAPACITY (drive) - NCB_MEDIA_GET_SIZE (drive);
+
+			size = gnome_vfs_format_file_size_for_display (remaining);
+			info = g_strdup_printf (_("The <b>%s</b> is ready.\nIt contains data.\nMore data can be added (%s free)."),
+						nautilus_burn_drive_media_type_get_string (type),
+						size);
+			g_free (size);
+		}
+		else
+			info = g_strdup_printf (_("The <b>%s</b> is ready.\nIt contains data."),
+						nautilus_burn_drive_media_type_get_string (type));
+
 		pixbuf = brasero_utils_get_icon (types [type], ICON_SIZE);
 		can_record = TRUE;
 	}
@@ -499,24 +571,31 @@ brasero_recorder_selection_update_info (BraseroRecorderSelection *selection,
 }
 
 static void
-brasero_recorder_selection_drive_media_added_cb (NautilusBurnDrive *drive,
+brasero_recorder_selection_drive_media_added_cb (NautilusBurnDriveMonitor *monitor,
+						 NautilusBurnDrive *drive,
 						 BraseroRecorderSelection *selection)
 {
 	NautilusBurnMediaType type;
-	gboolean is_rewritable, has_audio, has_data, is_blank;
+	NautilusBurnDrive *selected_drive = NULL;
 
-	type = nautilus_burn_drive_get_media_type_full (drive,
-							&is_blank,
-							&is_rewritable,
-							&has_data,
-							&has_audio);
+	/* we must make sure that the change was triggered
+	 * by the current selected drive */
+	selected_drive =  
+		nautilus_burn_drive_selection_get_active (NAUTILUS_BURN_DRIVE_SELECTION (selection->priv->selection));
 
-	brasero_recorder_selection_update_info (selection,
-						type,
-						is_rewritable,
-						has_audio,
-						has_data,
-						is_blank);
+	if (!selected_drive)
+		return;
+
+	if (!nautilus_burn_drive_equal (drive, selected_drive)) {
+		nautilus_burn_drive_unref (selected_drive);
+		return;
+	}
+
+	nautilus_burn_drive_unref (selected_drive);
+
+	brasero_recorder_selection_update_info (selection, drive);
+
+	type = nautilus_burn_drive_get_media_type (drive);
 	g_signal_emit (selection,
 		       brasero_recorder_selection_signals [MEDIA_CHANGED_SIGNAL],
 		       0,
@@ -524,9 +603,27 @@ brasero_recorder_selection_drive_media_added_cb (NautilusBurnDrive *drive,
 }
 
 static void
-brasero_recorder_selection_drive_media_removed_cb (NautilusBurnDrive *drive,
+brasero_recorder_selection_drive_media_removed_cb (NautilusBurnDriveMonitor *monitor,
+						   NautilusBurnDrive *drive,
 						   BraseroRecorderSelection *selection)
 {
+	NautilusBurnDrive *selected_drive = NULL;
+
+	/* we must make sure that the change was triggered
+	 * by the current selected drive */
+	selected_drive =  
+		nautilus_burn_drive_selection_get_active (NAUTILUS_BURN_DRIVE_SELECTION (selection->priv->selection));
+
+	if (!selected_drive)
+		return;
+
+	if (!nautilus_burn_drive_equal (drive, selected_drive)) {
+		nautilus_burn_drive_unref (selected_drive);
+		return;
+	}
+
+	nautilus_burn_drive_unref (selected_drive);
+
 	if (selection->priv->dialog)
 		gtk_dialog_response (GTK_DIALOG (selection->priv->dialog),
 				     GTK_RESPONSE_CANCEL);
@@ -534,12 +631,7 @@ brasero_recorder_selection_drive_media_removed_cb (NautilusBurnDrive *drive,
 	/* we don't look at the drive contents since we already
 	 * know that there is nothing inside. If we did, it could
 	 * force the drive to reload the disc */
-	brasero_recorder_selection_update_info (selection,
-						NAUTILUS_BURN_MEDIA_TYPE_ERROR,
-						FALSE,
-						FALSE,
-						FALSE,
-						FALSE);
+	brasero_recorder_selection_update_info (selection, NULL);
 	g_signal_emit (selection,
 		       brasero_recorder_selection_signals [MEDIA_CHANGED_SIGNAL],
 		       0,
@@ -549,19 +641,20 @@ brasero_recorder_selection_drive_media_removed_cb (NautilusBurnDrive *drive,
 static void
 brasero_recorder_selection_update_image_path (BraseroRecorderSelection *selection)
 {
-	char *info;
+	gchar *info;
 
 	if (!selection->priv->image_path) {
-		char *path;
+		gchar *path;
 
 		path = brasero_recorder_selection_get_new_image_path (selection);
 		info = g_strdup_printf (_("The <b>image</b> will be saved to\n%s"),
-					path);
+					path? path:"");
 		g_free (path);
 	}
 	else
 		info = g_strdup_printf (_("The <b>image</b> will be saved to\n%s"),
 					selection->priv->image_path);
+
 	gtk_label_set_markup (GTK_LABEL (selection->priv->infos), info);
 	g_free (info);
 }
@@ -572,15 +665,17 @@ brasero_recorder_selection_update_drive_info (BraseroRecorderSelection *selectio
 	guint added_signal = 0;
 	guint removed_signal = 0;
 	NautilusBurnDrive *drive;
-	gboolean can_record = FALSE;
 	NautilusBurnMediaType type;
-	gboolean is_rewritable, has_audio, has_data, is_blank;
+	gboolean can_record = FALSE;
+	NautilusBurnDriveMonitor *monitor;
 
+	monitor = nautilus_burn_get_drive_monitor ();
 	drive = nautilus_burn_drive_selection_get_active (NAUTILUS_BURN_DRIVE_SELECTION (selection->priv->selection));
+
 	if (drive == NULL) {
 		GdkPixbuf *pixbuf;
 
-		gtk_widget_set_sensitive (selection->priv->selection, FALSE);
+	    	gtk_widget_set_sensitive (selection->priv->selection, FALSE);
 		gtk_label_set_markup (GTK_LABEL (selection->priv->infos),
 				      _("<b>There is no available drive.</b>"));
 
@@ -611,25 +706,13 @@ brasero_recorder_selection_update_drive_info (BraseroRecorderSelection *selectio
 		goto end;
 	}
 
-	type = nautilus_burn_drive_get_media_type_full (drive,
-							&is_blank,
-							&is_rewritable,
-							&has_data,
-							&has_audio);
-
-	can_record = brasero_recorder_selection_update_info (selection,
-							     type,
-							     is_rewritable,
-							     has_audio,
-							     has_data,
-							     is_blank);
-
-	g_object_set (G_OBJECT (drive), "enable-monitor", TRUE, NULL);
-	added_signal = g_signal_connect (G_OBJECT (drive),
+	type = nautilus_burn_drive_get_media_type (drive);
+	can_record = brasero_recorder_selection_update_info (selection, drive);
+	added_signal = g_signal_connect (G_OBJECT (monitor),
 					 "media-added",
 					 G_CALLBACK (brasero_recorder_selection_drive_media_added_cb),
 					 selection);
-	removed_signal = g_signal_connect (G_OBJECT (drive),
+	removed_signal = g_signal_connect (G_OBJECT (monitor),
 					   "media-removed",
 					   G_CALLBACK (brasero_recorder_selection_drive_media_removed_cb),
 					   selection);
@@ -637,23 +720,19 @@ brasero_recorder_selection_update_drive_info (BraseroRecorderSelection *selectio
 end:
 
 	if (selection->priv->added_signal) {
-		g_signal_handler_disconnect (selection->priv->drive,
+		g_signal_handler_disconnect (monitor,
 					     selection->priv->added_signal);
 		selection->priv->added_signal = 0;
 	}
 
 	if (selection->priv->removed_signal) {
-		g_signal_handler_disconnect (selection->priv->drive,
+		g_signal_handler_disconnect (monitor,
 					     selection->priv->removed_signal);
 		selection->priv->removed_signal = 0;
 	}
 
-	if (selection->priv->drive) {
-		g_object_set (G_OBJECT (selection->priv->drive),
-			      "enable-monitor", FALSE,
-			      NULL);
+	if (selection->priv->drive)
 		nautilus_burn_drive_unref (selection->priv->drive);
-	}
 
 	selection->priv->drive = drive;
 	selection->priv->added_signal = added_signal;
@@ -666,7 +745,7 @@ end:
 
 static void
 brasero_recorder_selection_drive_changed_cb (NautilusBurnDriveSelection *selector,
-					     const char *device,
+					     NautilusBurnDrive *drive,
 					     BraseroRecorderSelection *selection)
 {
 	brasero_recorder_selection_update_drive_info (selection);
@@ -679,7 +758,7 @@ brasero_recorder_selection_new (void)
 
 	obj = BRASERO_RECORDER_SELECTION (g_object_new (BRASERO_TYPE_RECORDER_SELECTION,
 					                NULL));
-	brasero_recorder_selection_update_drive_info (obj);
+
 	return GTK_WIDGET (obj);
 }
 
@@ -692,8 +771,6 @@ brasero_recorder_selection_set_source_track (BraseroRecorderSelection *selection
 
 	selection->priv->track_source = brasero_track_source_copy (source);
 	if (source->type == BRASERO_TRACK_SOURCE_DISC) {
-		NautilusBurnMediaType type;
-
 		if (NCB_DRIVE_GET_TYPE (selection->priv->drive) == NAUTILUS_BURN_DRIVE_TYPE_FILE) {
 			/* in case the user asked to copy to a file on the hard disk
 			 * then we need to update the name to change the extension
@@ -703,24 +780,13 @@ brasero_recorder_selection_set_source_track (BraseroRecorderSelection *selection
 
 		/* try to see if we need to update the image
 		 * type selection when copying a drive */
-		if (!selection->priv->image_type_combo)
+		if (!selection->priv->image_type_widget)
 			return;
 
-		/* FIXME: this could force the disc to reload */
-		type = nautilus_burn_drive_get_media_type (source->contents.drive.disc);
-		if (type > NAUTILUS_BURN_MEDIA_TYPE_CDRW) {
-			gtk_combo_box_remove_text (GTK_COMBO_BOX (selection->priv->image_type_combo), 3);
-			gtk_combo_box_remove_text (GTK_COMBO_BOX (selection->priv->image_type_combo), 2);
-
-			if (gtk_combo_box_get_active (GTK_COMBO_BOX (selection->priv->image_type_combo)) == -1)
-				gtk_combo_box_set_active (GTK_COMBO_BOX (selection->priv->image_type_combo), 0);
-		}
-		else {
-			gtk_combo_box_append_text (GTK_COMBO_BOX (selection->priv->image_type_combo),
-						   _("Raw image (only with CDS)"));
-			gtk_combo_box_append_text (GTK_COMBO_BOX (selection->priv->image_type_combo),
-						   _("Cue image (only with CDs)"));
-		}
+		brasero_image_type_chooser_set_source (BRASERO_IMAGE_TYPE_CHOOSER (selection->priv->image_type_widget),
+						       selection->priv->drive,
+						       source->type,
+						       selection->priv->image_format);
 	}
 }
 
@@ -731,6 +797,8 @@ brasero_recorder_selection_drive_properties (BraseroRecorderSelection *selection
 	NautilusBurnMediaType media;
 	BraseroDriveProp *prop;
 	BraseroBurnFlag flags = 0;
+	BraseroBurnFlag supported = 0;
+	BraseroBurnFlag compulsory = 0;
 	GtkWidget *combo;
 	GtkWindow *toplevel;
 	GtkWidget *toggle_otf = NULL;
@@ -739,9 +807,9 @@ brasero_recorder_selection_drive_properties (BraseroRecorderSelection *selection
 	GtkWidget *toggle_burnproof = NULL;
 	GtkWidget *dialog;
 	gchar *header, *text;
-	gchar *drive_name;
+	gchar *display_name;
 	gint result, i;
-	gint speed;
+	gint max_rate, max_speed;
 	GSList *list = NULL;
 
 	/* */
@@ -749,8 +817,8 @@ brasero_recorder_selection_drive_properties (BraseroRecorderSelection *selection
 	drive = selection->priv->drive;
 
 	/* dialog */
-	drive_name = nautilus_burn_drive_get_name_for_display (drive);
-	header = g_strdup_printf (_("Properties of %s"), drive_name);
+	display_name = nautilus_burn_drive_get_name_for_display (drive);
+	header = g_strdup_printf (_("Properties of %s"), display_name);
 
 	/* search for the main window */
 	toplevel = GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (selection)));
@@ -767,23 +835,20 @@ brasero_recorder_selection_drive_properties (BraseroRecorderSelection *selection
 
 	/* Speed combo */
 	media = nautilus_burn_drive_get_media_type (drive);
-	speed = nautilus_burn_drive_get_max_speed_write (drive);
+	max_rate = nautilus_burn_drive_get_max_speed_write (drive);
 
 	combo = gtk_combo_box_new_text ();
 	gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Max speed"));
+	
+	if (NAUTILUS_BURN_DRIVE_MEDIA_TYPE_IS_DVD (media))
+		max_speed = NAUTILUS_BURN_DRIVE_DVD_SPEED (max_rate);
+	else
+		max_speed = NAUTILUS_BURN_DRIVE_CD_SPEED (max_rate);
 
-	/* FIXME : we should use nautilus_burn_drive_get_speeds in the future
-	 * when it actually works */
-	if (media > NAUTILUS_BURN_MEDIA_TYPE_CDRW)
-		for (i = 2; i < speed; i += 2) {
-		/* FIXME : this has changed in 2.15/16 speeds are given in bytes */
-		text = g_strdup_printf ("%i x (DVD)", i);
-		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), text);
-		g_free (text);
-	}
-	else for (i = 2; i < speed;i += 2) {
-		/* FIXME : this has changed in 2.15/16 speeds are given in bytes */
-		text = g_strdup_printf ("%i x (CD)", i);
+	for (i = 2; i <= max_speed; i += 2) {
+		text = g_strdup_printf ("%i x (%s)",
+					i,
+					NAUTILUS_BURN_DRIVE_MEDIA_TYPE_IS_DVD (media) ? _("DVD"):_("CD"));
 		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), text);
 		g_free (text);
 	}
@@ -794,30 +859,33 @@ brasero_recorder_selection_drive_properties (BraseroRecorderSelection *selection
 			    FALSE, FALSE, 0);
 
 	prop = g_hash_table_lookup (selection->priv->settings,
-				    drive_name); /* FIXME what about drives with the same display names */
+				    display_name); /* FIXME what about drives with the same display names */
 
 	if (!prop) {
 		prop = g_new0 (BraseroDriveProp, 1);
 		brasero_recorder_selection_set_drive_default_properties (selection, prop);
 		g_hash_table_insert (selection->priv->settings,
-				     g_strdup (drive_name),
+				     display_name,
 				     prop);
 	}
+	else
+		g_free (display_name);
 
 	if (prop->props.drive_speed == 0
 	||  prop->props.drive_speed >= nautilus_burn_drive_get_max_speed_write (drive))
 		gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 0);
 	else
-		gtk_combo_box_set_active (GTK_COMBO_BOX (combo),
-					  prop->props.drive_speed / 2);
+		gtk_combo_box_set_active (GTK_COMBO_BOX (combo), prop->props.drive_speed / 2);
 
 	/* properties */
-	brasero_burn_caps_get_supported_flags (selection->priv->caps,
-					       selection->priv->track_source,
-					       selection->priv->drive,
-					       &flags);
+	brasero_burn_caps_get_flags (selection->priv->caps,
+				     selection->priv->track_source,
+				     selection->priv->drive,
+				     NULL,
+				     &compulsory,
+				     &supported);
 
-	if (flags & BRASERO_BURN_FLAG_DUMMY) {
+	if (supported & BRASERO_BURN_FLAG_DUMMY) {
 		toggle_simulation = gtk_check_button_new_with_label (_("Simulate the burning"));
 		if (prop->flags & BRASERO_BURN_FLAG_DUMMY)
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle_simulation), TRUE);
@@ -825,7 +893,7 @@ brasero_recorder_selection_drive_properties (BraseroRecorderSelection *selection
 		list = g_slist_prepend (list, toggle_simulation);
 	}
 
-	if (flags & BRASERO_BURN_FLAG_EJECT) {
+	if (supported & BRASERO_BURN_FLAG_EJECT) {
 		toggle_eject = gtk_check_button_new_with_label (_("Eject after burning"));
 		if (prop->flags & BRASERO_BURN_FLAG_EJECT)
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle_eject), TRUE);
@@ -833,7 +901,7 @@ brasero_recorder_selection_drive_properties (BraseroRecorderSelection *selection
 		list = g_slist_prepend (list, toggle_eject);
 	}
 
-	if (flags & BRASERO_BURN_FLAG_BURNPROOF) {
+	if (supported & BRASERO_BURN_FLAG_BURNPROOF) {
 		toggle_burnproof = gtk_check_button_new_with_label (_("Use burnproof (decrease the risk of failures)"));
 		if (prop->flags & BRASERO_BURN_FLAG_BURNPROOF)
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle_burnproof), TRUE);
@@ -841,14 +909,12 @@ brasero_recorder_selection_drive_properties (BraseroRecorderSelection *selection
 		list = g_slist_prepend (list, toggle_burnproof);
 	}
 
-	if (flags & BRASERO_BURN_FLAG_ON_THE_FLY) {
+	if (supported & BRASERO_BURN_FLAG_ON_THE_FLY) {
 		toggle_otf = gtk_check_button_new_with_label (_("Burn the image directly without saving it to disc"));
 		if (prop->flags & BRASERO_BURN_FLAG_ON_THE_FLY)
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle_otf), TRUE);
 
-		if (media > NAUTILUS_BURN_MEDIA_TYPE_CDRW
-		&&  selection->priv->track_source
-		&&  selection->priv->track_source->type != BRASERO_TRACK_SOURCE_DISC)
+		if (compulsory & BRASERO_BURN_FLAG_ON_THE_FLY)
 			gtk_widget_set_sensitive (toggle_otf, FALSE);
 
 		list = g_slist_prepend (list, toggle_otf);
@@ -873,8 +939,12 @@ brasero_recorder_selection_drive_properties (BraseroRecorderSelection *selection
 	}
 
 	prop->props.drive_speed = gtk_combo_box_get_active (GTK_COMBO_BOX (combo));
-	if (prop->props.drive_speed == 0)
-		prop->props.drive_speed = nautilus_burn_drive_get_max_speed_write (drive);
+	if (prop->props.drive_speed == 0) {
+		if (NAUTILUS_BURN_DRIVE_MEDIA_TYPE_IS_DVD (media))
+			prop->props.drive_speed = NAUTILUS_BURN_DRIVE_DVD_SPEED (max_rate);
+		else
+			prop->props.drive_speed = NAUTILUS_BURN_DRIVE_CD_SPEED (max_rate);
+	}
 	else
 		prop->props.drive_speed = prop->props.drive_speed * 2;
 
@@ -908,7 +978,6 @@ brasero_recorder_selection_drive_properties (BraseroRecorderSelection *selection
 
 	gtk_widget_destroy (dialog);
 	nautilus_burn_drive_unref (drive);
-	g_free (drive_name);
 }
 
 static void
@@ -916,7 +985,7 @@ brasero_recorder_selection_image_properties (BraseroRecorderSelection *selection
 {
 	GtkWindow *toplevel;
 	GtkWidget *dialog;
-	int answer;
+	gint answer;
 
 	toplevel = GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (selection)));
 	dialog = gtk_file_chooser_dialog_new (_("Choose a location for the disc image"),
@@ -930,7 +999,7 @@ brasero_recorder_selection_image_properties (BraseroRecorderSelection *selection
 	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (dialog), TRUE);
 
 	if (selection->priv->image_path) {
-		char *name;
+		gchar *name;
 
 		gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (dialog),
 					       selection->priv->image_path);
@@ -941,8 +1010,9 @@ brasero_recorder_selection_image_properties (BraseroRecorderSelection *selection
 		if (!name) {
 			name = g_path_get_basename (selection->priv->image_path);
 			gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), name);
-			g_free (name);
 		}
+
+		g_free (name);
 	}
 	else
 		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog),
@@ -966,17 +1036,12 @@ brasero_recorder_selection_image_properties (BraseroRecorderSelection *selection
 static void
 brasero_recorder_selection_disc_image_properties (BraseroRecorderSelection *selection)
 {
-	BraseroTrackSourceType *targets;
-	BraseroTrackSourceType *iter;
-	GtkWidget *type_combo;
+	GtkWidget *format_chooser;
 	GtkWindow *toplevel;
 	GtkWidget *chooser;
 	GtkWidget *dialog;
-	GtkWidget *label;
-	GtkWidget *hbox;
 	GtkWidget *vbox;
 	gint answer;
-	gint type;
 
 	toplevel = GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (selection)));
 	dialog = gtk_dialog_new_with_buttons (_("Disc image file properties"),
@@ -1001,7 +1066,7 @@ brasero_recorder_selection_disc_image_properties (BraseroRecorderSelection *sele
 
 	/* we reset the previous settings */
 	if (selection->priv->image_path) {
-		char *name;
+		gchar *name;
 
 		gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (chooser),
 					       selection->priv->image_path);
@@ -1012,8 +1077,8 @@ brasero_recorder_selection_disc_image_properties (BraseroRecorderSelection *sele
 		if (!name) {
 			name = g_path_get_basename (selection->priv->image_path);
 			gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (chooser), name);
-			g_free (name);
 		}
+	    	g_free (name);
 	}
 	else
 		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser),
@@ -1021,66 +1086,21 @@ brasero_recorder_selection_disc_image_properties (BraseroRecorderSelection *sele
 
 	gtk_box_pack_start (GTK_BOX (vbox), chooser, TRUE, TRUE, 0);
 
-	hbox = gtk_hbox_new (FALSE, 6);
-	gtk_widget_show (hbox);
-	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+	format_chooser = brasero_image_type_chooser_new ();
+	gtk_box_pack_start (GTK_BOX (vbox), format_chooser, FALSE, FALSE, 0);
+	gtk_widget_show (format_chooser);
 
-	label = gtk_label_new (_("Image type: "));
-	gtk_widget_show (label);
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-
-	type_combo = gtk_combo_box_new_text ();
-	gtk_widget_show (type_combo);
-	gtk_box_pack_end (GTK_BOX (hbox), type_combo, TRUE, TRUE, 0);
-
-	/* now we get the targets available and display them */
-	gtk_combo_box_append_text (GTK_COMBO_BOX (type_combo), _("Let brasero choose (safest)"));
-	brasero_burn_caps_get_imager_available_targets (selection->priv->caps,
-							&targets,
-							selection->priv->track_source);
-
-	for (iter = targets; iter [0] != BRASERO_TRACK_SOURCE_UNKNOWN; iter ++) {
-		switch (iter [0]) {
-		case BRASERO_TRACK_SOURCE_ISO:
-			gtk_combo_box_append_text (GTK_COMBO_BOX (type_combo), _("*.iso image"));
-			break; 
-		case BRASERO_TRACK_SOURCE_RAW:
-			gtk_combo_box_append_text (GTK_COMBO_BOX (type_combo), _("*.raw image"));
-			break;
-		case BRASERO_TRACK_SOURCE_CUE:
-			gtk_combo_box_append_text (GTK_COMBO_BOX (type_combo), _("*.cue image"));
-			break;
-		default:
-			break;
-		}
-	}
-
-	if (selection->priv->image_type != BRASERO_TRACK_SOURCE_DEFAULT) {
-		gint i;
-
-		/* we find the number of the target if it is still available */
-		for (i = 0; targets [i] != BRASERO_TRACK_SOURCE_UNKNOWN; i++) {
-			if (targets [i] == selection->priv->image_type) {
-				gtk_combo_box_set_active (GTK_COMBO_BOX (type_combo), i);
-				break;
-			}
-		}
-	}
-	else
-		gtk_combo_box_set_active (GTK_COMBO_BOX (type_combo), 0);
-
-	/* just to make sure we see if there is a line which is active. It can 
-	 * happens that the last time it was a CD and the user chose RAW. If it
-	 * is now a DVD it can't be raw any more */
-	if (gtk_combo_box_get_active (GTK_COMBO_BOX (type_combo)) == -1)
-		gtk_combo_box_set_active (GTK_COMBO_BOX (type_combo), 0);
+	brasero_image_type_chooser_set_source (BRASERO_IMAGE_TYPE_CHOOSER (format_chooser),
+					       selection->priv->drive,
+					       selection->priv->track_source->type,
+					       selection->priv->image_format);
 
 	/* and here we go */
 	gtk_widget_show (dialog);
 
-	selection->priv->image_type_combo = type_combo;
+	selection->priv->image_type_widget = format_chooser;
 	answer = gtk_dialog_run (GTK_DIALOG (dialog));
-	selection->priv->image_type_combo = NULL;
+	selection->priv->image_type_widget = NULL;
 
 	if (answer != GTK_RESPONSE_OK)
 		goto end;
@@ -1090,15 +1110,11 @@ brasero_recorder_selection_disc_image_properties (BraseroRecorderSelection *sele
 
 	selection->priv->image_path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
 
-	type = gtk_combo_box_get_active (GTK_COMBO_BOX (type_combo));
-	if (type == 0) 
-		selection->priv->image_type = BRASERO_TRACK_SOURCE_DEFAULT;
-	else
-		selection->priv->image_type = targets [type - 1];
+	brasero_image_type_chooser_get_format (BRASERO_IMAGE_TYPE_CHOOSER (format_chooser),
+					       &selection->priv->image_format);
 
 end:
 	gtk_widget_destroy (dialog);
-	g_free (targets);
 }
 
 static void
@@ -1124,8 +1140,8 @@ brasero_recorder_selection_get_drive (BraseroRecorderSelection *selection,
 				      NautilusBurnDrive **drive,
 				      BraseroDriveProp *props)
 {
-	BraseroDriveProp *setting;
-	gchar *device_name;
+	BraseroDriveProp *setting = { 0, };
+	gchar *display_name;
 
 	g_return_if_fail (drive != NULL);
 
@@ -1149,10 +1165,10 @@ brasero_recorder_selection_get_drive (BraseroRecorderSelection *selection,
 		return;
 	}
 
-	device_name = nautilus_burn_drive_get_name_for_display (selection->priv->drive);
+	display_name = nautilus_burn_drive_get_name_for_display (selection->priv->drive);
 	setting = g_hash_table_lookup (selection->priv->settings,
-				       device_name);
-	g_free (device_name);
+				       display_name);
+	g_free (display_name);
 
 	if (!setting) {
 		brasero_recorder_selection_set_drive_default_properties (selection, props);
@@ -1163,6 +1179,27 @@ brasero_recorder_selection_get_drive (BraseroRecorderSelection *selection,
 	props->flags = setting->flags;
 }
 
+void
+brasero_recorder_selection_get_media (BraseroRecorderSelection *selection,
+				      NautilusBurnMediaType *media)
+{
+	if (!media)
+		return;
+
+	if (selection->priv->drive)
+		*media = nautilus_burn_drive_get_media_type (selection->priv->drive);
+	else
+		*media = NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN;
+}
+
+void
+brasero_recorder_selection_set_drive (BraseroRecorderSelection *selection,
+				      NautilusBurnDrive *drive)
+{
+	nautilus_burn_drive_selection_set_active (NAUTILUS_BURN_DRIVE_SELECTION (selection->priv->selection),
+						  drive);
+}
+
 void
 brasero_recorder_selection_select_default_drive (BraseroRecorderSelection *selection,
 						 BraseroMediaType type)
@@ -1189,17 +1226,18 @@ brasero_recorder_selection_select_default_drive (BraseroRecorderSelection *selec
 		      NULL);
 
 	NCB_DRIVE_GET_LIST (drives, recorders, image);
+
 	for (iter = drives; iter; iter = iter->next) {
 		drive = iter->data;
 
 		if (!drive || NCB_DRIVE_GET_TYPE (drive) == NAUTILUS_BURN_DRIVE_TYPE_FILE)
 			continue;
 
-		media_type = nautilus_burn_drive_get_media_type_full (drive,
-								      &is_rewritable,
-								      &is_blank,
-								      &has_data,
-								      &has_audio);
+		media_type = NCB_DRIVE_MEDIA_GET_TYPE (drive,
+						       &is_rewritable,
+						       &is_blank,
+						       &has_data,
+						       &has_audio);
 
 		if ((type & BRASERO_MEDIA_WRITABLE) &&  nautilus_burn_drive_media_type_is_writable (media_type, is_blank)) {
 			/* the perfect candidate would be blank; if not keep for later and see if no better media comes up */
