@@ -291,7 +291,7 @@ brasero_search_entry_separator_func (GtkTreeModel *model,
 				     GtkTreeIter *iter,
 				     gpointer data)
 {
-	char *display;
+	gchar *display;
 	gboolean result;
 
 	gtk_tree_model_get (model, iter,
@@ -461,6 +461,7 @@ brasero_search_entry_init (BraseroSearchEntry *obj)
 
 	/* add tooltips */
 	obj->priv->tooltip = gtk_tooltips_new ();
+	g_object_ref_sink (obj->priv->tooltip);
 
 	gtk_tooltips_set_tip (obj->priv->tooltip, 
 			      GTK_BIN (obj->priv->combo)->child,
@@ -557,7 +558,7 @@ brasero_search_entry_finalize (GObject *object)
 	}
 
 	if (cobj->priv->tooltip) {
-		gtk_object_sink (GTK_OBJECT (cobj->priv->tooltip));
+		g_object_unref (cobj->priv->tooltip);
 		cobj->priv->tooltip = NULL;
 	}
 
@@ -581,19 +582,10 @@ static void
 brasero_search_entry_category_clicked_cb (GtkWidget *button,
 					  BraseroSearchEntry *entry)
 {
-/*	const char *keywords; */
-
 	if (entry->priv->keywords) {
 		g_free (entry->priv->keywords);
 		entry->priv->keywords = NULL;
 	}
-/*
-	keywords = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (entry->priv->combo)->child));
-	if (keywords && *keywords != '\0')
-		gtk_widget_set_sensitive (entry->priv->button, TRUE);
-	else
-		gtk_widget_set_sensitive (entry->priv->button, FALSE);
-*/
 }
 
 static void
@@ -642,10 +634,6 @@ static void
 brasero_search_entry_entry_activated_cb (GtkComboBox *combo,
 					 BraseroSearchEntry *entry)
 {
-//	const char *keywords;
-
-//	keywords = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (entry->priv->combo)->child));
-
 	if (gtk_combo_box_get_active (GTK_COMBO_BOX (entry->priv->combo)) != -1)
 		brasero_search_entry_activated (entry, TRUE);
 	else
@@ -843,9 +831,9 @@ brasero_search_entry_add_current_keyword_to_history (BraseroSearchEntry *entry)
 static void
 _add_mime_types_to_query (BeagleQuery *query, const MimeTypeGroup *group)
 {
-	char **mime;
+	gchar **mime;
 
-	mime = group->mimetypes;
+	mime = (gchar **) group->mimetypes;
 	while (*mime) {
 		beagle_query_add_mime_type (query, *mime);
 		mime ++;
