@@ -65,11 +65,15 @@
 #include "search-entry.h"
 #include "mime-filter.h"
 #include "search.h"
+#include "eggtreemultidnd.h"
+
 #include "brasero-uri-container.h"
+#include "brasero-layout-object.h"
 
 static void brasero_search_class_init (BraseroSearchClass *klass);
 static void brasero_search_init (BraseroSearch *sp);
 static void brasero_search_iface_uri_container_init (BraseroURIContainerIFace *iface);
+static void brasero_search_iface_layout_object_init (BraseroLayoutObjectIFace *iface);
 static void brasero_search_finalize (GObject *object);
 static void brasero_search_destroy (GtkObject *object);
 
@@ -80,6 +84,7 @@ struct BraseroSearchPrivate {
 	GtkWidget *tree;
 	GtkWidget *entry;
 	GtkWidget *filter;
+	GtkWidget *filters;
 	GtkWidget *right;
 	GtkWidget *left;
 	GtkWidget *results_label;
@@ -117,13 +122,13 @@ enum {
 
 
 static void brasero_search_entry_activated_cb (BraseroSearchEntry *entry,
-					        BraseroSearch *obj);
+					       BraseroSearch *obj);
 static void brasero_search_tree_activated_cb (GtkTreeView *tree,
-					       GtkTreeIter *row,
-					       GtkTreeViewColumn *column,
-					       BraseroSearch *search);
+					      GtkTreeIter *row,
+					      GtkTreeViewColumn *column,
+					      BraseroSearch *search);
 static void brasero_search_tree_selection_changed_cb (GtkTreeSelection *selection,
-						       BraseroSearch *search);
+						      BraseroSearch *search);
 static void brasero_search_drag_data_get_cb (GtkTreeView *tree,
 					     GdkDragContext *drag_context,
 					     GtkSelectionData *selection_data,
@@ -147,13 +152,21 @@ brasero_search_max_results_num_changed_cb (GtkComboBox *combo,
 					   BraseroSearch *search);
 
 static void brasero_search_empty_tree (BraseroSearch *search);
-static char ** brasero_search_get_selected_rows (BraseroSearch *search);
+static gchar ** brasero_search_get_selected_rows (BraseroSearch *search);
 
-static char **
+static gchar **
 brasero_search_get_selected_uris (BraseroURIContainer *container);
-static char *
+static gchar *
 brasero_search_get_selected_uri (BraseroURIContainer *container);
 
+static void
+brasero_search_get_proportion (BraseroLayoutObject *object,
+			       gint *header,
+			       gint *center,
+			       gint *footer);
+
+#define BRASERO_SEARCH_SPACING 6
+
 GType
 brasero_search_get_type ()
 {
@@ -178,6 +191,12 @@ brasero_search_get_type ()
 			NULL,
 			NULL
 		};
+		static const GInterfaceInfo layout_object_info =
+		{
+			(GInterfaceInitFunc) brasero_search_iface_layout_object_init,
+			NULL,
+			NULL
+		};
 
 		type = g_type_register_static (GTK_TYPE_VBOX,
 					       "BraseroSearch",
@@ -187,6 +206,9 @@ brasero_search_get_type ()
 		g_type_add_interface_static (type,
 					     BRASERO_TYPE_URI_CONTAINER,
 					     &uri_container_info);
+		g_type_add_interface_static (type,
+					     BRASERO_TYPE_LAYOUT_OBJECT,
+					     &layout_object_info);
 	}
 
 	return type;
@@ -210,6 +232,25 @@ brasero_search_iface_uri_container_init (BraseroURIContainerIFace *iface)
 	iface->get_selected_uris = brasero_search_get_selected_uris;
 }
 
+static void
+brasero_search_iface_layout_object_init (BraseroLayoutObjectIFace *iface)
+{
+	iface->get_proportion = brasero_search_get_proportion;
+}
+
+static void
+brasero_search_get_proportion (BraseroLayoutObject *object,
+			       gint *header,
+			       gint *center,
+			       gint *footer) 
+{
+	GtkRequisition requisition;
+
+	gtk_widget_size_request (BRASERO_SEARCH (object)->priv->filters,
+				 &requisition);
+	*footer = requisition.height + BRASERO_SEARCH_SPACING;
+}
+
 static void
 brasero_search_column_clicked (GtkTreeViewColumn *column,
 			       BraseroSearch *search)
@@ -267,9 +308,13 @@ brasero_search_init (BraseroSearch *obj)
 	GtkFileFilter *file_filter;
 	GtkTreeSelection *selection;
 
-	gtk_box_set_spacing (GTK_BOX (obj), 8);
+	gtk_box_set_spacing (GTK_BOX (obj), BRASERO_SEARCH_SPACING);
 	obj->priv = g_new0 (BraseroSearchPrivate, 1);
 
+	/* separator */
+	separator = gtk_hseparator_new ();
+	gtk_box_pack_start (GTK_BOX (obj), separator, FALSE, FALSE, 0);
+
 	/* Entry */
 	obj->priv->entry = brasero_search_entry_new ();
 	g_signal_connect (G_OBJECT (obj->priv->entry), 
@@ -286,7 +331,10 @@ brasero_search_init (BraseroSearch *obj)
 	box = gtk_hbox_new (FALSE, 6);
 	gtk_box_pack_start (GTK_BOX (obj), box, FALSE, FALSE, 0);
 
-	button = brasero_utils_make_button (_("Previous results"), GTK_STOCK_GO_BACK);
+	button = brasero_utils_make_button (_("Previous results"),
+					    GTK_STOCK_GO_BACK,
+					    NULL,
+					    GTK_ICON_SIZE_BUTTON);
 	gtk_button_set_alignment (GTK_BUTTON (button), 0.0, 0.5);
 	gtk_widget_set_sensitive (button, FALSE);
 	g_signal_connect (G_OBJECT (button), 
@@ -301,7 +349,10 @@ brasero_search_init (BraseroSearch *obj)
 	gtk_box_pack_start (GTK_BOX (box), label, TRUE, TRUE, 0);
 	obj->priv->results_label = label;
 
-	button = brasero_utils_make_button (_("Next results"), GTK_STOCK_GO_FORWARD);
+	button = brasero_utils_make_button (_("Next results"),
+					    GTK_STOCK_GO_FORWARD,
+					    NULL,
+					    GTK_ICON_SIZE_BUTTON);
 	gtk_button_set_alignment (GTK_BUTTON (button), 1.0, 0.5);
 	gtk_widget_set_sensitive (button, FALSE);
 	g_signal_connect (G_OBJECT (button), 
@@ -313,6 +364,8 @@ brasero_search_init (BraseroSearch *obj)
 
 	/* Tree */
 	obj->priv->tree = gtk_tree_view_new ();
+	gtk_tree_view_set_rubber_banding (GTK_TREE_VIEW (obj->priv->tree), TRUE);
+	egg_tree_multi_drag_add_drag_support (GTK_TREE_VIEW (obj->priv->tree));
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (obj->priv->tree));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
@@ -414,6 +467,7 @@ brasero_search_init (BraseroSearch *obj)
 
 	/* filter combo */
 	box = gtk_hbox_new (FALSE, 32);
+	obj->priv->filters = box;
 	gtk_box_pack_end (GTK_BOX (obj), box, FALSE, FALSE, 0);
 
 	obj->priv->filter = brasero_mime_filter_new ();
@@ -427,6 +481,7 @@ brasero_search_init (BraseroSearch *obj)
 	gtk_file_filter_add_pattern (file_filter, "*");
 	brasero_mime_filter_add_filter (BRASERO_MIME_FILTER (obj->priv->filter),
 					file_filter);
+	g_object_unref (file_filter);
 
 	gtk_box_pack_end (GTK_BOX (box), obj->priv->filter, FALSE, FALSE, 0);
 
@@ -574,8 +629,8 @@ static char *
 brasero_search_get_selected_uri (BraseroURIContainer *container)
 {
 	BraseroSearch *search;
-	char **uris = NULL;
-	char *uri;
+	gchar **uris = NULL;
+	gchar *uri;
 
 	search = BRASERO_SEARCH (container);
 	uris = brasero_search_get_selected_rows (search);
@@ -594,7 +649,7 @@ brasero_search_empty_tree (BraseroSearch *search)
 {
 	GtkTreeModel *model;
 	GtkTreeIter row;
-	char *mime;
+	gchar *mime;
 
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (search->priv->tree));
 	model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (model));
@@ -617,7 +672,7 @@ brasero_search_drag_data_get_cb (GtkTreeView *tree,
 				 guint info,
 				 guint time, BraseroSearch *search)
 {
-	char **uris;
+	gchar **uris;
 
 	uris = brasero_search_get_selected_rows (search);
 	gtk_selection_data_set_uris (selection_data, uris);
@@ -636,51 +691,30 @@ brasero_search_add_hit_to_tree (BraseroSearch *search,
 	GSList *iter;
 	GSList *next;
 
-	const char *description;
-	char *name, *mime, *uri;
-	int score;
-	int num;
+	const gchar *description;
+	gchar *name, *mime, *uri;
+	gint score;
+	gint num;
 
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (search->priv->tree));
 	model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (model));
 
 	num = 0;
 	for (iter = list; iter && num < max; iter = next, num ++) {
-		char *unescaped_uri;
+		gchar *unescaped_uri;
 
 		hit = iter->data;
 		next = iter->next;
 
 		uri = g_strdup (beagle_hit_get_uri (hit));
-	
-		/* FIXME : beagle return badly formed uri not encoded in
-		 * utf8 / locale charset so we check them just in case */
+
+		/* beagle return badly formed uri not encoded in UTF-8
+		 * locale charset so we check them just in case */
 		unescaped_uri = gnome_vfs_unescape_string_for_display (uri);
 		if (!g_utf8_validate (unescaped_uri, -1, NULL)) {
-			char *tmp;
-			const char *charset;
-			GError *error = NULL;
-
 			g_free (unescaped_uri);
 			g_free (uri);
 			continue;
-
-			g_get_charset (&charset);
-			g_print ("converting to Charset %s\n", charset);
-			tmp = g_convert (unescaped_uri,
-					 -1,
-					 "UTF8",
-					 "ISO-8859-1",
-					 NULL,
-					 NULL,
-					 &error);
-
-			if (error)
-				g_warning ("PROBLEM %s for %s\n", error->message, unescaped_uri);
-
-			g_free (unescaped_uri);
-			unescaped_uri = tmp;
-			uri = gnome_vfs_escape_host_and_path_string (unescaped_uri);
 		}
 
 		name = g_path_get_basename (unescaped_uri);
@@ -731,7 +765,7 @@ brasero_search_add_hit_to_tree (BraseroSearch *search,
 static void
 brasero_search_update_header (BraseroSearch *search)
 {
-	char *string;
+	gchar *string;
 
 	if (search->priv->hits_num) {
 		gint last;
@@ -909,9 +943,9 @@ brasero_search_beagle_hit_substracted_cb (BeagleQuery *query,
 					  BeagleHitsSubtractedResponse *response,
 					  BraseroSearch *search)
 {
-	char *uri;
+	gchar *uri;
 	GSList *list, *iter;
-	const char *removed_uri;
+	const gchar *removed_uri;
 
 	GtkTreeModel *model;
 	GtkTreeIter row;
@@ -1127,18 +1161,14 @@ brasero_search_get_selected_rows (BraseroSearch *search)
 	GtkTreeModel *model;
 	GtkTreeIter row;
 	GList *rows, *iter;
-	char **uris = NULL, *uri;
-	int i;
+	gchar **uris = NULL, *uri;
+	gint i;
 
-	model = gtk_tree_view_get_model (GTK_TREE_VIEW (search->priv->tree));
-	model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (model));
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (search->priv->tree));
-
 	rows = gtk_tree_selection_get_selected_rows (selection, &model);
 	if (rows == NULL)
 		return NULL;
 
-	rows = g_list_reverse (rows);
 	uris = g_new0 (char *, g_list_length (rows) + 1);
 	for (iter = rows, i = 0; iter != NULL; iter = iter->next, i++) {
 		gtk_tree_model_get_iter (model,
